/*  
 *  snake3d, a snake game in a 3d world.
 *  Copyright (C) 2005-2006 Yair Yarom
 *  
 *  This program is free software; you can redistribute it and/or modify it under
 *  the terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or (at your option) any later
 *  version.
 *  
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 *  PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 *  Place, Suite 330, Boston, MA 02111-1307 USA
 *  
 *  http://sourceforge.net/projects/worms3d
 *  
 */

// #include "includes.hpp"

#include <stdlib.h>
#include <cmath>

#include "snake.hpp"
#include "board.hpp"
#include "global.hpp"
#include "functions.hpp"
#include "preferences.hpp"

Board::Board() {
    //_numOfSnakes = 0;
    _obstaclesRotationAngle = 0;
    //_snakes = new Snake*[0];
    for (int i = 0; i < MAX_SNAKES; i++)
	_snakes[i] = NULL;
}

Snake* Board::addSnake(int color, int location) {
    if (_snakes[location] || location >= MAX_SNAKES || location < 0)
	return NULL;
    //Snake** oldSnakes = _snakes;
    //_snakes = new Snake*[_numOfSnakes + 1];
    //for (int i = 0; i < _numOfSnakes; i++)
    //_snakes[i] = oldSnakes[i];
    _snakes[location] = new Snake(_startingLocations[location][X],
				  _startingLocations[location][Y],
				  _startingLocations[location][Z],
				  _startingDirections[location],
				  _startingUps[location],
				  color, location);
    //delete [] oldSnakes;
    replaceFood();
    return _snakes[location];
}

float Board::getMaxSize() {
    return _maxSize;
}

const int* const Board::getFood() {
    return _food;
}

void Board::replaceFood() {
    do {
	_food[X] = (int)floor((double)random() / RAND_MAX * _maxSize);
	_food[Y] = (int)floor((double)random() / RAND_MAX * _maxSize);
	_food[Z] = (int)floor((double)random() / RAND_MAX * _maxSize);
    } while (checkLocation(_food, NULL) != o_food);
    if (networkState == net_server && p_mode == pm_game)
	sendFood();
}

void Board::replaceFood(const int* food) {
    if (networkState == net_client) {
	_food[X] = food[X];
	_food[Y] = food[Y];
	_food[Z] = food[Z];
	if (checkLocation(_food, NULL) != o_food)
	    networkError = true;
    }
}

occupancy_t Board::checkLocation(int* location, Snake* snake) {
    for (int i = 0; i < MAX_SNAKES; i++)
	if (_snakes[i]) {
	    occupancy_t check = _snakes[i]->collision(location, _snakes[i] == snake);
	    if (check != o_empty)
		return check;
	}
    if (location[X] == _food[X] &&
	location[Y] == _food[Y] &&
	location[Z] == _food[Z])
	return o_food;
    return o_empty;
}

void Board::step() {
    
    // doesn't work under network
    if (networkState == net_none) {
	for (int i = 0; i < MAX_SNAKES; i++)
	    if (_snakes[i])
		_snakes[i]->changeDirection();
    }
    
    for (int i = 0; i < MAX_SNAKES; i++) {
	if (_snakes[i] && !_snakes[i]->isDead())
	    _snakes[i]->move();
    }
    for (int i = 0; i < MAX_SNAKES; i++) {
	if (_snakes[i] && !_snakes[i]->isDead()) {
	    int* head = _snakes[i]->_joints[0];
	    occupancy_t occupancy = checkLocation(head, _snakes[i]);
	    
	    /*
	    if (occupancy == o_food) {
		if (networkState != net_client) {
		    replaceFood();
		    _snakes[i]->inc();
		}
            } else if (occupancy != o_empty && occupancy != o_intercept) {
		_snakes[i]->kill();
	    }
	    */
	    if (occupancy != o_empty && occupancy != o_intercept && occupancy != o_food)
		_snakes[i]->kill();
	    else if (head[X] == _food[X] && head[Y] == _food[Y] && head[Z] == _food[Z]) {
		if (networkState != net_client) {
		    replaceFood();
		    _snakes[i]->inc();
		}
	    }
	}
    }

    occupancy_t foodO = checkLocation(_food, NULL);
    if (foodO != o_food && foodO != o_intercept)
	replaceFood();
    
    for (int i = 0; i < MAX_SNAKES; i++)
	if (_snakes[i])
	    _snakes[i]->changeDirection();
}

void Board::resetSnakes() {
    timerLong = 0;
    for (int i = 0; i < MAX_SNAKES; i++)
	if (_snakes[i])
	    _snakes[i]->reset();	
    replaceFood();
    for (int i = 0; i < MAX_SNAKES; i++)
	if (_snakes[i])
    	    _snakes[i]->changeDirection();
}

void Board::drawFood() {
    glColor3fv(COLORS[CYAN]);
    glPushMatrix();
    glTranslatef(_food[X] + 0.5, _food[Y] + 0.5, _food[Z] + 0.5);
    if (_obstaclesRotationAngle > 360)
	_obstaclesRotationAngle -= 360;
    glRotatef(_obstaclesRotationAngle, 1, 1, 1);

    glMaterialfv(GL_FRONT, GL_AMBIENT, FOOD_AMBIENT);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, FOOD_DIFFUSE);
    glMaterialfv(GL_FRONT, GL_SPECULAR, FOOD_SPECULAR);
    glMaterialf(GL_FRONT, GL_SHININESS, FOOD_SHININESS);
    glMaterialfv(GL_FRONT, GL_EMISSION, FOOD_EMISSION);
    glScalef(0.5, 0.5, 0.5);
    glutSolidIcosahedron();

    glPopMatrix();
};

void Board::drawSnakes(bool withHead) {
    for (int i = 0; i < MAX_SNAKES; i++)
	if (_snakes[i])
	    //_snakes[i]->draw(withHead && i == playerID);
            _snakes[i]->draw(withHead && i == player->_id);
};

void Board::draw(bool withHead, int grid, bool walls) {
    _obstaclesRotationAngle += 2;
    glDisable(GL_LIGHTING);
    drawFrame();
    drawGrid(grid);
    glEnable(GL_LIGHTING);
    //drawObstacles();
    drawFood();
    drawSnakes(withHead);
    if (walls) drawWalls();
}

Board::~Board() {
    for (int i = 0; i < MAX_SNAKES; i++)
	if (_snakes[i])
	    delete _snakes[i];
    //delete [] _snakes;
    for (int i = 0; i < MAX_SNAKES; i++)
	delete [] _startingLocations[i];
    delete [] _startingLocations;
    delete [] _startingDirections;
    delete [] _startingUps;
}
