/*  
 *  snake3d, a snake game in a 3d world.
 *  Copyright (C) 2005-2006 Yair Yarom
 *  
 *  This program is free software; you can redistribute it and/or modify it under
 *  the terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or (at your option) any later
 *  version.
 *  
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 *  PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 *  Place, Suite 330, Boston, MA 02111-1307 USA
 *  
 *  http://sourceforge.net/projects/worms3d
 *  
 */

#ifndef BOARD_HPP
#define BOARD_HPP

#include "consts.hpp"

class Snake;

class Board {
private:
    int _obstaclesRotationAngle;

    void drawSnakes(bool withHead);
    void drawFood();

protected:
    
    float _maxSize; 
    Snake* _snakes[MAX_SNAKES];
    int _food[3];
    //int _numOfSnakes;
    int** _startingLocations;
    direction* _startingDirections;
    direction* _startingUps;
    //    int _id;
    
    void replaceFood();

    virtual void drawFrame() = 0;
    virtual void drawGrid(int grid) = 0;
    virtual void drawWalls() = 0;
    
public:
    Board();
    Snake* addSnake(int color, int location);
    void step();
    void resetSnakes();
    void draw(bool withHead, int grid, bool walls);
    float getMaxSize();
    const int* const getFood();
    void replaceFood(const int* food);
    
    virtual const char* getName() = 0;
    virtual Board* clone() = 0;
    virtual occupancy_t checkLocation(int* location, Snake* snake);
    virtual ~Board();
};

#endif
