/*  
 *  snake3d, a snake game in a 3d world.
 *  Copyright (C) 2005-2006 Yair Yarom
 *  
 *  This program is free software; you can redistribute it and/or modify it under
 *  the terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or (at your option) any later
 *  version.
 *  
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 *  PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 *  Place, Suite 330, Boston, MA 02111-1307 USA
 *  
 *  http://sourceforge.net/projects/worms3d
 *  
 */

#include "includes.hpp"
#include "classic2d.hpp"

Classic2d::Classic2d(int size) {
    if (size % 2 == 0)
	size--;
    size = (size < 3 ? 3 : size);
    _maxSize = size;
    _z = size / 2;
    _startingLocations = new int*[MAX_SNAKES];
    for (int i = 0; i < MAX_SNAKES; i++)
        _startingLocations[i] = new int[3];
    _startingDirections = new direction[MAX_SNAKES];
    _startingUps = new direction[MAX_SNAKES];
    
    _startingLocations[0][X] = size / 2 - 1;
    _startingLocations[0][Y] = size / 2;
    _startingLocations[0][Z] = _z;
    _startingDirections[0] = left;
    _startingUps[0] = backward;
	
    _startingLocations[1][X] = size / 2 + 1;
    _startingLocations[1][Y] = size / 2;
    _startingLocations[1][Z] = _z;
    _startingDirections[1] = right;
    _startingUps[1] = backward;

    _startingLocations[2][X] = size / 2;
    _startingLocations[2][Y] = size / 2 - 1;
    _startingLocations[2][Z] = _z;
    _startingDirections[2] = down;
    _startingUps[2] = backward;

    _startingLocations[3][X] = size / 2;
    _startingLocations[3][Y] = size / 2 + 1;
    _startingLocations[3][Z] = _z;
    _startingDirections[3] = up;
    _startingUps[3] = backward;
}

const char* Classic2d::getName() {
    if      (_maxSize == (TINY_BOARD + (TINY_BOARD % 2 - 1)))   return "Classic 2d (tiny)";
    else if (_maxSize == (SMALL_BOARD + (SMALL_BOARD % 2 - 1)))  return "Classic 2d (small)";
    else if (_maxSize == (MEDIUM_BOARD + (MEDIUM_BOARD % 2 - 1))) return "Classic 2d (medium)";
    else if (_maxSize == (LARGE_BOARD + (LARGE_BOARD % 2 - 1)))  return "Classic 2d (large)";
    else if (_maxSize == (HUGE_BOARD + (HUGE_BOARD % 2 - 1)))   return "Classic 2d (huge)";
    return "Classic 2d (unknown)";
};

Board* Classic2d::clone() {
    return new Classic2d((int)_maxSize);
}

occupancy_t Classic2d::checkLocation(int* location, Snake* snake) {
    if (location[X] < 0 || location[X] >= _maxSize ||
        location[Y] < 0 || location[Y] >= _maxSize ||
        location[Z] != _z)
	return o_outside;
    return Board::checkLocation(location, snake);
}

void Classic2d::drawFrame() {
    glColor3fv(COLORS[WHITE]);
    glLineWidth(2);

    glBegin(GL_LINE_STRIP);
    glVertex3d(       0,        0, _z);
    glVertex3d(       0,        0, _z + 1);
    glVertex3d(       0, _maxSize, _z + 1);
    glVertex3d(       0, _maxSize, _z);
    glVertex3d(       0,        0, _z);
    glVertex3d(_maxSize,        0, _z); 
    glVertex3d(_maxSize,        0, _z + 1);
    glVertex3d(_maxSize, _maxSize, _z + 1);
    glVertex3d(_maxSize, _maxSize, _z);
    glVertex3d(_maxSize,        0, _z);  
    glEnd();

    glBegin(GL_LINES);
    glVertex3d(       0,        0, _z + 1);
    glVertex3d(_maxSize,        0, _z + 1);

    glVertex3d(       0, _maxSize, _z + 1);
    glVertex3d(_maxSize, _maxSize, _z + 1);
    
    glVertex3d(       0, _maxSize, _z);
    glVertex3d(_maxSize, _maxSize, _z);
    glEnd();
    glLineWidth(1);
};

void Classic2d::drawGrid(int grid) {
    glColor3fv(COLORS[LIGHT_GRAY]);
    glBegin(GL_LINES);
    switch (grid) {
      case 3:
          for (int j = 1; j < _maxSize; j++)
              for (int i = 1; i < _maxSize; i++) {
                  glVertex3d(i, j, _z);
                  glVertex3d(i, j, _z + 1);
              };
	  
      case 2:
          for (int i = 1; i <_maxSize; i++) {
              glVertex3d(i, _maxSize, _z);
              glVertex3d(i, _maxSize, _z + 1);
              
              glVertex3d(       i,        0, _z + 1);
              glVertex3d(       i, _maxSize, _z + 1);
              glVertex3d(       0,        i, _z + 1);
              glVertex3d(_maxSize,        i, _z + 1);
              
	      glVertex3d(_maxSize, i, _z);
	      glVertex3d(_maxSize, i, _z + 1);
	  }
	  
      case 1:
	  for (int i = 1; i < _maxSize; i++) {
	      // down
	      glVertex3d(i,    0, _z);
	      glVertex3d(i,    0, _z + 1);
	      
	      // back
	      glVertex3d(	i,        0, _z);
	      glVertex3d(	i, _maxSize, _z);
	      glVertex3d(	0,        i, _z);
	      glVertex3d(_maxSize,        i, _z);
	      
	      // left
	      glVertex3d(0, i, _z);
	      glVertex3d(0, i, _z + 1);
	  }

    }       
    glEnd();
};

void Classic2d::drawWalls() {
    glEnable(GL_BLEND);
    glDepthMask(GL_FALSE);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE);
    glMaterialfv(GL_FRONT, GL_AMBIENT, WALL_AMBIENT);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, WALL_DIFFUSE);
    glMaterialfv(GL_FRONT, GL_SPECULAR, WALL_SPECULAR);
    glMaterialf(GL_FRONT, GL_SHININESS, WALL_SHININESS);
    glMaterialfv(GL_FRONT, GL_EMISSION, WALL_EMISSION);
    glColor4fv(WALL_DIFFUSE);
    glPushMatrix();

    glTranslatef(_maxSize / 2, _maxSize / 2, _maxSize / 2);
    glScalef(1.0, 1.0, 1.0 / _maxSize);
    glutSolidCube(_maxSize);

    glPopMatrix();
    glDepthMask(GL_TRUE);
    glDisable(GL_BLEND);
}
