/*  
 *  snake3d, a snake game in a 3d world.
 *  Copyright (C) 2005-2006 Yair Yarom
 *  
 *  This program is free software; you can redistribute it and/or modify it under
 *  the terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or (at your option) any later
 *  version.
 *  
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 *  PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 *  Place, Suite 330, Boston, MA 02111-1307 USA
 *  
 *  http://sourceforge.net/projects/worms3d
 *  
 */

#include <cmath>

#include "consts.hpp"
#include "includes.hpp"

const int X = 0;
const int Y = 1;
const int Z = 2;

const int RED         = 0;
const int GREEN       = 1;
const int BLUE        = 2;
const int CYAN        = 3;
const int YELLOW      = 4;
const int MAGENTA     = 5;
const int BLACK       = 6;
const int WHITE       = 7;
const int BROWN       = 8;
const int ORANGE      = 9;
const int PURPLE      = 10;
const int LIGHT_GRAY  = 11;
const int GRAY        = 12;
const int DARK_GRAY   = 13;
const int DARK_GREEN  = 14;
const int DARK_BLUE   = 15;
const int DARK_RED    = 16;
const int LIGHT_GREEN = 17;

const int NCOLORS = 18;

float COLORS[NCOLORS][3] = {
{ 1.0,  0.0,  0.0},  // RED
{ 0.0,  1.0,  0.0},  // GREEN
{ 0.0,  0.0,  1.0},  // BLUE
{ 0.0,  1.0,  1.0},  // CYAN
{ 1.0,  1.0,  0.0},  // YELLOW
{ 1.0,  0.0,  1.0},  // MAGENTA
{ 0.0,  0.0,  0.0},  // BLACK
{ 1.0,  1.0,  1.0},  // WHITE
{ 0.6,  0.4,  0.2},  // BROWN
{ 1.0,  0.5,  0.0},  // ORANGE
{ 0.5,  0.0,  0.5},  // PURPLE
{0.75, 0.75, 0.75},  // LIGHT_GRAY
{ 0.5,  0.5,  0.5},  // GRAY
{0.25, 0.25, 0.25},  // DARK_GRAY
{ 0.0,  0.5,  0.0},  // DARK_GREEN
{ 0.0,  0.0,  0.5},  // DARK_BLUE
{ 0.5,  0.0,  0.0},  // DARK_RED
{ 0.5,  1.0,  0.5}  // LIGHT_GREEN
};

char* COLORS_NAMES[NCOLORS] = {
    "Red",
    "Green",
    "Blue",
    "Cyan",
    "Yellow",
    "Magenta",
    "Black",
    "White",
    "Brown",
    "Orange",
    "Purple",
    "Light Gray",
    "Gray",
    "Dark Gray",
    "Dark Green",
    "Dark Blue",
    "Dark Red",
    "Light Green"
};


//const int NKEYS = 33;

char* KEY_NAME[NKEYS] = {
    "Left", //k_moveLeft,
    "Down", //k_moveDown,
    "Right", //k_moveRight,
    "Up", //k_moveUp,
    "Forward", //k_moveForward,
    "Backward", //k_moveBackward,
    "Left", //k_selfMoveLeft,
    "Down", //k_selfMoveDown,
    "Right", //k_selfMoveRight,
    "Up", //k_selfMoveUp,
    "Yaw left", //k_selfYawLeft,
    "Yaw right", //k_selfYawRight,
    "Left", //k_boardRotateLeft,
    "Down", //k_boardRotateDown,
    "Right", //k_boardRotateRight,
    "Up", //k_boardRotateUp,
    "Yaw left", //k_boardYawLeft,
    "Yaw right", //k_boardYawRight,
    "Zoom in", //k_zoomIn,
    "Zoom out", //k_zoomOut,
    "Snake view grid level", //k_grid1,
    "Global view grid level", //k_grid3,
    "Auto pilot", //k_autoPilot,
    "Reset board", //k_resetBoard,
    "Reset snakes", //k_resetSnakes,
    "Background color", //k_backgroundColor,
    "Speed up", //k_speedUp,
    "Speed down", //k_speedDown,
    "Show walls", //k_showWalls,
    "Full screen", //k_fullScreen,
    "Change lighting", //k_lighting,
    "Pause", 	       //k_pause,
    "Quit",  	       //k_quit
    "Abort", 	       //k_abort
    "Collision indicator" // k_indicator
};

const bool ALLOWED_KEYS[NKEYS][3] = {
    // net_none, net_server, net_client
    {true , true , true }, //k_moveLeft,
    {true , true , true }, //k_moveDown,
    {true , true , true }, //k_moveRight,
    {true , true , true }, //k_moveUp,
    {true , true , true }, //k_moveForward,
    {true , true , true }, //k_moveBackward,
    {true , true , true }, //k_selfMoveLeft,
    {true , true , true }, //k_selfMoveDown,
    {true , true , true }, //k_selfMoveRight,
    {true , true , true }, //k_selfMoveUp,
    {true , true , true }, //k_selfYawLeft,
    {true , true , true }, //k_selfYawRight,
    {true , true , true }, //k_boardRotateLeft,
    {true , true , true }, //k_boardRotateDown,
    {true , true , true }, //k_boardRotateRight,
    {true , true , true }, //k_boardRotateUp,
    {true , true , true }, //k_boardYawLeft,
    {true , true , true }, //k_boardYawRight,
    {true , true , true }, //k_zoomIn,
    {true , true , true }, //k_zoomOut,
    {true , true , true }, //k_grid1,
    {true , true , true }, //k_grid3,
    {true , true , true }, //k_autoPilot,
    {true , true , true }, //k_resetBoard,
    {true , true , false}, //k_resetSnakes,
    {true , true , true }, //k_backgroundColor,
    {true , true , false}, //k_speedUp,
    {true , true , false}, //k_speedDown,
    {true , true , true }, //k_showWalls,
    {true , true , true }, //k_fullScreen,
    {false, false, false}, //k_lighting,
    {true , true , true }, //k_pause,
    {true , true , true }, //k_quit,
    {true , true , true }, //k_abort,
    {true , true , true }  //k_indicator
};

const keys_t k_move[] = {
    k_moveLeft,
    k_moveDown,
    k_moveRight,
    k_moveUp,
    k_moveForward,
    k_moveBackward
};

const keys_t k_selfMove[] = {
    k_selfMoveLeft,
    k_selfMoveDown,
    k_selfMoveRight,
    k_selfMoveUp
};


const keys_t k_selfYaw[] = {
    k_selfYawLeft,
    k_selfYawRight
};
    
const keys_t k_boardRotate[] = {
    k_boardRotateLeft,
    k_boardRotateDown,
    k_boardRotateRight,
    k_boardRotateUp
};

const keys_t k_boardYaw[] = {
    k_boardYawLeft,
    k_boardYawRight
};


/*
const char DEFAULT_KEYS[NKEYS] = {
    '4',  //  k_moveLeft,
    '2',  //  k_moveDown,		      
    '6',  //  k_moveRight,	      
    '8',  //  k_moveUp,		      
    '9',  //  k_moveForward,	      
    '3',  //  k_moveBackward,	      
    'j',  //  k_selfMoveLeft,	      
    ',',  //  k_selfMoveDown,	      
    'l',  //  k_selfMoveRight,	      
    'i',  //  k_selfMoveUp,	      
    'm',  //  k_selfYawLeft,	      
    '.',  //  k_selfYawRight,	      
    'a',  //  k_boardRotateLeft,	      
    'x',  //  k_boardRotateDown,	      
    'd',  //  k_boardRotateRight,	      
    'w',  //  k_boardRotateUp,	      
    'z',  //  k_boardYawLeft,	      
    'c',  //  k_boardYawRight,	      
    '=',  //  k_zoomIn,		      
    '-',  //  k_zoomOut,		      
    'G',  //  k_grid1,		      
    'g',  //  k_grid3,		      
    'n',  //  k_autoPilot,	      
    'r',  //  k_resetBoard,	      
    'R',  //  k_resetSnakes,	      
    'b',  //  k_backgroundColor,	      
    '>',  //  k_speedUp,		      
    '<',  //  k_speedDown,	      
    'e',  //  k_showWalls,	      
    'f',  //  k_fullScreen,	      
    (char)0,  //  k_lighting,		      
    'p',  //  k_pause,		      
    'q',  //  k_quit,
    'a'   //  k_abort
};
*/

const SDL_keysym DEFAULT_KEYS_SDL[NKEYS] = {
    {(Uint8)NULL, SDLK_KP4     , KMOD_NONE  , (Uint16)NULL},  //  k_moveLeft,
    {(Uint8)NULL, SDLK_KP2     , KMOD_NONE  , (Uint16)NULL},  //  k_moveDown,		      
    {(Uint8)NULL, SDLK_KP6     , KMOD_NONE  , (Uint16)NULL},  //  k_moveRight,	      
    {(Uint8)NULL, SDLK_KP8     , KMOD_NONE  , (Uint16)NULL},  //  k_moveUp,		      
    {(Uint8)NULL, SDLK_KP9     , KMOD_NONE  , (Uint16)NULL},  //  k_moveForward,	      
    {(Uint8)NULL, SDLK_KP3     , KMOD_NONE  , (Uint16)NULL},  //  k_moveBackward,	      
    {(Uint8)NULL, SDLK_j       , KMOD_NONE  , (Uint16)NULL},  //  k_selfMoveLeft,	      
    {(Uint8)NULL, SDLK_COMMA   , KMOD_NONE  , (Uint16)NULL},  //  k_selfMoveDown,	      
    {(Uint8)NULL, SDLK_l       , KMOD_NONE  , (Uint16)NULL},  //  k_selfMoveRight,	      
    {(Uint8)NULL, SDLK_i       , KMOD_NONE  , (Uint16)NULL},  //  k_selfMoveUp,	      
    {(Uint8)NULL, SDLK_m       , KMOD_NONE  , (Uint16)NULL},  //  k_selfYawLeft,	      
    {(Uint8)NULL, SDLK_PERIOD  , KMOD_NONE  , (Uint16)NULL},  //  k_selfYawRight,	      
    {(Uint8)NULL, SDLK_a       , KMOD_NONE  , (Uint16)NULL},  //  k_boardRotateLeft,
    {(Uint8)NULL, SDLK_x       , KMOD_NONE  , (Uint16)NULL},  //  k_boardRotateDown,	      
    {(Uint8)NULL, SDLK_d       , KMOD_NONE  , (Uint16)NULL},  //  k_boardRotateRight,	      
    {(Uint8)NULL, SDLK_w       , KMOD_NONE  , (Uint16)NULL},  //  k_boardRotateUp,
    {(Uint8)NULL, SDLK_z       , KMOD_NONE  , (Uint16)NULL},  //  k_boardYawLeft,
    {(Uint8)NULL, SDLK_c       , KMOD_NONE  , (Uint16)NULL},  //  k_boardYawRight,	      
    {(Uint8)NULL, SDLK_EQUALS  , KMOD_NONE  , (Uint16)NULL},  //  k_zoomIn,		      
    {(Uint8)NULL, SDLK_MINUS   , KMOD_NONE  , (Uint16)NULL},  //  k_zoomOut,		      
    {(Uint8)NULL, SDLK_g       , KMOD_LSHIFT, (Uint16)NULL},  //  k_grid1, 
    {(Uint8)NULL, SDLK_g       , KMOD_NONE  , (Uint16)NULL},  //  k_grid3,		      
    {(Uint8)NULL, SDLK_n       , KMOD_NONE  , (Uint16)NULL},  //  k_autoPilot,	      
    {(Uint8)NULL, SDLK_r       , KMOD_NONE  , (Uint16)NULL},  //  k_resetBoard,	      
    {(Uint8)NULL, SDLK_r       , KMOD_LSHIFT, (Uint16)NULL},  //  k_resetSnakes,	      
    {(Uint8)NULL, SDLK_b       , KMOD_NONE  , (Uint16)NULL},  //  k_backgroundColor,	      
    {(Uint8)NULL, SDLK_PERIOD  , KMOD_LSHIFT, (Uint16)NULL},  //  k_speedUp,		      
    {(Uint8)NULL, SDLK_COMMA   , KMOD_LSHIFT, (Uint16)NULL},  //  k_speedDown,	      
    {(Uint8)NULL, SDLK_e       , KMOD_NONE  , (Uint16)NULL},  //  k_showWalls,	      
    {(Uint8)NULL, SDLK_f       , KMOD_NONE  , (Uint16)NULL},  //  k_fullScreen,	      
    {(Uint8)NULL, (SDLKey)NULL , KMOD_NONE  , (Uint16)NULL},  //  k_lighting,		      
    {(Uint8)NULL, SDLK_p       , KMOD_NONE  , (Uint16)NULL},  //  k_pause,
#if (defined __APPLE__ && defined __MACH__)
    {(Uint8)NULL, SDLK_q       , KMOD_LMETA , (Uint16)NULL},  //  k_quit,
#else
    {(Uint8)NULL, SDLK_q       , KMOD_LALT  , (Uint16)NULL},  //  k_quit,
#endif     
    //    {(Uint8)NULL, SDLK_a       , KMOD_LALT  , (Uint16)NULL},  //  k_abort,
    {(Uint8)NULL, SDLK_ESCAPE  , KMOD_NONE  , (Uint16)NULL},  //  k_abort,
    {(Uint8)NULL, SDLK_t       , KMOD_NONE  , (Uint16)NULL}   //  k_indicator
};

/*
const int DEFAULT_KEYS_MOD[NKEYS] = {
    0,  		//  k_moveLeft,
    0,  		//  k_moveDown,		      
    0,  		//  k_moveRight,	      
    0,  		//  k_moveUp,		      
    0,  		//  k_moveForward,	      
    0,  		//  k_moveBackward,	      
    0,  		//  k_selfMoveLeft,	      
    0,  		//  k_selfMoveDown,	      
    0,  		//  k_selfMoveRight,	      
    0,  		//  k_selfMoveUp,	      
    0,  		//  k_selfYawLeft,	      
    0,  		//  k_selfYawRight,	      
    0,  		//  k_boardRotateLeft,	      
    0,  		//  k_boardRotateDown,	      
    0,  		//  k_boardRotateRight,	      
    0,  		//  k_boardRotateUp,	      
    0,  		//  k_boardYawLeft,	      
    0,  		//  k_boardYawRight,	      
    0,  		//  k_zoomIn,		      
    0,  		//  k_zoomOut,		      
    GLUT_ACTIVE_SHIFT,  //  k_grid1,
    0,  		//  k_grid3,		      
    0,  		//  k_autoPilot,	      
    0,  		//  k_resetBoard,	      
    GLUT_ACTIVE_SHIFT,  //  k_resetSnakes,	      
    0,  		//  k_backgroundColor, 
    GLUT_ACTIVE_SHIFT,  //  k_speedUp,		      
    GLUT_ACTIVE_SHIFT,  //  k_speedDown,	      
    0,  		//  k_showWalls,	      
    0,  		//  k_fullScreen,	      
    0,  		//  k_lighting,		      
    0,  		//  k_pause,    
    GLUT_ACTIVE_ALT,    //  k_quit
    GLUT_ACTIVE_ALT     //  k_abort
};
*/

const int DEFAULT_STEP_INTERVAL = 5;// 0x20;

const int FINE_TUNE_SPEED_THRESHOLD = 4;
const int MAX_FINE_TUNE = 10; //128;

const int NSPEEDS = 12;

const int SPEEDS[] = {
    0x001,
    0x002,
    0x004,
    0x008,
    0x010,
    0x020,  // default
    0x040,
    0x080,
    0x100,
    0x200,
    0x400,
    0x800
};

const char* SPEEDS_NAMES[] = {
    "As Fast As Possible",  //0x001
    "Faster", 		    //0x002
    "Fast",   		    //0x004
    "Quicker",              //0x008
    "Quick",                //0x010
    "Normal", 		    //0x020
    "Slow",   		    //0x040
    "Slower", 		    //0x080
    "Snail",                //0x100
    "Snailer",              //0x200
    "Snailest",             //0x400
    "I'll make some coffee" //0x800
};

    
const direction OPPOSITE[] = {right, up, left, down, backward, forward};
const int BASE_INTERVAL = 10; // 10

void* FONT = GLUT_BITMAP_9_BY_15;
void* STROKE_FONT = GLUT_STROKE_ROMAN;

/*
const int NRESOLUTIONS = 5;
const int RESOLUTIONS[NRESOLUTIONS][2] = {
    {640,480},
    {800,600},
    {1024,768},
    {1280,1024},
    {1600,1200}
};
const int DEFAULT_RESOLUTION = 1;
*/

const int INIT_WINDOW_POS[] = {100, 100};
const int INIT_FIELD_OF_VIEW = 35;
const int MIN_FIELD_OF_VIEW = 10;
const int MAX_FIELD_OF_VIEW = 179;

/* snake */
const int MAX_SNAKE_LENGTH = 200;
const int STARTING_SNAKE_LENGTH = 3;

//const direction STARTING_UP[] = {up, left, up, right, up, up};
const direction ROTATING_UP[][6][6] = {
// up =   left,     down,    right,       up, forward, backward
    {{      up,     down,       up,       up, forward, backward},  // new = left, old = left
     {      up,    right,       up,     left, forward, backward},  // new = down
     {      up,     down,       up,       up, forward, backward},  // new = right
     {      up,     left,       up,    right, forward, backward},  // new = up
     {      up,     down,       up,       up,   right,     left},  // new = forward
     {      up,     down,       up,       up,    left,    right}}, // new = backword

    {{      up,       up,     down,       up, forward, backward},  // new = left, old = down
     {    left,       up,    right,       up, forward, backward},  // new = down
     {    down,       up,       up,       up, forward, backward},  // new = right
     {    left,       up,    right,       up, forward, backward},  // new = up
     {    left,       up,    right,       up,      up,     down},  // new = forward
     {    left,       up,    right,       up,    down,       up}}, // new = backward

    {{      up,     down,       up,       up, forward, backward},  // new = left, old = right
     {      up,     left,       up,    right, forward, backward},  // new = down
     {      up,     down,       up,       up, forward, backward},  // new = right
     {      up,    right,       up,     left, forward, backward},  // new = up
     {      up,     down,       up,       up,    left,    right},  // new = forward
     {      up,     down,       up,       up,   right,     left}}, // new = backward

    {{    down,       up,       up,       up, forward, backward},  // new = left, old = up
     {    left,       up,    right,       up, forward, backward},  // new = down
     {      up,       up,     down,       up, forward, backward},  // new = right
     {    left,       up,    right,       up, forward, backward},  // new = up
     {    left,       up,    right,       up,    down,       up},  // new = forward
     {    left,       up,    right,       up,      up,     down}}, // new = backward

    {{backward,     down,  forward,       up,      up,       up},  // new = left, old = forward
     {    left, backward,    right,  forward,      up,       up},  // new = down
     { forward,     down, backward,       up,      up,       up},  // new = right
     {    left,  forward,    right, backward,      up,       up},  // new = up
     {    left,     down,    right,       up,      up,       up},  // new = forward
     {    left,     down,    right,       up,      up,       up}}, // new = backward

    {{ forward,     down, backward,       up,      up,       up},  // new = left, old = backward
     {    left,  forward,    right, backward,      up,       up},  // new = down
     {backward,     down,  forward,       up,      up,       up},  // new = right
     {    left, backward,    right,  forward,      up,       up},  // new = up
     {    left,     down,    right,       up,      up,       up},  // new = forward
     {    left,     down,    right,       up,      up,       up}}  // new = backward
};


const float UP_VECTOR[][3] = {
    {-1,  0,  0},
    { 0, -1,  0},
    { 1,  0,  0},
    { 0,  1,  0},
    { 0,  0, -1},
    { 0,  0,  1}};

const direction SELF_ROTATING[][4][6] = {
// up =   left,     down,    right,       up,  forward, backward
    {{    left,  forward,     left, backward,       up,     down},  // new = left, old = left
     {    left,       up,     left,     down, backward,  forward},  // new = down
     {    left, backward,     left,  forward,     down,       up},  // new = right
     {    left,     down,     left,       up,  forward, backward}   // new = up
    }, 
    {{backward,     left,  forward,     left,     left,    right},  // new = left, old = down
     {   right,     left,     left,     left, backward,  forward},  // new = down
     { forward,     left, backward,     left,    right,     left},  // new = right
     {    left,     left,    right,     left,  forward, backward}   // new = up
    },
    {{    left, backward,     left,  forward,     down,       up},  // new = left, old = right
     {    left,       up,     left,     down, backward,  forward},  // new = down
     {    left,  forward,     left, backward,       up,     down},  // new = right
     {    left,     down,     left,       up,  forward, backward}   // new = up
    },
    {{ forward,     left, backward,     left,    right,     left},  // new = left, old = up
     {   right,     left,     left,     left, backward,  forward},  // new = down
     {backward,     left,  forward,     left,     left,    right},  // new = right
     {    left,     left,    right,     left,  forward, backward}   // new = up
    },
    {{    down,    right,       up,     left,     left,     left},  // new = left, old = forward
     {   right,       up,     left,     down,     left,     left},  // new = down
     {      up,     left,     down,    right,     left,     left},  // new = right
     {    left,     down,    right,       up,     left,     left}   // new = up
    },
    {{      up,     left,     down,    right,     left,     left},  // new = left, old = backward
     {   right,       up,     left,     down,     left,     left},  // new = down
     {    down,    right,       up,     left,     left,     left},  // new = right
     {    left,     down,    right,       up,     left,     left}   // new = up
    }
};

const direction YAW_UP[][6][2] = {
// up = left, down, right, up, forward, backward
    {{left, left}, {forward, backward}, {left, left}, {backward, forward}, {up, down}, {down, up}},   // old = left
    {{backward, forward}, {left, left}, {forward, backward}, {left, left}, {left, right}, {right, left}},   // old = down
    {{left, left}, {backward, forward}, {left, left}, {forward, backward}, {down, up}, {up, down}},   // old = right
    {{forward, backward}, {left, left}, {backward, forward}, {left, left}, {right, left}, {left, right}},   // old = up
    {{down, up}, {right, left}, {up, down}, {left, right}, {left, left}, {left, left}},   // old = forward
    {{up, down}, {left, right}, {down, up}, {right, left}, {left, left}, {left, left}}   // old = backward
};

const direction RELATIVE_ROTATION[][6][6] = {
// up =  left,    down,   right,      up, forward, backward
    {{forward, forward, forward, forward, forward, forward}, // new = left, old = left
     {forward,      up, forward,    down,   right,    left}, // new = down
     {forward, forward, forward, forward, forward, forward}, // new = right
     {forward,    down, forward,      up,    left,   right}, // new = up
     {forward,    left, forward,   right,      up,    down}, // new = forward
     {forward,   right, forward,    left,    down,      up}  // new = backward
    },
    {{     up, forward,    down, forward,    left,   right}, // new = left, old = down
     {forward, forward, forward, forward, forward, forward}, // new = down
     {   down, forward,      up, forward,   right,    left}, // new = right
     {forward, forward, forward, forward, forward, forward}, // new = up
     {  right, forward,    left, forward,      up,    down}, // new = forward
     {   left, forward,   right, forward,    down,      up}  // new = backward
    },
    {{forward, forward, forward, forward, forward, forward}, // new = left, old = right
     {forward,      up, forward,    down,    left,   right}, // new = down
     {forward, forward, forward, forward, forward, forward}, // new = right
     {forward,    down, forward,      up,   right,    left}, // new = up
     {forward,   right, forward,    left,      up,    down}, // new = forward
     {forward,    left, forward,   right,    down,      up}  // new = backward
    },
    {{     up, forward,    down, forward,   right,    left}, // new = left, old = up
     {forward, forward, forward, forward, forward, forward}, // new = down
     {   down, forward,      up, forward,    left,   right}, // new = right
     {forward, forward, forward, forward, forward, forward}, // new = up
     {   left, forward,   right, forward,      up,    down}, // new = forward
     {  right, forward,    left, forward,    down,      up}  // new = backward
    },
    {{     up,   right,    down,    left, forward, forward}, // new = left, old = forward
     {   left,      up,   right,    down, forward, forward}, // new = down
     {   down,    left,      up,   right, forward, forward}, // new = right
     {  right,    down,    left,      up, forward, forward}, // new = up
     {forward, forward, forward, forward, forward, forward}, // new = forward
     {forward, forward, forward, forward, forward, forward}  // new = backward
    },
    {{     up,    left,    down,   right, forward, forward}, // new = left, old = backward
     {  right,      up,    left,    down, forward, forward}, // new = down
     {   down,   right,      up,    left, forward, forward}, // new = right
     {   left,    down,   right,      up, forward, forward}, // new = up
     {forward, forward, forward, forward, forward, forward}, // new = forward
     {forward, forward, forward, forward, forward, forward}  // new = backward
    }
};


const float HEAD_BEND[][6][2][3] = { //[old][new][action#][parameter#]
  {{{	0,   0,   0},{ 0, 0, 0}},  // new = left, old = left
   {{	0,-0.5,   0},{ 0, 0, 1}},  // new = down    
   {{	0,   0,   0},{ 0, 0, 0}},  // new = right   
   {{	0, 0.5,   0},{ 0, 0,-1}},  // new = up      
   {{	0,   0,-0.5},{ 0,-1, 0}},  // new = forward 
   {{	0,   0, 0.5},{ 0, 1, 0}}}, // new = backward

  {{{-0.5,   0,	  0},{ 0, 0,-1}},  // new = left, old = down
   {{   0,   0,	  0},{ 0, 0, 0}},  // new = down    
   {{ 0.5,   0,	  0},{ 0, 0, 1}},  // new = right   
   {{	0,   0,	  0},{ 0, 0, 0}},  // new = up      
   {{	0,   0,-0.5},{ 1, 0, 0}},  // new = forward 
   {{	0,   0, 0.5},{-1, 0, 0}}}, // new = backward

  {{{	0,   0,   0},{ 0, 0, 0}},  // new = left, old = right
   {{	0,-0.5,   0},{ 0, 0,-1}},  // new = down    
   {{	0,   0,   0},{ 0, 0, 0}},  // new = right   
   {{	0, 0.5,   0},{ 0, 0, 1}},  // new = up      
   {{	0,   0,-0.5},{ 0, 1, 0}},  // new = forward 
   {{	0,   0, 0.5},{ 0,-1, 0}}}, // new = backward

  {{{-0.5,   0,	  0},{ 0, 0, 1}},  // new = left, old = up
   {{   0,   0,	  0},{ 0, 0, 0}},  // new = down    
   {{ 0.5,   0,	  0},{ 0, 0,-1}},  // new = right   
   {{	0,   0,	  0},{ 0, 0, 0}},  // new = up      
   {{	0,   0,-0.5},{-1, 0, 0}},  // new = forward 
   {{	0,   0, 0.5},{ 1, 0, 0}}}, // new = backward
  
  {{{-0.5,   0,	  0},{ 0, 1, 0}},  // new = left, old = forward
   {{   0,-0.5,	  0},{-1, 0, 0}},  // new = down    
   {{ 0.5,   0,	  0},{ 0,-1, 0}},  // new = right   
   {{   0, 0.5,	  0},{ 1, 0, 0}},  // new = up      
   {{   0,   0,	  0},{ 0, 0, 0}},  // new = forward 
   {{   0,   0,	  0},{ 0, 0, 0}}}, // new = backward

  {{{-0.5,   0,	  0},{ 0,-1, 0}}, // new = left, old = backward
   {{	0,-0.5,	  0},{ 1, 0, 0}}, // new = down    
   {{ 0.5,   0,	  0},{ 0, 1, 0}}, // new = right   
   {{	0, 0.5,	  0},{-1, 0, 0}}, // new = up      
   {{	0,   0,	  0},{ 0, 0, 0}}, // new = forward 
   {{	0,   0,	  0},{ 0, 0, 0}}} // new = backward
};


const float HEAD_BEND_BASE[6][3] = {
  { 0.5,    0, 	  0}, // old = left
  {   0,  0.5, 	  0}, // old = down
  {-0.5,    0, 	  0}, // old = right
  {   0, -0.5, 	  0}, // old = up
  {   0,    0,  0.5}, // old = forward
  {   0,    0, -0.5}  // old = backward
};

const float JOINT_EDGES[][3] = {  // 0.3*sin45 = 0.212132034
    {0.3         , -0.5, 0           },
    {0.212132034 , -0.5, 0.212132034 },
    {0           , -0.5, 0.3         },
    {-0.212132034, -0.5, 0.212132034 },
    {-0.3        , -0.5, 0           },
    {-0.212132034, -0.5, -0.212132034},
    {0           , -0.5, -0.3        },
    {0.212132034 , -0.5, -0.212132034},
    {0.3         , -0.5, 0           }
};


const float JOINT_NORMALS[][3] = {
    { 1, 0,  0},
    { 1, 0,  1},
    { 0, 0,  1},
    {-1, 0,  1},
    {-1, 0,  0},
    {-1, 0, -1},
    { 0, 0, -1},
    { 1, 0, -1},
    { 1, 0,  0}
};


const int NAI = 3;
const int NAI_TARGET = 3;

extern const char* AI_NAMES[NAI + 1] = {
    "Off",
    "Human",
    "Basic",
    "Remote"
};

extern const char* AIT_NAMES[NAI_TARGET] = {
    "Food",
    "Head",
    "Tail"
};
    
// const int AP_SAFE = -2;
// const int AP_SWITCH = -1;

// const int AP_OFF   = 0;
// const int AP_FOOD  = 1;
// const int AP_SNAKE = 2;


/* board */

//const int MAX_SNAKES = 4;



const int TINY_BOARD   = 5;
const int SMALL_BOARD  = 10;
const int MEDIUM_BOARD = 15;
const int LARGE_BOARD  = 20;
const int HUGE_BOARD   = 25;


/*
const int BOARD_2ROOMS_1COR = 5;

const int BOARDS = 6;

const int BOARD_SIZE[] = {5, 10, 15, 20, 25,
                          20};

const int STARTING_LOCATIONS[][4][4] = {
    {{0, 0, 0, right}, {0, 0,  4, right}, {0,  4, 0, right}, {0,  4,  4, right}},   // TINY_BOARD;
    {{0, 0, 0, right}, {0, 0,  9, right}, {0,  9, 0, right}, {0,  9,  9, right}},   // SMALL_BOARD;
    {{0, 0, 0, right}, {0, 0, 14, right}, {0, 14, 0, right}, {0, 14, 14, right}},   // MEDIUM_BOARD;
    {{0, 0, 0, right}, {0, 0, 19, right}, {0, 19, 0, right}, {0, 19, 19, right}},   // BIG_BOARD;
    {{0, 0, 0, right}, {0, 0, 24, right}, {0, 24, 0, right}, {0, 24, 24, right}},   // HUGE_BOARD;
    {{0, 6, 6, right}, {0, 6, 13, right}, {0, 13, 6, right}, {0, 13, 13, right}},   // BOARD_2ROOMS_1COR;
};
*/


//const int NBOARDS = 5;
/*
const char* BOARDS_NAMES[NBOARDS] = {
    "cube(tiny)",
    "cube(small)",
    "cube(medium)",
    "cube(large)",
    "cube(huge)"
};
*/

const int DEFAULT_BOARD_TYPE = 0;
const int DEFAULT_BOARD = 2;

const double BOARD_REAL_SIZE = 1;
const double BOARD_DEPTH = 3;
const double BOARD_RADII = sqrt(((BOARD_REAL_SIZE / 2) * (BOARD_REAL_SIZE / 2)) * 3);

const float BOARD_ROTATION_ANGLE = 1.0;
const float BOARD_ZOOM = 0.25;


//const int EMPTY   = 0;
//const int FOOD    = 1;
//const int SNAKE   = 2;
//const int OUTSIDE = 3;

/* light and material */
const float LIGHT_AMBIENT[] = {0.5, 0.5, 0.5, 0.0};
const float LIGHT_DIFFUSE[] = {0.7, 0.7, 0.7, 0.0};
const float LIGHT_SPECULAR[] = {0.3, 0.3, 0.3, 0.0};
const float LIGHT_POSITION[] = {BOARD_RADII * 2, BOARD_RADII * 2, -BOARD_DEPTH / 2, 1.0};

const float FOOD_AMBIENT[] = {0.4, 0.4, 0.4};
const float FOOD_DIFFUSE[] = {0.0, 1.0, 1.0};
const float FOOD_SPECULAR[] = {0.1, 0.1, 0.1};
const float FOOD_SHININESS = 0.0;
const float FOOD_EMISSION[] = {0.0, 0.0, 0.0};

const float SNAKE_AMBIENT[] = {0.3, 0.3, 0.3};
const float SNAKE_SPECULAR[] = {1.0, 1.0, 1.0};
const float SNAKE_SHININESS = 5.0;
const float SNAKE_EMISSION[] = {0.0, 0.0, 0.0};

const float EYES_AMBIENT[] = {0.4, 0.4, 0.4};
const float EYES_DIFFUSE[] = {COLORS[YELLOW][RED], COLORS[YELLOW][GREEN], COLORS[YELLOW][BLUE]};
const float EYES_SPECULAR[] = {1.0, 1.0, 1.0};
const float EYES_SHININESS = 10.0;
const float EYES_EMISSION[] = {COLORS[YELLOW][RED], COLORS[YELLOW][GREEN], COLORS[YELLOW][BLUE]};

const float WALL_AMBIENT[] = {0.4, 0.4, 0.4};
const float WALL_DIFFUSE[] = {0.5, 0.75, 1.0, 0.20};
const float WALL_SPECULAR[] = {0.7, 0.7, 0.7};
const float WALL_SHININESS = 2.0;
const float WALL_EMISSION[] = {0.0, 0.0, 0.0};


const int MENU_COLOR = GREEN;
const int MENU_SELECT_COLOR = WHITE;
const int MENU_INSIDE_COLOR = RED;
const int MENU_HEADER_COLOR = CYAN;
const int ABOUT_COLOR = LIGHT_GRAY;
const int INSTRUCTIONS_COLOR = WHITE;
const int HIGHLIGHT_COLOR = LIGHT_GREEN;
const int INPUT_COLOR = PURPLE;

const keys_t KEY_MENUS_MOVEMENT[NKEYS][4] = {
    // left,             down,                right,                up
    {k_moveLeft,         k_moveDown,          k_selfMoveLeft,       k_speedDown          }, //k_moveLeft,                 
    {k_moveDown,         k_moveRight,         k_selfMoveDown,       k_moveLeft           }, //k_moveDown,                 
    {k_moveRight,        k_moveUp,            k_selfMoveRight,      k_moveDown           }, //k_moveRight,        
    {k_moveUp,           k_moveForward,       k_selfMoveUp,         k_moveRight          }, //k_moveUp,           
    {k_moveForward,      k_moveBackward,      k_selfYawLeft,        k_moveUp             }, //k_moveForward,      
    {k_moveBackward,     k_boardRotateLeft,   k_selfYawRight,       k_moveForward        }, //k_moveBackward,
    
    {k_moveLeft,         k_selfMoveDown,      k_selfMoveLeft,       k_quit               }, //k_selfMoveLeft,     
    {k_moveDown,         k_selfMoveRight,     k_selfMoveDown,       k_selfMoveLeft       }, //k_selfMoveDown,     
    {k_moveRight,        k_selfMoveUp,        k_selfMoveRight,      k_selfMoveDown       }, //k_selfMoveRight,    
    {k_moveUp,           k_selfYawLeft,       k_selfMoveUp,         k_selfMoveRight      }, //k_selfMoveUp,       
    {k_moveForward,      k_selfYawRight,      k_selfYawLeft,        k_selfMoveUp         }, //k_selfYawLeft,      
    {k_moveBackward,     k_zoomIn,            k_selfYawRight,       k_selfYawLeft        }, //k_selfYawRight,
    
    {k_boardRotateLeft,  k_boardRotateDown,   k_zoomIn,             k_moveBackward       }, //k_boardRotateLeft,  
    {k_boardRotateDown,  k_boardRotateRight,  k_zoomOut,            k_boardRotateLeft    }, //k_boardRotateDown,  
    {k_boardRotateRight, k_boardRotateUp,     k_showWalls,          k_boardRotateDown    }, //k_boardRotateRight,         
    {k_boardRotateUp,    k_boardYawLeft,      k_fullScreen,         k_boardRotateRight   }, //k_boardRotateUp,    
    {k_boardYawLeft,     k_boardYawRight,     k_backgroundColor,    k_boardRotateUp      }, //k_boardYawLeft,     
    {k_boardYawRight,    k_autoPilot,         k_grid1,              k_boardYawLeft       }, //k_boardYawRight,
    
    {k_boardRotateLeft,  k_zoomOut,           k_zoomIn,             k_selfYawRight       }, //k_zoomIn,           
    {k_boardRotateDown,  k_showWalls,         k_zoomOut,            k_zoomIn             }, //k_zoomOut,
    
    {k_boardYawRight,    k_grid3,             k_grid1,              k_backgroundColor    }, //k_grid1,            
    {k_grid3,            k_indicator,         k_grid3,              k_grid1              }, //k_grid3,

    {k_autoPilot,        k_resetBoard,        k_pause,              k_boardYawRight      }, //k_autoPilot,        
    {k_resetBoard,       k_resetSnakes,       k_abort,              k_autoPilot          }, //k_resetBoard,       
    {k_resetSnakes,      k_speedUp,           k_quit,               k_resetBoard         }, //k_resetSnakes,      
    {k_boardYawLeft,     k_grid1,             k_backgroundColor,    k_fullScreen         }, //k_backgroundColor,  
    {k_speedUp,          k_speedDown,         k_speedUp,            k_resetSnakes        }, //k_speedUp,          
    {k_speedDown,        k_moveLeft,          k_speedDown,          k_speedUp            }, //k_speedDown,        
    {k_boardRotateRight, k_fullScreen,        k_showWalls,          k_zoomOut            }, //k_showWalls,        
    {k_boardRotateUp,    k_backgroundColor,   k_fullScreen,         k_showWalls          }, //k_fullScreen,       
    {k_lighting,         k_lighting,          k_lighting,           k_lighting           }, //k_lighting,                 
    {k_autoPilot,        k_abort,             k_pause,              k_indicator          }, //k_pause,            
    {k_resetSnakes,      k_selfMoveLeft,      k_quit,               k_abort              }, //k_quit,
    {k_resetBoard,       k_quit,              k_abort,              k_pause              }, //k_abort
    {k_indicator,        k_pause,             k_indicator,          k_grid3              }  //k_indicator
    
    // left,             down,                right,                up
};

extern const char* GRID_NAMES[4] = {
    "Non",
    "Back walls",
    "All walls",
    "Full"
};

const Uint16 DEFAULT_PORT = 3830;

const char* VERSION = "0.9";
