/*  
 *  snake3d, a snake game in a 3d world.
 *  Copyright (C) 2005-2006 Yair Yarom
 *  
 *  This program is free software; you can redistribute it and/or modify it under
 *  the terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or (at your option) any later
 *  version.
 *  
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 *  PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 *  Place, Suite 330, Boston, MA 02111-1307 USA
 *  
 *  http://sourceforge.net/projects/worms3d
 *  
 */

#ifndef CONSTS_HPP
#define CONSTS_HPP

#include "types.hpp"
#include "includes.hpp"

#ifndef NULL
#define NULL 0
#endif

extern const int X;
extern const int Y;
extern const int Z;

extern const int RED;
extern const int GREEN;
extern const int BLUE;
extern const int CYAN;
extern const int YELLOW;
extern const int MAGENTA;
extern const int BLACK;
extern const int WHITE;
extern const int BROWN;
extern const int ORANGE;
extern const int PURPLE;
extern const int LIGHT_GRAY;
extern const int GRAY;
extern const int DARK_GRAY;
extern const int DARK_GREEN;
extern const int DARK_BLUE;
extern const int DARK_RED;

extern const int NCOLORS;

extern float COLORS[][3];
extern char* COLORS_NAMES[];

//extern const int NKEYS;
#define NKEYS 35

extern char* KEY_NAME[];
//extern const char DEFAULT_KEYS[];
//extern const int DEFAULT_KEYS_MOD[];
extern const SDL_keysym DEFAULT_KEYS_SDL[];
extern const bool ALLOWED_KEYS[NKEYS][3];

extern const keys_t k_move[];
extern const keys_t k_selfMove[];
extern const keys_t k_selfYaw[];
extern const keys_t k_boardRotate[];
extern const keys_t k_boardYaw[];


extern const int DEFAULT_STEP_INTERVAL;
extern const int NSPEEDS;
extern const char* SPEEDS_NAMES[];
extern const int SPEEDS[];
extern const int FINE_TUNE_SPEED_THRESHOLD;
extern const int MAX_FINE_TUNE;

extern const direction OPPOSITE[];

extern const int BASE_INTERVAL;

extern void* FONT;
extern void* STROKE_FONT;

/*
extern const int NRESOLUTIONS;
extern const int RESOLUTIONS[][2];
extern const int DEFAULT_RESOLUTION;
*/

extern const int INIT_WINDOW_POS[];
extern const int INIT_FIELD_OF_VIEW;
extern const int MIN_FIELD_OF_VIEW;
extern const int MAX_FIELD_OF_VIEW;

/* snake */
extern const int MAX_SNAKE_LENGTH;
extern const int STARTING_SNAKE_LENGTH;

//extern const direction STARTING_UP[];
extern const direction ROTATING_UP[][6][6];  // [old][new][up]
extern const float UP_VECTOR[][3];
extern const direction SELF_ROTATING[][4][6]; // [old][new][up]
extern const direction YAW_UP[][6][2]; // [old][up][left, !=left (right)]
extern const direction RELATIVE_ROTATION[][6][6]; // [old][new][up]
extern const float HEAD_BEND_BASE[][3];  //[old][parameter#]
extern const float HEAD_BEND[][6][2][3]; //[old][new][action#][parameter#]

extern const float JOINT_EDGES[][3];
extern const float JOINT_NORMALS[][3];

/* AI */
extern const int NAI;
extern const int NAI_TARGET;
extern const char* AI_NAMES[];
extern const char* AIT_NAMES[];

// /* auto pilot */
// extern const int AP_SWITCH;
// extern const int AP_OFF;
// extern const int AP_FOOD;
// extern const int AP_SNAKE;
// extern const int AP_SAFE;

/* board */
#define MAX_SNAKES 4
//extern const int MAX_SNAKES;

extern const int TINY_BOARD;
extern const int SMALL_BOARD;
extern const int MEDIUM_BOARD;
extern const int LARGE_BOARD;
extern const int HUGE_BOARD;


//#define NBOARDS 8
extern const int DEFAULT_BOARD;
extern const int DEFAULT_BOARD_TYPE;


extern const double BOARD_REAL_SIZE;
extern const double BOARD_DEPTH;
extern const double BOARD_RADII;

extern const float BOARD_ROTATION_ANGLE;
extern const float BOARD_ZOOM;

//extern const int EMPTY;
//extern const int FOOD;
//extern const int SNAKE;
//extern const int OUTSIDE;

/* light and material */
extern const float LIGHT_AMBIENT[];
extern const float LIGHT_DIFFUSE[];
extern const float LIGHT_SPECULAR[];
extern const float LIGHT_POSITION[];

extern const float FOOD_AMBIENT[];
extern const float FOOD_DIFFUSE[];
extern const float FOOD_SPECULAR[];
extern const float FOOD_SHININESS;
extern const float FOOD_EMISSION[];

extern const float SNAKE_AMBIENT[];
extern const float SNAKE_SPECULAR[];
extern const float SNAKE_SHININESS;
extern const float SNAKE_EMISSION[];

extern const float EYES_AMBIENT[];
extern const float EYES_DIFFUSE[];
extern const float EYES_SPECULAR[];
extern const float EYES_SHININESS;
extern const float EYES_EMISSION[];

extern const float WALL_AMBIENT[];
extern const float WALL_DIFFUSE[];
extern const float WALL_SPECULAR[];
extern const float WALL_SHININESS;
extern const float WALL_EMISSION[];


extern const int MENU_COLOR;
extern const int MENU_SELECT_COLOR;
extern const int MENU_INSIDE_COLOR;
extern const int MENU_HEADER_COLOR;
extern const int ABOUT_COLOR;
extern const int INSTRUCTIONS_COLOR;
extern const int HIGHLIGHT_COLOR;
extern const int INPUT_COLOR;

extern const keys_t KEY_MENUS_MOVEMENT[NKEYS][4];

extern const char* GRID_NAMES[4];

extern const Uint16 DEFAULT_PORT;

extern const char* VERSION;
#endif
