/*  
 *  snake3d, a snake game in a 3d world.
 *  Copyright (C) 2005-2006 Yair Yarom
 *  
 *  This program is free software; you can redistribute it and/or modify it under
 *  the terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or (at your option) any later
 *  version.
 *  
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 *  PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 *  Place, Suite 330, Boston, MA 02111-1307 USA
 *  
 *  http://sourceforge.net/projects/worms3d
 *  
 */

#include "includes.hpp"
#include "cube.hpp"

Cube::Cube(int size) {
    size = (size < 2 ? 2 : size);
    _maxSize = size;
    _startingLocations = new int*[MAX_SNAKES];
    _startingDirections = new direction[MAX_SNAKES];
    _startingUps = new direction[MAX_SNAKES];
    for (int i = 0; i < MAX_SNAKES; i++)
        _startingLocations[i] = new int[4];
    _startingLocations[0][X] = 0;
    _startingLocations[0][Y] = 0;
    _startingLocations[0][Z] = 0;
    _startingDirections[0] = up;
    _startingUps[0] = right;

    _startingLocations[1][X] = 0;
    _startingLocations[1][Y] = 0;
    _startingLocations[1][Z] = size - 1;
    _startingDirections[1] = forward;
    _startingUps[1] = right;

    _startingLocations[2][X] = 0;
    _startingLocations[2][Y] = size - 1;
    _startingLocations[2][Z] = 0;
    _startingDirections[2] = backward;
    _startingUps[2] = right;

    _startingLocations[3][X] = 0;
    _startingLocations[3][Y] = size - 1;
    _startingLocations[3][Z] = size - 1;
    _startingDirections[3] = down;
    _startingUps[3] = right;

    
//     if      (size == TINY_BOARD)   _id = 0;
//     else if (size == SMALL_BOARD)  _id = 1;
//     else if (size == MEDIUM_BOARD) _id = 2;
//     else if (size == LARGE_BOARD)  _id = 3;
//     else if (size == HUGE_BOARD)   _id = 4;

}

const char* Cube::getName() {
    if      (_maxSize == TINY_BOARD)   return "Cube(tiny)";
    else if (_maxSize == SMALL_BOARD)  return "Cube(small)";
    else if (_maxSize == MEDIUM_BOARD) return "Cube(medium)";
    else if (_maxSize == LARGE_BOARD)  return "Cube(large)";
    else if (_maxSize == HUGE_BOARD)   return "Cube(huge)";
    return "Cube(unknown)";
};

Board* Cube::clone() {
    return new Cube((int)_maxSize);
}

occupancy_t Cube::checkLocation(int* location, Snake* snake) {
    if (location[X] < 0 || location[X] >= _maxSize ||
        location[Y] < 0 || location[Y] >= _maxSize ||
        location[Z] < 0 || location[Z] >= _maxSize)
	return o_outside;
    return Board::checkLocation(location, snake);
}

void Cube::drawFrame() {
    glColor3fv(COLORS[WHITE]);
    glLineWidth(2);

    glBegin(GL_LINE_STRIP);
    glVertex3d(       0,        0,        0);
    glVertex3d(       0,        0, _maxSize);
    glVertex3d(       0, _maxSize, _maxSize);
    glVertex3d(       0, _maxSize,        0);
    glVertex3d(       0,        0,        0);
    glVertex3d(_maxSize,        0,        0); 
    glVertex3d(_maxSize,        0, _maxSize);
    glVertex3d(_maxSize, _maxSize, _maxSize);
    glVertex3d(_maxSize, _maxSize,        0);
    glVertex3d(_maxSize,        0,        0);  
    glEnd();

    glBegin(GL_LINES);
    glVertex3d(       0,        0, _maxSize);
    glVertex3d(_maxSize,        0, _maxSize);

    glVertex3d(       0, _maxSize, _maxSize);
    glVertex3d(_maxSize, _maxSize, _maxSize);
    
    glVertex3d(       0, _maxSize,        0);
    glVertex3d(_maxSize, _maxSize,        0);
    glEnd();
    glLineWidth(1);
};

void Cube::drawGrid(int grid) {
    glColor3fv(COLORS[LIGHT_GRAY]);
    glBegin(GL_LINES);
    switch (grid) {
      case 3:
          for (int j = 1; j < _maxSize; j++)
              for (int i = 1; i < _maxSize; i++) {
                  glVertex3d(i, j,    0);
                  glVertex3d(i, j, _maxSize);
                  
                  glVertex3d(i, 0,    j);
                  glVertex3d(i, _maxSize, j);
                  
                  glVertex3d(0   , i, j);
                  glVertex3d(_maxSize, i, j);
              };
      case 2:
          for (int i = 1; i <_maxSize; i++) {
              glVertex3d(   i, _maxSize,    0);
              glVertex3d(   i, _maxSize, _maxSize);
              glVertex3d(0   , _maxSize,    i);
              glVertex3d(_maxSize, _maxSize,    i);
              
              glVertex3d(   i,    0, _maxSize);
              glVertex3d(   i, _maxSize, _maxSize);
              glVertex3d(0   ,    i, _maxSize);
              glVertex3d(_maxSize,    i, _maxSize);
              
	      glVertex3d(_maxSize, i,    0);
	      glVertex3d(_maxSize, i, _maxSize);
	      glVertex3d(_maxSize,    0, i);
	      glVertex3d(_maxSize, _maxSize, i);
	  }
      case 1:
	  for (int i = 1; i < _maxSize; i++) {
	      // down
	      glVertex3d(   i,    0,    0);
	      glVertex3d(   i,    0, _maxSize);
	      glVertex3d(   0,    0,    i);
	      glVertex3d(_maxSize,    0,    i);
	      
	      // back
	      glVertex3d(i,    0, 0);
	      glVertex3d(i, _maxSize, 0);
	      glVertex3d(0   , i, 0);
	      glVertex3d(_maxSize, i, 0);
	      
	      // left
	      glVertex3d(0, i,    0);
	      glVertex3d(0, i, _maxSize);
	      glVertex3d(0,    0, i);
	      glVertex3d(0, _maxSize, i);
	  }

    }       
    glEnd();
};

void Cube::drawWalls() {
    glEnable(GL_BLEND);
    glDepthMask(GL_FALSE);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE);
    glMaterialfv(GL_FRONT, GL_AMBIENT, WALL_AMBIENT);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, WALL_DIFFUSE);
    glMaterialfv(GL_FRONT, GL_SPECULAR, WALL_SPECULAR);
    glMaterialf(GL_FRONT, GL_SHININESS, WALL_SHININESS);
    glMaterialfv(GL_FRONT, GL_EMISSION, WALL_EMISSION);
    glColor4fv(WALL_DIFFUSE);
    glPushMatrix();

    glTranslatef(_maxSize / 2, _maxSize / 2, _maxSize / 2);
    glutSolidCube(_maxSize);

    glPopMatrix();
    glDepthMask(GL_TRUE);
    glDisable(GL_BLEND);
}
