/*  
 *  snake3d, a snake game in a 3d world.
 *  Copyright (C) 2005-2006 Yair Yarom
 *  
 *  This program is free software; you can redistribute it and/or modify it under
 *  the terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or (at your option) any later
 *  version.
 *  
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 *  PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 *  Place, Suite 330, Boston, MA 02111-1307 USA
 *  
 *  http://sourceforge.net/projects/worms3d
 *  
 */

#include <cmath>
#include <string.h>
#include <stdlib.h>

#include "functions.hpp"
#include "global.hpp"
#include "game.hpp"
#include "menu.hpp"
#include "board.hpp"
#include "snake.hpp"
#include "preferences.hpp"

double distance(int* pointA, int* pointB) {
    int x = pointA[X] - pointB[X];
    int y = pointA[Y] - pointB[Y];
    int z = pointA[Z] - pointB[Z];
    return sqrt(x * x + y * y + z * z);
}

bool deadEnd(occupancy_t occupancy) {
    switch (occupancy) {
      case o_empty:
      case o_food:
	  return false;

      case o_intercept:
      case o_snake:
      case o_outside:
	  return true;
    }
    return true;
}

/*
void lighting(bool activation) {
    if (activation && lightingEnabled)
        glEnable(GL_LIGHTING);
    else
        glDisable(GL_LIGHTING);
}
*/

int drawString(GLfloat x, GLfloat y, GLfloat z, char *string) {
    int len, i;
    glRasterPos3f(x, y, z);
    len = (int) strlen(string);
    for (i = 0; i < len; i++) {
        glutBitmapCharacter(FONT, string[i]);
    }
    return glutBitmapLength(FONT, (const unsigned char*)string);
}

int drawString(GLfloat x, GLfloat y, const char *string) {
    int len, i;
    glRasterPos2f(x, y);
    len = (int) strlen(string);

//     glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
//     glEnable(GL_BLEND);
//     glEnable(GL_LINE_SMOOTH);
//     glLineWidth(2.0);
    
    for (i = 0; i < len; i++) {
        glutBitmapCharacter(FONT, string[i]);
	//glutStrokeCharacter(GLUT_STROKE_ROMAN, string[i]);
    }
    
//     glDisable(GL_BLEND);
//     glDisable(GL_LINE_SMOOTH);
//     glLineWidth(1.0);
    return glutBitmapLength(FONT, (const unsigned char*)string);
}

void drawColoredString(GLfloat x, GLfloat y, const char *string) {
    char* end = (char*)string + strlen(string);
    char* i = (char*)string;
    char* pointer;
    char str[255];
    str[0] = 0;
    while (i < end) {
	if (i == string)
	    pointer = (char*)memchr(i, '$', 255);
	else 
	    pointer = (char*)memchr(i + 1, '$', 255);
	if (pointer == NULL || pointer > end) {
	    pointer = end;
	}

	memcpy(str, i, (int)pointer - (int)i);
	str[(int)pointer - (int)i] = 0;

	drawString(x, y, str);

	i = pointer + 2;
	if (*pointer == '$') {
	    if (*(pointer + 1) == 'I')
		colorIt(INSTRUCTIONS_COLOR);
	    else if (*(pointer + 1) == 'H')
		colorIt(HIGHLIGHT_COLOR);
	    else if (*(pointer + 1) == '$')
		i = pointer + 1;
	}
	x += glutBitmapLength(FONT, (const unsigned char*)str);
    }

}

void drawChar(GLfloat x, GLfloat y, GLfloat z, char character) {
    glRasterPos3f(x, y, z);
    glutBitmapCharacter(GLUT_BITMAP_9_BY_15, character);
}

void drawChar(GLfloat x, GLfloat y, char character) {
    glRasterPos2f(x, y);
    glutBitmapCharacter(GLUT_BITMAP_9_BY_15, character);
}

void drawInt(GLfloat x, GLfloat y, int integer) {
    char c;
    int length = (int)floor(log10((float)integer)) + 1;
    for (int i = length; i > 0; i--) {
        int digit = (integer / (int)pow(10, i - 1));
        c = digit + (int)'0';
        drawChar(x, y, c);
        x += 9;
        integer -= digit * (int)pow(10, i - 1);
    }
}

void drawStroke(GLfloat x, GLfloat y, GLfloat z, char *text) {
  //glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  //glEnable(GL_BLEND);
  glEnable(GL_LINE_SMOOTH);
  glLineWidth(20);
  glPushMatrix();
  glTranslatef(x, y, z);
  for (char *p = text; *p; p++)
    glutStrokeCharacter(STROKE_FONT, *p);
  glPopMatrix();
  glLineWidth(1);
  glDisable(GL_LINE_SMOOTH);
  //glDisable(GL_BLEND);
}

void drawScreen(bool lines) {
    glDisable(GL_LIGHTING);
    glBegin(GL_LINES);
    
    glColor3fv(COLORS[LIGHT_GRAY]);
    glVertex2d(201,             0);
    glVertex2d(201, prefs.windowSize[Y]);

    if (lines) {
	glVertex2d(  0, 201);
	glVertex2d(200, 201);
	
	glVertex2d(  0, 401);
	glVertex2d(200, 401);
    }
    
    glColor3fv(COLORS[DARK_GRAY]);
    glVertex2d(199,             0);

    if (lines) {
	glVertex2d(199,           200);
	glVertex2d(199,           202);
	glVertex2d(199,           400);
	glVertex2d(199,           402);
	glVertex2d(199, prefs.windowSize[Y]);
    
	glVertex2d(0, 199);
	glVertex2d(199, 199);

	glVertex2d(  0, 399);
	glVertex2d(199, 399);
    } else 
	glVertex2d(199, prefs.windowSize[Y]);
    
    glColor3fv(COLORS[GRAY]);
    glVertex2d(200,             0);
    glVertex2d(200, prefs.windowSize[Y]);

    if (lines) {
	glVertex2d(  0, 200);
	glVertex2d(200, 200);

	glVertex2d(  0, 400);
	glVertex2d(200, 400);
    }
    
    glEnd();

}

void loadPerspectives() {
    float aspect = (float)(prefs.windowSize[X] - 200)/prefs.windowSize[Y];
    glMatrixMode(GL_PROJECTION) ;

    
    // third person view
    float fov = prefs.fieldOfView;
    if (aspect < 1.0)
	fov = fov / aspect;

    if (fov < MIN_FIELD_OF_VIEW)
	fov = MIN_FIELD_OF_VIEW;
    if (fov > MAX_FIELD_OF_VIEW)
	fov = MAX_FIELD_OF_VIEW;
    
    glLoadIdentity() ;
    gluPerspective(fov,  aspect ,
		   BOARD_DEPTH - BOARD_RADII /* near */,
		   BOARD_DEPTH + BOARD_RADII /* far */);
    glGetFloatv(GL_PROJECTION_MATRIX, thirdPersonProjectionMatrix);

    // first person view
    glLoadIdentity();
    gluPerspective(INIT_FIELD_OF_VIEW * 2, 1, 0.0001, BOARD_REAL_SIZE * 1.5);
    glGetFloatv(GL_PROJECTION_MATRIX, firstPersonProjectionMatrix);

    // compass
    glLoadIdentity();
    glOrtho(-1, 1, -1, 1, 0, 2);
    //gluPerspective(FIELD_OF_VIEW * 3, 1, 0.001, 2);
    glGetFloatv(GL_PROJECTION_MATRIX, compassProjectionMatrix);
    //glViewport(0, 0, (GLsizei)w, (GLsizei)h) ;

    // screen
    glLoadIdentity();
    gluOrtho2D(0, prefs.windowSize[X], 0, prefs.windowSize[Y]);
    glGetFloatv(GL_PROJECTION_MATRIX, screenProjectionMatrix);

    // view
    glLoadIdentity();
    gluOrtho2D(0, prefs.windowSize[X] - 200, 0, prefs.windowSize[Y]);
    glGetFloatv(GL_PROJECTION_MATRIX, viewProjectionMatrix);
}

void reshape() {
    const SDL_VideoInfo* video = SDL_GetVideoInfo( );
    
    if (!video) {
        fprintf(stderr, "Couldn't get video information: %s\n", SDL_GetError());
	quit(1);
    }

    Uint32 flags = SDL_OPENGL;
    if (prefs.fullscreen)
	flags |= SDL_FULLSCREEN;
    /*
    else
	flags |= SDL_RESIZABLE;
    */

    if (SDL_SetVideoMode(prefs.windowSize[X], prefs.windowSize[Y], video->vfmt->BitsPerPixel, flags) == 0) {
        fprintf(stderr, "Couldn't set video mode: %s\n", SDL_GetError());
	quit(1);
    }

    SDL_WM_SetCaption("Snake 3d", "Snake 3d");
 
    glClearColor(COLORS[prefs.backgroundColor][RED], COLORS[prefs.backgroundColor][GREEN], COLORS[prefs.backgroundColor][BLUE], 0.0) ;
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    glEnable(GL_NORMALIZE);
    glShadeModel(GL_SMOOTH);
    glLightfv(GL_LIGHT0, GL_AMBIENT, LIGHT_AMBIENT);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, LIGHT_DIFFUSE);
    glLightfv(GL_LIGHT0, GL_SPECULAR, LIGHT_SPECULAR);
    if (cylinder) gluDeleteQuadric(cylinder);
    cylinder = gluNewQuadric();
    gluQuadricNormals(cylinder, GLU_SMOOTH);

    loadPerspectives();
}

void drawboard(bool first) {
    
    // move to location
    if (first) {
	
	// scale
	//double temp = /* 0.5 * */ BOARD_REAL_SIZE / BOARD_SIZE[BOARDS - 1];
	double temp = BOARD_REAL_SIZE / mainBoard->getMaxSize();
	glScalef(temp, temp, temp);
	player->lookAt();
    } else {
	glTranslatef(0, 0, -BOARD_DEPTH);
        
	// rotate
	glMultMatrixf(prefs.boardRotationMatrix);
            
	// scale
	//double temp = BOARD_REAL_SIZE / BOARD_SIZE[BOARDS - 1];
	double temp = BOARD_REAL_SIZE / mainBoard->getMaxSize();
	glScalef(temp, temp, temp);

	// center
	temp = -mainBoard->getMaxSize() / 2.0;
	glTranslatef(temp, temp, temp);

    }
    
    mainBoard->draw(first, (first ? prefs.gridLevel1 : prefs.gridLevel3), prefs.showWalls);
}

void switchFullScreen() {
    if (nResolutions == 0)
	return;
    
    if (prefs.fullscreen) {
	prefs.windowSize[X] = prefs.realWindowSize[X];
	prefs.windowSize[Y] = prefs.realWindowSize[Y];
    } else {
	prefs.realWindowSize[X] = prefs.windowSize[X];
	prefs.realWindowSize[Y] = prefs.windowSize[Y];
	prefs.windowSize[X] = prefs.fullScreenResolution[X];
	prefs.windowSize[Y] = prefs.fullScreenResolution[Y];
    }
    prefs.fullscreen ^= true;
    reshape();
}

void emptyState() {
    if (!state)
	state = new State;
    state->boardRotation = 0;
    for (int i = 0; i < 6; i++) 
	state->boardRotating[i] = false;
    state->boardZoom[0] = false;
    state->boardZoom[1] = false;
}

void quit() {
    quit(-1);
}

void quit(int status) {
    if (p_mode == pm_quit)
	return;
    p_mode = pm_quit;
    savePreferences();
    delete mainBoard;
    delete state;
    for (int i = 0; i < nBoardsTypes; i++) {
	for (int j = 0; j < nBoards[i]; j++)
	    delete boardPrototype[i][j];
	delete [] boardPrototype[i];
    }
    delete [] boardPrototype;
    delete [] nBoards;
    for (int i = 0; i < nResolutions; i++)
	delete resolutions[i];
    delete resolutions;
    if (status >= 0)
	exit(status);
}

bool keyPressed(SDL_keysym pressed, keys_t key) {
    return (prefs.SDLkeys[key].sym == pressed.sym &&
	    prefs.SDLkeys[key].mod == pressed.mod &&
	    ALLOWED_KEYS[key][networkState]);
}

void speedDown() {
    /*
    if (prefs.stepInterval < (0x1 << (NSPEEDS - 1))) {
	float percent = ((float)(timerLong % prefs.stepInterval) / 
			 (prefs.stepInterval + 1));
	prefs.stepInterval <<= 1;
	timerLong = (long)(percent * (prefs.stepInterval + 1));
    }
    */

    if (prefs.stepInterval == NSPEEDS - 1)
	return;

    //float percent = ((float)(timerLong % SPEEDS[prefs.stepInterval]) / 
    //(SPEEDS[prefs.stepInterval] + 1));
    prefs.stepInterval++;
    fineTuneSpeed = 0;
    timerLong = (long)(stepPercent * (SPEEDS[prefs.stepInterval] + 1));
    stepPercent = ((float)(timerLong % SPEEDS[prefs.stepInterval]) / 
		   (SPEEDS[prefs.stepInterval] + 1));
    
    if (networkState == net_server)
	sendSpeed();
    recentSpeedChange = true;
}

void speedUp() {
    /*
      if (prefs.stepInterval > 0x1 && (networkState == net_none || prefs.stepInterval > 2)) {
	float percent = ((float)(timerLong % prefs.stepInterval) / 
			 (prefs.stepInterval + 1));
	prefs.stepInterval >>= 1;
	timerLong = (long)(percent * (prefs.stepInterval + 1));
    }
    */

    if (prefs.stepInterval == 0) // || (networkState != net_none && prefs.stepInterval == 1))
	return;

    //float percent = ((float)(timerLong % SPEEDS[prefs.stepInterval]) / 
    //(SPEEDS[prefs.stepInterval] + 1));
    prefs.stepInterval--;
    fineTuneSpeed = 0;
    timerLong = (long)(stepPercent * (SPEEDS[prefs.stepInterval] + 1));
    stepPercent = ((float)(timerLong % SPEEDS[prefs.stepInterval]) / 
		   (SPEEDS[prefs.stepInterval] + 1));
    if (networkState == net_server)
	sendSpeed();
    recentSpeedChange = true;
}

char* keyString(keys_t k, char* str) {
    char* result = str;
    SDL_keysym key = prefs.SDLkeys[k];
    if (key.mod & KMOD_SHIFT) {
	strcpy(str, "S-");
	str += 2;
    }
    if (key.mod & KMOD_CTRL) {
	strcpy(str, "C-");
	str += 2;
    }
    if (key.mod & KMOD_ALT) {
	strcpy(str, "A-");
	str += 2;
    }
    if (key.mod & KMOD_META) {
	strcpy(str, "M-");
	str += 2;
    }
    
    //*str = key.sym;
    //*++str = 0;
    strncpy(str, SDL_GetKeyName(key.sym), 247);
    return result;
}

void globalTick(int times) {
    if (state->boardZoom[0]) {
	prefs.fieldOfView -= times * BOARD_ZOOM;
	if (prefs.fieldOfView < MIN_FIELD_OF_VIEW)
	    prefs.fieldOfView = MIN_FIELD_OF_VIEW;
	//reshape(prefs.windowSize[X], prefs.windowSize[Y]);
	loadPerspectives();
    }
    if (state->boardZoom[1]) {
	prefs.fieldOfView += times * BOARD_ZOOM;
	if (prefs.fieldOfView > MAX_FIELD_OF_VIEW)
	    prefs.fieldOfView = MAX_FIELD_OF_VIEW;
	//reshape(prefs.windowSize[X], prefs.windowSize[Y]);
	loadPerspectives();
    }

    if (state->boardRotating > 0) {
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
 
	if (state->boardRotating[left])
	    glRotatef(-BOARD_ROTATION_ANGLE * times,   0, 1.0,   0);
	if (state->boardRotating[right])
	    glRotatef( BOARD_ROTATION_ANGLE * times,   0, 1.0,   0);
	if (state->boardRotating[up])
	    glRotatef(-BOARD_ROTATION_ANGLE * times, 1.0,   0,   0); 
	if (state->boardRotating[down])
	    glRotatef( BOARD_ROTATION_ANGLE * times, 1.0,   0,   0); 
	if (state->boardRotating[forward])
	    glRotatef( BOARD_ROTATION_ANGLE * times,   0,   0, 1.0); // yaw left
	if (state->boardRotating[backward])
	    glRotatef(-BOARD_ROTATION_ANGLE * times,   0,   0, 1.0); // yaw right
       
	glMultMatrixf(prefs.boardRotationMatrix);
	glGetFloatv(GL_MODELVIEW_MATRIX, prefs.boardRotationMatrix);
    }
}

Uint32 timer(Uint32 interval, void* value) {
    
    //if ((int)value != gameModeValue)
    if ((int)value != p_mode)
	return 0;

    SDL_Event event;
    SDL_UserEvent userevent;

    userevent.type = SDL_USEREVENT;
    userevent.code = (int)value;
    userevent.data1 = NULL;
    userevent.data2 = NULL;

    event.type = SDL_USEREVENT;
    event.user = userevent;

    SDL_PushEvent(&event);
    
    return BASE_INTERVAL;
    return 0;
}

bool updateKeyPressed(SDL_keysym* key) {
    if (key->sym == SDLK_NUMLOCK   ||
	key->sym == SDLK_CAPSLOCK  ||
	key->sym == SDLK_SCROLLOCK ||
	key->sym == SDLK_RSHIFT	   ||
	key->sym == SDLK_LSHIFT	   ||
	key->sym == SDLK_RCTRL	   ||
	key->sym == SDLK_LCTRL	   ||
	key->sym == SDLK_RALT	   ||
	key->sym == SDLK_LALT	   ||
	key->sym == SDLK_RMETA	   ||
	key->sym == SDLK_LMETA	   ||
	key->sym == SDLK_LSUPER	   ||
	key->sym == SDLK_RSUPER)
	return false;

    if (key->mod | KMOD_NUM)     key->mod = (SDLMod)(key->mod & ~KMOD_NUM);
    if (key->mod == KMOD_RSHIFT) key->mod = KMOD_LSHIFT;
    if (key->mod == KMOD_RALT)   key->mod = KMOD_LALT;
    if (key->mod == KMOD_RCTRL)  key->mod = KMOD_LCTRL;
    if (key->mod == KMOD_RMETA)  key->mod = KMOD_LMETA;
    if (key->sym == SDLK_KP_ENTER)  key->sym = SDLK_RETURN;
    return true;
}

void step(int* oldLocation, direction dir, int* newLocation) {
    int newX = oldLocation[X];
    int newY = oldLocation[Y];
    int newZ = oldLocation[Z];
    switch (dir) {
        case left:      newX--; break;
        case right:     newX++; break;
        case down:      newY--; break;
        case up:        newY++; break;
        case forward:   newZ--; break;
        case backward:  newZ++; break;
    }
    newLocation[X] = newX;
    newLocation[Y] = newY;
    newLocation[Z] = newZ;
}

void resetNames() {
    strncpy(names[0], prefs.name, 20);
    strcpy(names[1], "Snake 2");
    strcpy(names[2], "Snake 3");
    strcpy(names[3], "Snake 4");
}


