/*  
 *  snake3d, a snake game in a 3d world.
 *  Copyright (C) 2005-2006 Yair Yarom
 *  
 *  This program is free software; you can redistribute it and/or modify it under
 *  the terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or (at your option) any later
 *  version.
 *  
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 *  PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 *  Place, Suite 330, Boston, MA 02111-1307 USA
 *  
 *  http://sourceforge.net/projects/worms3d
 *  
 */

#include <cmath>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>

#include "game.hpp"
#include "menu.hpp"
#include "global.hpp"
#include "functions.hpp"
#include "board.hpp"
#include "snake.hpp"
#include "preferences.hpp"
#include "network.hpp"

static inline void drawCompass() {

    glDisable(GL_LIGHTING);

    
    glTranslatef(0, 0, -1);
    glMultMatrixf(prefs.boardRotationMatrix);
    glScalef(0.7, 0.7, 0.7);
    
    glColor3fv(COLORS[YELLOW]);
    glBegin(GL_LINES);
    glVertex3d( 0,  0,  1);
    glVertex3d( 0,  0, -1);

    glVertex3d( 0,  1,  0);
    glVertex3d( 0, -1,  0);

    glVertex3d( 1,  0,  0);
    glVertex3d(-1,  0,  0);

    glEnd();

    glColor3fv(COLORS[BLUE]);
    //drawChar(-1.1, 0, 0, prefs.keys[k_moveLeft]); //keyMovement[left]);
    //drawChar( 1.1, 0, 0, prefs.keys[k_moveRight]); //keyMovement[right]);
    //drawChar(0, -1.1, 0, prefs.keys[k_moveDown]); //keyMovement[down]);
    //drawChar(0,  1.1, 0, prefs.keys[k_moveUp]); //keyMovement[up]);
    //drawChar(0, 0, -1.1, prefs.keys[k_moveForward]); //keyMovement[forward]);
    //drawChar(0, 0,  1.1, prefs.keys[k_moveBackward]); //keyMovement[backward]);
    char str[255];
    drawString(-1.1, 0, 0, keyString(k_moveLeft, str));
    drawString( 1.1, 0, 0, keyString(k_moveRight, str)); //keyMovement[right]);
    drawString(0, -1.1, 0, keyString(k_moveDown, str)); //keyMovement[down]);
    drawString(0,  1.1, 0, keyString(k_moveUp, str)); //keyMovement[up]);
    drawString(0, 0, -1.1, keyString(k_moveForward, str)); //keyMovement[forward]);
    drawString(0, 0,  1.1, keyString(k_moveBackward, str)); //keyMovement[backward]);
    
    glColor3fv(COLORS[YELLOW]);
    glTranslatef(0, 0, -0.7);
    glRotatef(180, 0, 1, 0);
    glutSolidCone(0.09, 0.29, 8, 1);
    glColor3fv(COLORS[BLACK]);
    glutWireCone(0.1, 0.3, 8, 1);
    
    glFlush();
    glEnable(GL_LIGHTING);
}

static inline void drawText() {
    
    glColor3fv(COLORS[BLUE]);
    //drawChar(  7, 300, prefs.keys[k_selfMoveLeft]); //keySelfMovement[left]);
    //drawChar( 97, 210, prefs.keys[k_selfMoveDown]); //keySelfMovement[down]);
    //drawChar(187, 300, prefs.keys[k_selfMoveRight]); //keySelfMovement[right]);
    //drawChar( 97, 385, prefs.keys[k_selfMoveUp]); //keySelfMovement[up]);
    //drawChar(  7, 210, prefs.keys[k_selfYawLeft]); //keySelfYaw[left]);
    //drawChar(187, 210, prefs.keys[k_selfYawRight]); //keySelfYaw[!left]);

    char str[255];
    drawString(  7, 300, keyString(k_selfMoveLeft, str)); //keySelfMovement[left]);
    drawString( 97, 210, keyString(k_selfMoveDown, str)); //keySelfMovement[down]);
    drawString(187, 300, keyString(k_selfMoveRight, str)); //keySelfMovement[right]);
    drawString( 97, 385, keyString(k_selfMoveUp, str)); //keySelfMovement[up]);
    drawString(  7, 210, keyString(k_selfYawLeft, str)); //keySelfYaw[left]);
    drawString(187, 210, keyString(k_selfYawRight, str)); //keySelfYaw[!left]);


    /*
      drawChar(207, windowSize[Y] / 2, keyBoardRotate[left]);
      drawChar(windowSize[X] / 2 + 100, 10, keyBoardRotate[down]);
      drawChar(windowSize[X] - 13, windowSize[Y] / 2, keyBoardRotate[right]);
      drawChar(windowSize[X] / 2 + 100, windowSize[Y] - 15, keyBoardRotate[up]);
      drawChar(207, 10, keyBoardYaw[left]);
      drawChar(windowSize[X] - 13, 10, keyBoardYaw[!left]);
    */

    if (paused) {
	glPushMatrix();
	int length = glutStrokeLength(STROKE_FONT, (unsigned char*)"PAUSE");
	glTranslatef(((prefs.windowSize[X] - 200) - (float)length / sqrt(2)) / 2 + 170, 
		     (prefs.windowSize[Y] + (float)length / sqrt(2)) / 2 - 30, 1);
	glRotatef(45, 0, 0, -1);
	glColor3fv(COLORS[RED]);
	//drawString(0, 0, 0, "PAUSE");
	drawStroke(0, 0, 0, "PAUSE");
	glPopMatrix();
    }

    int height = 1;
    glDisable(GL_LIGHTING);
    for (int i = 0; i < MAX_SNAKES; i++) {
	if (snakes[i]) {
	    glColor3fv(COLORS[snakes[i]->getColor()]);
	    //drawString(9, prefs.windowSize[Y] - height * 15, "Length: ");
	    //drawInt(glutBitmapLength(FONT, (unsigned char*)"Length: ") + 9, prefs.windowSize[Y] - height++ * 15, snakes[i]->getLength());
	    snprintf(str, 255, "%-10.10s %i", names[i], snakes[i]->getLength());
	    drawString(9, prefs.windowSize[Y] - height++ * 15, str);
	}
    }

    height++;
    glColor3fv(COLORS[BLACK]);
    drawString(9, prefs.windowSize[Y] - height++ * 15, "Speed:");
    drawString(9, prefs.windowSize[Y] - height++ * 15, SPEEDS_NAMES[prefs.stepInterval]);

    height++;
    glColor3fv(COLORS[GREEN]);
    int length = glutBitmapLength(FONT, (unsigned char*)"--- Auto pilot ---") / 2;
    if (player->getAI() == ai_safe)
	drawString(100 - length, prefs.windowSize[Y] - height * 15, "--- Auto pilot ---");

    if (prefs.showIndicator) {
	int nextLocation[3];
	player->headLocation(nextLocation);
	step(nextLocation, player->getDirection(), nextLocation);
	occupancy_t occupancy = mainBoard->checkLocation(nextLocation, player);
	char* oString;
	switch (occupancy) {
	  case o_empty:     oString = "      Clear      "; break;
	  case o_food:      oString = "      Food       "; break;
	  case o_snake:     oString = "***** SNAKE *****"; break;
	  case o_intercept: oString = "*** INTERCEPT ***"; break;
	  case o_outside:   oString = "**** OUTSIDE ****"; break;
	}
	
	if (deadEnd(occupancy))
	    glColor3fv(COLORS[RED]);
	else if (occupancy == o_food)
	    glColor3fv(COLORS[BLUE]);
	else
	    glColor3fv(COLORS[GREEN]);
	
	length = glutBitmapLength(FONT, (unsigned char*)oString) / 2;
	
    /*
      height += 2;
      drawString(100 - length, prefs.windowSize[Y] - height++ * 15, oString);
    */
	
	glBegin(GL_QUADS);
	glVertex3d(prefs.windowSize[X] / 2 + 95 - length, 32, -0.1);
	glVertex3d(prefs.windowSize[X] / 2 + 105 + length, 32, -0.1);
	glVertex3d(prefs.windowSize[X] / 2 + 105 + length, 10, -0.1);
	glVertex3d(prefs.windowSize[X] / 2 + 95 - length, 10, -0.1);
	glEnd();
	
	glColor3fv(COLORS[WHITE]);
	drawString(prefs.windowSize[X] / 2 + 100 - length, 15, oString);
	glEnable(GL_LIGHTING);
    }
}

/* display - here OpenGL should display everything to the window */
static inline void display() {
    
#ifdef FPS
    static char fpsString[100] = "";
    static struct timeval fpsTime;
    static int frames = 0;
    
    if (frames % FPS == 0) {
        struct timeval time2;
        gettimeofday(&time2, NULL);
        fpsTime.tv_sec = time2.tv_sec - fpsTime.tv_sec;
        fpsTime.tv_usec = time2.tv_usec - fpsTime.tv_usec;
        double passed = ((double)fpsTime.tv_sec * 1000000 + fpsTime.tv_usec) / (double)1000000;
        //sprintf(fpsString, "FPS: %f, seconds: %f, frames: %i: time: %li", (double)FPS / passed, passed, frames, fpsTime.tv_sec);
        snprintf(fpsString, 100, "FPS: %.2f", (double)FPS / passed);
        gettimeofday(&fpsTime, NULL);
    }
    frames++;
#endif
    
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    
    glLightfv(GL_LIGHT0, GL_POSITION, LIGHT_POSITION);
    
    // draw first person view
    glMatrixMode(GL_PROJECTION);
    glLoadMatrixf(firstPersonProjectionMatrix);
    glViewport(0, 200, 200, 200);
    glMatrixMode(GL_MODELVIEW) ;
    glLoadIdentity();
    drawboard(true);


    // draw compass
    glMatrixMode(GL_PROJECTION);
    glLoadMatrixf(compassProjectionMatrix);
    glViewport(0, 0, 200, 200);
    glMatrixMode(GL_MODELVIEW) ;
    glLoadIdentity();
    drawCompass();
                  
    // draw third person view
    glMatrixMode(GL_PROJECTION);
    glLoadMatrixf(thirdPersonProjectionMatrix);
    glViewport(200, 0, prefs.windowSize[X] - 200, prefs.windowSize[Y]);
    glMatrixMode(GL_MODELVIEW) ;
    glLoadIdentity();
    drawboard(false);

    // draw screen
    glMatrixMode(GL_PROJECTION);
    glLoadMatrixf(screenProjectionMatrix);
    glViewport(0, 0, prefs.windowSize[X], prefs.windowSize[Y]);
    glMatrixMode(GL_MODELVIEW) ;
    glLoadIdentity();
    drawScreen(true);
    drawText();

#ifdef FPS
    glDisable(GL_LIGHTING);
    glColor3fv(COLORS[LIGHT_GRAY]);
    drawString(210, prefs.windowSize[Y] - 12, fpsString);
    glEnable(GL_LIGHTING);
#endif
    
    glFlush();
    //glutSwapBuffers();
    SDL_GL_SwapBuffers();
    redisplay = false;
}


//static inline void keyboardUp(unsigned char key, int x, int y) {
static inline void keyboardUp(SDL_keysym key) {
    if (keyPressed(key, k_boardRotateLeft)) { //keyBoardRotate[left]) {
	state->boardRotation--;
	state->boardRotating[left] = false;
    } else if (keyPressed(key, k_boardRotateRight)) { //keyBoardRotate[right]) {
	state->boardRotation--;
	state->boardRotating[right] = false;
    } else if (keyPressed(key, k_boardRotateUp)) { //keyBoardRotate[up]) {
	state->boardRotation--;
	state->boardRotating[up] = false;
    } else if (keyPressed(key, k_boardRotateDown)) { //keyBoardRotate[down]) {
	state->boardRotation--;
	state->boardRotating[down] = false;
    } else if (keyPressed(key, k_boardYawLeft)) { //keyBoardYaw[left]) {
	state->boardRotation--;
	state->boardRotating[forward] = false;
    } else if (keyPressed(key, k_boardYawRight)) { //keyBoardYaw[!left]) {
	state->boardRotation--;
	state->boardRotating[backward] = false;
    } else if (keyPressed(key, k_zoomIn)) // keyZoomIn) 
	state->boardZoom[0] = false;
    else if (keyPressed(key, k_zoomOut)) //keyZoomOut)
	state->boardZoom[1] = false;
    //else if (key == keys[k_speedUp]) // keySpeedUp)
    //state->changeSpeed[0] = false;
    //else if (key == keys[k_speedDown]) //keySpeedDown)
    //state->changeSpeed[1] = false;
    redisplay = true;
}

static inline void keyboard(SDL_keysym key) {
    //int modifiers = glutGetModifiers();
      
    bool moved = false;
    for (int i = 0; i < 6; i++) {
	if (keyPressed(key, k_move[i])) {
	    if (networkState == net_client && sentTick) {
		SDL_Event event;
		event.key.type = SDL_KEYDOWN;
		event.key.state = SDL_PRESSED;
		event.key.keysym = key;
		SDL_PushEvent(&event);
		DEBUG_PRINT("pushed event\n");
		return;
	    }
	    //SDL_SemWait(stepLock);
	    if (networkState != net_none)
		lockStep("keys change dir");
	    player->changeDirection((direction)i);
	    //SDL_SemPost(stepLock);
	    if (networkState != net_none)
		unlockStep("end keys change dir");
	    moved = true;
	    break;
	}
    }
    if (!moved) {
	for (int i = 0; i < 4; i++) {
	    if (keyPressed(key, k_selfMove[i])) {
		if (networkState == net_client && sentTick) {
		    SDL_Event event;
		    event.key.type = SDL_KEYDOWN;
		    event.key.state = SDL_PRESSED;
		    event.key.keysym = key;
		    SDL_PushEvent(&event);
		    DEBUG_PRINT("pushed event\n");
		    return;
		}
		//SDL_SemWait(stepLock);
		if (networkState != net_none)
		    lockStep("keys change dir");
		player->changeSelfDirection((direction)i);
		//SDL_SemPost(stepLock);
		if (networkState != net_none)
		    unlockStep("end keys change dir");
		moved = true;
		break;
	    }
	}
    }
    
    if (moved) {
	if (networkState == net_none)
	    paused = false;
    } else {
	if (keyPressed(key, k_selfYaw[left])) { 
	    player->changeYaw(left);
	    if (networkState == net_none)
		paused = false;
	}
	else if (keyPressed(key, k_selfYaw[!left])) { 
	    player->changeYaw(right);
	    if (networkState == net_none)
		paused = false;
	}
	
	else if (keyPressed(key, k_resetSnakes)) {
	    if (networkState == net_server) {
		//SDL_SemWait(stepLock);
		lockStep("keys reset");
		reseting = true;
		for (int i = 0; i < MAX_SNAKES; i++)
		    recvReset[i] = false;
		sendReset();
		mainBoard->resetSnakes();
		//SDL_SemPost(stepLock);
		unlockStep("end keys reset");
	    } else
		mainBoard->resetSnakes();
	}
	
	else if (keyPressed(key, k_resetBoard)) {
	    defaultRotation(true);
	    //glMatrixMode(GL_MODELVIEW);
	    //glLoadIdentity();
	    //glGetFloatv(GL_MODELVIEW_MATRIX, prefs.boardRotationMatrix);
	    //prefs.fieldOfView = INIT_FIELD_OF_VIEW;
	    //reshape(windowSize[X], windowSize[Y]);
	    //stepInterval = DEFAULT_STEP_INTERVAL;
	} else if (keyPressed(key, k_boardRotate[left])) {
	    state->boardRotation++;
	    state->boardRotating[left] = true;
 	} else if (keyPressed(key, k_boardRotate[right])) {
	    state->boardRotation++;
	    state->boardRotating[right] = true;
	} else if (keyPressed(key, k_boardRotate[up])) {
	    state->boardRotation++;
	    state->boardRotating[up] = true;
	} else if (keyPressed(key, k_boardRotate[down])) {
	    state->boardRotation++;
	    state->boardRotating[down] = true;
	} else if (keyPressed(key, k_boardYaw[left])) {
	    state->boardRotation++;
	    state->boardRotating[forward] = true;
	} else if (keyPressed(key, k_boardYaw[!left])) {
	    state->boardRotation++;
	    state->boardRotating[backward] = true;
	
	} else if (keyPressed(key, k_zoomIn)) 
	    state->boardZoom[0] = true;
	else if (keyPressed(key, k_zoomOut))
	    state->boardZoom[1] = true;
	
	else if (keyPressed(key, k_autoPilot))
	    player->autoPilot(ai_switch, ait_food, 0);
	else if (keyPressed(key, k_grid1))
	    prefs.gridLevel1 = (prefs.gridLevel1 == 3 ? 0 : prefs.gridLevel1 + 1);
	else if (keyPressed(key, k_grid3))
	    prefs.gridLevel3 = (prefs.gridLevel3 == 3 ? 0 : prefs.gridLevel3 + 1);
	else if (keyPressed(key, k_backgroundColor)) {
	    prefs.backgroundColor++;
	    prefs.backgroundColor %= NCOLORS;
	    glClearColor(COLORS[prefs.backgroundColor][RED], COLORS[prefs.backgroundColor][GREEN], COLORS[prefs.backgroundColor][BLUE], 0.0);
	} else if (keyPressed(key, k_speedUp)) 
	    speedUp();
	else if (keyPressed(key, k_speedDown))
	    speedDown();
	else if (keyPressed(key, k_showWalls))
	    prefs.showWalls ^= true;
	/*else if (keyPressed(key, k_lighting)) {
	  lightingEnabled = !lightingEnabled;
	  lighting(true);
	  }*/
	else if (keyPressed(key, k_fullScreen)) {
	    switchFullScreen();
	    if (!paused) {
		paused = true;
		sendPause(true);
	    }
	}
	
	else if (keyPressed(key, k_quit)) {
	    quit(0);
	}
	else if (keyPressed(key, k_pause)) {
	    paused ^= true;
	    sendPause(paused);
	} else if (keyPressed(key, k_abort)) {
	    //menuCallbacks(true);
	    p_mode = pm_menu;
	} else if (keyPressed(key, k_indicator)) {
	    prefs.showIndicator ^= true;
	}
    }
    redisplay = true;
}


static inline void tick(int times) {
    if (p_mode != pm_game)// || value != gameModeValue)
	return;

    globalTick(times);

    if (!paused) {
	switch (networkState) {
	  case net_none:
	      for (int i = 0; i < times; i++) {
		  timerLong++;
		  stepPercent = ((float)(timerLong % SPEEDS[prefs.stepInterval]) / 
				 (SPEEDS[prefs.stepInterval] + 1));
		  if (timerLong % SPEEDS[prefs.stepInterval] == 0) {
		      mainBoard->step();
		  }
	      }
	      break;

	  case net_client:
	      static unsigned char tempFineTuneSpeed = 0;
	      if (sentTick && !recvTick) {
		  tempFineTuneSpeed += times;
		  //printf("sent with no reply\n");
		  break;
	      }
	      for (int i = 0; i < times; i++) {
		  timerLong++;
		  if (timerLong % (SPEEDS[prefs.stepInterval] + fineTuneSpeed) == 0) {
		      if (sentTick) {
			  if (recvTick) {
			      timerLong = 0;
			      if (recentSpeedChange) {
				  DEBUG_PRINT("recent speed change => not fine tuning\n");
			      } else if (prefs.fineTune) {
				  fineTuneSpeed += tempFineTuneSpeed / 2;
				  if (SPEEDS[prefs.stepInterval] + fineTuneSpeed < 1)
				      fineTuneSpeed = 1 - SPEEDS[prefs.stepInterval];
				  if (fineTuneSpeed > MAX_FINE_TUNE)
				      fineTuneSpeed = MAX_FINE_TUNE;
				  else if (fineTuneSpeed < - MAX_FINE_TUNE)
				      fineTuneSpeed = - MAX_FINE_TUNE;
				  DEBUG_PRINT2("fine tune speed: %i\n", fineTuneSpeed);
			      }
			      recentSpeedChange = false;
			      tempFineTuneSpeed = 0;
			      recvTick = false;
			      sentTick = false;
			  } else {
			      tempFineTuneSpeed++;
			      timerLong--;
			  }
		      } else {
			  lockStep("change dir before end");
			  player->changeDirection();
			  unlockStep("end change dir before end");
			  sendTick();
			  timerLong--;
		      }
		      break;
		  }
		  stepPercent = ((float)(timerLong % (SPEEDS[prefs.stepInterval] + fineTuneSpeed)) / 
				 (SPEEDS[prefs.stepInterval] + fineTuneSpeed + 1));
	      }
	      break;

	  case net_server:
	      static unsigned char clientFineTuneSpeed[MAX_SNAKES] = {0, 0, 0, 0};
	      for (int i = 0; i < times; i++) {
		  timerLong++;
		  /*if (timerLong % SPEEDS[prefs.stepInterval] == SPEEDS[prefs.stepInterval] - 1) {
		      //SDL_SemWait(stepLock);
		      lockStep("change dir before tick");
		      for (int i = 0; i < MAX_SNAKES; i++)
			  if (snakes[i])
			      snakes[i]->changeDirection();
		      //SDL_SemPost(stepLock);
		      unlockStep("end change dir before tick");
		  } else */
		  
		  if (timerLong % SPEEDS[prefs.stepInterval] == 0) {
		      bool go = true;
		      for (int i = 0; i < MAX_SNAKES; i++) {
			  //go = go && (recvTicks[i] || prefs.ai[i] != ai_remote);
			  if (!recvTicks[i] && prefs.ai[i] == ai_remote) {
			      go = false;
			      clientFineTuneSpeed[i]++;
			  }
		      }
		      if (reseting && go) {
			  for (int i = 0; i < MAX_SNAKES; i++)
			      go = go && (recvReset[i] || prefs.ai[i] != ai_remote);
		      }
		      if (go) {
			  reseting = false;

			  lockStep("change dir before tick + step");
			  for (int i = 0; i < MAX_SNAKES; i++)
			      if (snakes[i])
				  snakes[i]->changeDirection();
			  //SDL_SemPost(stepLock);

			  sendTick();
			  mainBoard->step();
			  //SDL_mutexV(stepLock);
                          //SDL_SemPost(stepLock);
			  unlockStep("end step");
			  times = 0;
			  for (int i = 0; i < MAX_SNAKES; i++) {
			      recvTicks[i] = false;
			      if (recentSpeedChange) {
				  DEBUG_PRINT("recent speed change => not fine tuning\n");
			      } else if (prefs.fineTune) {
				  if (clientFineTuneSpeed[i] > FINE_TUNE_SPEED_THRESHOLD) {
				      clientFineTuneSpeed[i] /= 2;
				      DEBUG_PRINT3("send fine tune %i to %i\n", clientFineTuneSpeed[i], i);
				      sendFineTuneSpeed(i, clientFineTuneSpeed[i]);
				  }
			      }
			      clientFineTuneSpeed[i] = 0;
			      recentSpeedChange = false;
			  }
		      } else {
			  timerLong--;
			  break;
		      }
		  }
		  stepPercent = ((float)(timerLong % SPEEDS[prefs.stepInterval]) / 
				 (SPEEDS[prefs.stepInterval] + 1));
	      }
	      break;

	}
    }

    redisplay = true;
}

void gameLoop() {
    emptyState();

    SDL_Event event;
    redisplay = true;

    if (networkState == net_client) {
	sentTick = false;
	recvTick = false;
    }
    reseting = false;
    fineTuneSpeed = 0;
    
    //SDL_AddTimer(BASE_INTERVAL, timer, (void*)++gameModeValue);
    SDL_AddTimer(BASE_INTERVAL, timer, (void*)p_mode);
    /*gameModeValue++;*/
    //bool addTimer = true;

    for (int i = 0; i < MAX_SNAKES; i++) 
	if (snakes[i])
	    snakes[i]->changeDirection();
    
    while (p_mode == pm_game) {

	if (redisplay) {
            switch (networkState) {
              case net_none:
                  display();
                  break;

              case net_client:
                  if (sentTick && !paused)
                      break;

              case net_server:
                  lockStep("start display");
                  display();
                  unlockStep("end display");
            }
	}

	/*
	if (addTimer) {
	    SDL_AddTimer(BASE_INTERVAL, timer, (void*)gameModeValue);
	    addTimer = false;
	}
	*/
	
	SDL_WaitEvent(&event);
	
	switch (event.type) {

	  case SDL_ACTIVEEVENT:
	      if (event.active.gain == 0 && event.active.state != SDL_APPMOUSEFOCUS) {
		  paused = true;
		  sendPause(paused);
	      }
	      break;
	      
	  case SDL_VIDEORESIZE:
	      prefs.windowSize[X] = event.resize.w;
	      prefs.windowSize[Y] = event.resize.h;
	      reshape();
	      break;
	      
	  case SDL_KEYDOWN:
	      if (updateKeyPressed(&event.key.keysym))
		  keyboard(event.key.keysym);
	      break;
	      
	  case SDL_KEYUP:
	      updateKeyPressed(&event.key.keysym);
	      keyboardUp(event.key.keysym);
	      break;
 
	  case SDL_USEREVENT:
	      if (event.user.code == pm_game) {
		  int times = 1;
		  SDL_Event e;
		  while (SDL_PeepEvents(&e, 1, SDL_PEEKEVENT, SDL_EVENTMASK(SDL_USEREVENT))) {
		      if (e.user.code == pm_game) {
			  times++;
			  SDL_PeepEvents(&e, 1, SDL_GETEVENT, SDL_EVENTMASK(SDL_USEREVENT));
		      } else
			  break;
		  }
		  tick(times);
		  //addTimer = true;
	      }
	      break;
	      
	  case SDL_QUIT:
	      quit(0);
	      break;
	}

	if (networkError)
	    p_mode = pm_menu;
    }
}
