/*  
 *  snake3d, a snake game in a 3d world.
 *  Copyright (C) 2005-2006 Yair Yarom
 *  
 *  This program is free software; you can redistribute it and/or modify it under
 *  the terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or (at your option) any later
 *  version.
 *  
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 *  PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 *  Place, Suite 330, Boston, MA 02111-1307 USA
 *  
 *  http://sourceforge.net/projects/worms3d
 *  
 */

#include <string.h>
#include <stdlib.h>

#include "menu.hpp"
#include "game.hpp"
#include "global.hpp"
#include "snake.hpp"
#include "board.hpp"
#include "cube.hpp"
#include "functions.hpp"
#include "preferences.hpp"
#include "network.hpp"
#include "userInput.hpp"

typedef enum {
    m_start,
    m_keys,
    m_worlds,
    m_snakes,
    m_settings,
    m_network,
    //m_about,
    m_quit
} menu_action;

typedef enum {
    s_fullScreen,
    s_resolution,
    s_resetKeys,
    s_resetSettings,
    s_grid1,
    s_grid3,
    s_walls,
    s_background,
    s_speed,
    s_resetBoard
} settings_t;

typedef enum {
    st_name,
    st_ai,
    st_aiTarget,
    st_aitSnake,
    st_color
} snakes_t;

const int MENU_SIZE = 7;
const char* MENU[MENU_SIZE] = {
    "Start",
    "Keys",
    "Worlds",
    "Snakes",
    "Settings",
    "Network",
    //"About",
    "Quit"
};

const int NSETTINGS = 10;

const settings_t SETTINGS_MENU_MOVEMENT_UP[NSETTINGS] = {
    s_resetSettings, // s_fullScreen,
    s_fullScreen,    // s_resolution,
    s_resetBoard,    // s_resetKeys,
    s_resetKeys,     // s_resetSettings,
    s_speed,	     // s_grid1,
    s_grid1,         // s_grid3,
    s_resolution,    // s_walls,
    s_grid3,         // s_background,
    s_walls,         // s_speed
    s_background     // s_resetBoard
};

const settings_t SETTINGS_MENU_MOVEMENT_DOWN[NSETTINGS] = {
    s_resolution,    // s_fullScreen,
    s_walls,         // s_resolution,
    s_resetSettings, // s_resetKeys,
    s_fullScreen,    // s_resetSettings,
    s_grid3,         // s_grid1,
    s_background,    // s_grid3,
    s_speed,         // s_walls,
    s_resetBoard,    // s_background,
    s_grid1,         // s_speed
    s_resetKeys      // s_resetBoard http://revansh.com/odds.php?line=2
};

const int N_NET_MENUS[] = {2, 3, 4}; // net_none, net_server, net_client

bool rebuildBoard = false;

bool insideMenu = false;
int currentMenu = 0;
//int currentBoard = DEFAULT_BOARD;

bool insideKey = false;
int currentKey = -1;

int currentSetting = -1;

int currentSnake = 0;
int currentSnakePrefs = -1;

int currentNetMenu = -1;
netState_t currentNetSelect = networkState;

inline void rebuildBoardFunc() {
    rebuildBoard = false;
    
    if (p_mode != pm_menu)
	return;
    
    if (mainBoard)
	delete mainBoard;

    mainBoard = boardPrototype[prefs.currentBoardType][prefs.currentBoard]->clone();
    
    /* players */
    //Snake* snakes[MAX_SNAKES];
    for (int i = 0; i < MAX_SNAKES; i++)
	snakes[i] = NULL;
    for (int i = 0; i < MAX_SNAKES; i++) {
	if (prefs.ai[i] != ai_off && !(prefs.ai[i] == ai_remote && (remoteSnakes[i] == ai_off || remoteSnakes[i] == ai_remote))) {
	    snakes[i] = mainBoard->addSnake(prefs.snakeColor[i], i);
	}
    }
    for (int i = 0; i < MAX_SNAKES; i++) {
	if (snakes[i]) {
	    if (snakes[prefs.aitSnake[i]])
		snakes[i]->autoPilot(prefs.ai[i], prefs.aiTarget[i], snakes[prefs.aitSnake[i]]);
	    else
		snakes[i]->autoPilot(prefs.ai[i], prefs.aiTarget[i], snakes[0]);
	}
    }
    
    player = NULL;
    switch (networkState) {
      case net_server:
	  sendBoardConfig();
	  
      case net_none:
	  player = snakes[0];
	  break;
	  
      case net_client:
	  for (int i = 0; i < MAX_SNAKES; i++)
	      if (prefs.ai[i] == ai_human) {
		  player = snakes[i];
		  //playerID = i;
		  break;
	      }
	  mainBoard->replaceFood(clientFood);
	  break;
    }
    player->autoPilot(ai_safe, ait_food, 0); /*disable human1*/
}

static inline void activateMenu() {
    insideMenu = true;
    switch (currentMenu) {
      case m_start:
	  switch (networkState) {
	    case net_client:
		sendStart();
		break;

	    case net_none:
		p_mode = pm_game;
		break;

	    case net_server:
		bool go = true;
		for (int i = 0; i < MAX_SNAKES; i++)
		    go = go && (recvTicks[i] || prefs.ai[i] != ai_remote);
		if (go) {
		    p_mode = pm_game;
		    sendStart();
		}
		insideMenu = false;
		break;

	  }
	  break;

      case m_keys:
	  currentKey = k_moveLeft;
	  break;
      case m_settings:
	  currentSetting = s_fullScreen;
	  break;
      case m_snakes:
	  if (networkState == net_client) {
	      int i;
	      for (i = 0; i < MAX_SNAKES; i++)
		  if (prefs.ai[i] == ai_human) break;
	      currentSnake = i;
	  } else {
	      currentSnake = 0;
	  }
	  currentSnakePrefs = st_color;
	  break;
      case m_network:
	  switch (networkState) {
	    case net_none:   currentNetMenu = 0; break;
	    case net_server: currentNetMenu = 2; break;
	    case net_client: currentNetMenu = 3; break;
	  }
	  currentNetSelect = networkState;
	  break;
      case m_worlds:
	  if (networkState == net_client)
	      insideMenu = false;
	  break;
	  /*
      case m_about:
	  insideMenu = false;
	  break;
	  */
      case m_quit:
	  quit(0);
	  break;
    }
}

static inline void deactivateMenu() {
    insideMenu = false;
    switch (currentMenu) {
      case m_start:
	  if (networkState == net_client)
	      sendStop();
	  break;
      case m_keys:
	  savePreferences();
	  currentKey = -1;
	  break;
      case m_worlds:
	  savePreferences();
	  break;
      case m_settings:
	  savePreferences();
	  currentSetting = -1;
	  break;
      case m_network:
	  savePreferences();
	  currentNetSelect = networkState;
	  currentNetMenu = -1;
	  break;
      case m_snakes:
	  currentSnake = 0;
	  currentSnakePrefs = -1;
	  if (networkState != net_client)
	      rebuildBoard = true;
	  savePreferences();
	  break;
	  //case m_about:
      case m_quit:
	  ;
    }
}

static inline void drawMenu() {
    /*
      glColor3fv(COLORS[CYAN]);
      int length = glutBitmapLength(FONT, (unsigned char*)"Menu");
      length = (200 - length) / 2;
      drawString(length, windowSize[Y] - 40, "Menu");
      
      glBegin(GL_LINES);
      glVertex2d(length - 10, windowSize[Y] - 45);
      glVertex2d(200 - length + 10, windowSize[Y] - 45);
      glEnd();
    */

    for (int i = 0; i < MENU_SIZE; i++) {
	if (i == currentMenu)
	    glColor3fv(COLORS[(insideMenu? MENU_INSIDE_COLOR : MENU_SELECT_COLOR)]);
	else
	    glColor3fv(COLORS[MENU_COLOR]);
	int length = glutBitmapLength(FONT, (unsigned char*)MENU[i]);
	drawString((200 - length) / 2 , prefs.windowSize[Y] - 70 - 20 * i, MENU[i]);
    }
    
    if (currentMenu == m_start) {
	glColor3fv(COLORS[BLACK]);
	drawString(9, prefs.windowSize[Y] - 90 - 20 * MENU_SIZE, "Speed:");

	/*
	int i = -1;
	int step = prefs.stepInterval;
	while (step) {
	    step >>= 1;
	    i++;
	}
	*/
	
	drawString(9, prefs.windowSize[Y] - 90 - 20 * (MENU_SIZE + 1), SPEEDS_NAMES[prefs.stepInterval]);
    }
}

static inline void keyMenuColor(keys_t key) {
    int color;
    if (key == currentKey) {
	if (insideKey)
	    color = MENU_INSIDE_COLOR;
	else
	    color = MENU_SELECT_COLOR;
    } else
	color = MENU_COLOR;
    glColor3fv(COLORS[color]);
}

static inline void drawKey(int x, int y, keys_t k) {
    char str[255] = ": ";
    keyString(k, str + 2);
    drawString(x, y, str);
}

static inline void drawKeys() {
    static const int INDENT = 10;
    static const int MOVEMENT1[3] = {10 , 20 , 100};
    static const int MOVEMENT2[3] = {310, 20 , 405};
    static const int BOARD[3]     = {10 , 180, 110};
    static const int VIEW[3]      = {310, 180, 525};
    static const int GAME[3]      = {10 , 380, 145};
    static const int OTHERS[3]    = {310, 380, 375};
    
    // movement global view
    glColor3fv(COLORS[MENU_HEADER_COLOR]);
    drawString(MOVEMENT1[X], prefs.windowSize[Y] - MOVEMENT1[Y], "Movement (Global view)");
    for (int i = 0; i < 6; i++) {
	keyMenuColor(k_move[i]);
	drawString(MOVEMENT1[X] + INDENT, prefs.windowSize[Y] - MOVEMENT1[Y] - (i + 1) * 20, KEY_NAME[k_move[i]]);
	drawKey(MOVEMENT1[Z], prefs.windowSize[Y] - MOVEMENT1[Y] - (i + 1) * 20, k_move[i]);
    }

    // movement snake view
    glColor3fv(COLORS[MENU_HEADER_COLOR]);
    drawString(MOVEMENT2[X], prefs.windowSize[Y] - MOVEMENT2[Y], "Movement (snake view)");
    for (int i = 0; i < 4; i++) {
	keyMenuColor(k_selfMove[i]);
	drawString(MOVEMENT2[X] + INDENT, prefs.windowSize[Y] - MOVEMENT2[Y] - (i + 1) * 20, KEY_NAME[k_selfMove[i]]);
	drawKey(MOVEMENT2[Z], prefs.windowSize[Y] - MOVEMENT2[Y] - (i + 1) * 20, k_selfMove[i]);
    }

    // movement yaw
    for (int i = 0; i < 2; i++) {
	keyMenuColor(k_selfYaw[i]);
	drawString(MOVEMENT2[X] + INDENT, prefs.windowSize[Y] - MOVEMENT2[Y] - (i + 5) * 20, KEY_NAME[k_selfYaw[i]]);
	drawKey(405, prefs.windowSize[Y] - MOVEMENT2[Y] - (i + 5) * 20, k_selfYaw[i]);
    }

    // board rotation
    glColor3fv(COLORS[MENU_HEADER_COLOR]);
    drawString(BOARD[X], prefs.windowSize[Y] - BOARD[Y], "Board Rotation");
    for (int i = 0; i < 4; i++) {
	keyMenuColor(k_boardRotate[i]);
	drawString(BOARD[X] + INDENT, prefs.windowSize[Y] - BOARD[Y] - (i + 1) * 20, KEY_NAME[k_boardRotate[i]]);
	drawKey(BOARD[Z], prefs.windowSize[Y] - BOARD[Y] - (i + 1) * 20, k_boardRotate[i]);
    }

    // board rotation yaw
    for (int i = 0; i < 2; i++) {
	keyMenuColor(k_boardYaw[i]);
	drawString(BOARD[X] + INDENT, prefs.windowSize[Y] - BOARD[Y] - (i + 5) * 20, KEY_NAME[k_boardYaw[i]]);
	drawKey(BOARD[Z], prefs.windowSize[Y] - BOARD[Y] - (i + 5) * 20, k_boardYaw[i]);
    }

    // viewing
    glColor3fv(COLORS[MENU_HEADER_COLOR]);
    drawString(VIEW[X], prefs.windowSize[Y] - VIEW[Y], "View");

    // viewing - zoom
    keyMenuColor(k_zoomIn);
    drawString(VIEW[X] + INDENT, prefs.windowSize[Y] - VIEW[Y] - 20, KEY_NAME[k_zoomIn]);
    drawKey(VIEW[Z], prefs.windowSize[Y] - VIEW[Y] - 20, k_zoomIn);
    keyMenuColor(k_zoomOut);
    drawString(VIEW[X] + INDENT, prefs.windowSize[Y] - VIEW[Y] - 40, KEY_NAME[k_zoomOut]);
    drawKey(VIEW[Z], prefs.windowSize[Y] - VIEW[Y] - 40, k_zoomOut);
    
    // viewing - walls/full screen/background
    keyMenuColor(k_showWalls);
    drawString(VIEW[X] + INDENT, prefs.windowSize[Y] - VIEW[Y] - 60, KEY_NAME[k_showWalls]);
    drawKey(VIEW[Z], prefs.windowSize[Y] - VIEW[Y] - 60, k_showWalls);
    keyMenuColor(k_fullScreen);
    drawString(VIEW[X] + INDENT, prefs.windowSize[Y] - VIEW[Y] - 80, KEY_NAME[k_fullScreen]);
    drawKey(VIEW[Z], prefs.windowSize[Y] - VIEW[Y] - 80, k_fullScreen);
    keyMenuColor(k_backgroundColor);
    drawString(VIEW[X] + INDENT, prefs.windowSize[Y] - VIEW[Y] - 100, KEY_NAME[k_backgroundColor]);
    drawKey(VIEW[Z], prefs.windowSize[Y] - VIEW[Y] - 100, k_backgroundColor);

    // viewing - grids
    keyMenuColor(k_grid1);
    drawString(VIEW[X] + INDENT, prefs.windowSize[Y] - VIEW[Y] - 120, KEY_NAME[k_grid1]);
    drawKey(VIEW[Z], prefs.windowSize[Y] - VIEW[Y] - 120, k_grid1);
    keyMenuColor(k_grid3);
    drawString(VIEW[X] + INDENT, prefs.windowSize[Y] - VIEW[Y] - 140, KEY_NAME[k_grid3]);
    drawKey(VIEW[Z], prefs.windowSize[Y] - VIEW[Y] - 140, k_grid3);

    // viewing - indicator
    keyMenuColor(k_indicator);
    drawString(VIEW[X] + INDENT, prefs.windowSize[Y] - VIEW[Y] - 160, KEY_NAME[k_indicator]);
    drawKey(VIEW[Z], prefs.windowSize[Y] - VIEW[Y] - 160, k_indicator);
    
    // game play
    glColor3fv(COLORS[MENU_HEADER_COLOR]);
    drawString(10, prefs.windowSize[Y] - GAME[Y], "Game play");
    keyMenuColor(k_autoPilot);
    drawString(GAME[X] + INDENT, prefs.windowSize[Y] - GAME[Y] - 20, KEY_NAME[k_autoPilot]);
    drawKey(GAME[Z], prefs.windowSize[Y] - GAME[Y] - 20, k_autoPilot);
    keyMenuColor(k_resetBoard);
    drawString(GAME[X] + INDENT, prefs.windowSize[Y] - GAME[Y] - 40, KEY_NAME[k_resetBoard]);
    drawKey(GAME[Z], prefs.windowSize[Y] - GAME[Y] - 40, k_resetBoard);
    keyMenuColor(k_resetSnakes);
    drawString(GAME[X] + INDENT, prefs.windowSize[Y] - GAME[Y] - 60, KEY_NAME[k_resetSnakes]);
    drawKey(GAME[Z], prefs.windowSize[Y] - GAME[Y] - 60, k_resetSnakes);
    keyMenuColor(k_speedUp);
    drawString(GAME[X] + INDENT, prefs.windowSize[Y] - GAME[Y] - 80, KEY_NAME[k_speedUp]);
    drawKey(GAME[Z], prefs.windowSize[Y] - GAME[Y] - 80, k_speedUp);
    keyMenuColor(k_speedDown);
    drawString(GAME[X] + INDENT, prefs.windowSize[Y] - GAME[Y] - 100, KEY_NAME[k_speedDown]);
    drawKey(GAME[Z], prefs.windowSize[Y] - GAME[Y] - 100, k_speedDown);

    // others 
    glColor3fv(COLORS[MENU_HEADER_COLOR]);
    drawString(OTHERS[X], prefs.windowSize[Y] - OTHERS[Y], "Others");
    keyMenuColor(k_pause);
    drawString(OTHERS[X] + INDENT, prefs.windowSize[Y] - OTHERS[Y] - 20, KEY_NAME[k_pause]);
    drawKey(OTHERS[Z], prefs.windowSize[Y] - OTHERS[Y] - 20, k_pause);

    keyMenuColor(k_abort);
    drawString(OTHERS[X] + INDENT, prefs.windowSize[Y] - OTHERS[Y] - 40, KEY_NAME[k_abort]);
    drawKey(OTHERS[Z], prefs.windowSize[Y] - OTHERS[Y] - 40, k_abort);
    
    keyMenuColor(k_quit);
    drawString(OTHERS[X] + INDENT, prefs.windowSize[Y] - OTHERS[Y] - 60, KEY_NAME[k_quit]);
    drawKey(OTHERS[Z], prefs.windowSize[Y] - OTHERS[Y] - 60, k_quit);
}

static inline void drawAbout() {

    // about
    glColor3fv(COLORS[ABOUT_COLOR]);
    int height = 2;
    //int length = glutBitmapLength(FONT, (const unsigned char*)"About");
    //drawString((prefs.windowSize[X] - 200 - length) / 2, prefs.windowSize[Y] - 20 * height++, "About");
    char version[80];
    strcpy(version, "Snake3d ");
    strcpy(version + 8, VERSION);
    int length = glutBitmapLength(FONT, (const unsigned char*)version);
    drawString((prefs.windowSize[X] - 200 - length) / 2, prefs.windowSize[Y] - 20 * height++, version);

    height++;

    drawString(30, prefs.windowSize[Y] - 20 * height++, "snake3d is a variant of the snake game. You are a snake in a");
    drawString(30, prefs.windowSize[Y] - 20 * height++, "3d world trying to eat as many icosahedrons as possible,    ");
    drawString(30, prefs.windowSize[Y] - 20 * height++, "which makes your tail longer. There are other snakes in the ");
    drawString(30, prefs.windowSize[Y] - 20 * height++, "world which will try to fail you.                           ");
    height++;
    drawString(30, prefs.windowSize[Y] - 20 * height++, "When the game starts, your snake is in auto-pilot mode (i.e.");
    drawString(30, prefs.windowSize[Y] - 20 * height++, "it will try to avoid walls). When you want to take over,    ");
    drawString(30, prefs.windowSize[Y] - 20 * height++, "simply press a movement key and you'll gain control of you  ");
    drawString(30, prefs.windowSize[Y] - 20 * height++, "snake.                                                      ");
    height++;
    
    // line
    glBegin(GL_LINES);

    glColor3fv(COLORS[LIGHT_GRAY]);
    glVertex3d(1, prefs.windowSize[Y] - 20 * height + 11, 0.5);
    glVertex3d(prefs.windowSize[X] - 200, prefs.windowSize[Y] - 20 * height + 11, 0.5);

    glColor3fv(COLORS[GRAY]);
    glVertex3d(0, prefs.windowSize[Y] - 20 * height + 10, 0.5);
    glVertex3d(prefs.windowSize[X] - 200, prefs.windowSize[Y] - 20 * height + 10, 0.5);

    glColor3fv(COLORS[DARK_GRAY]);
    glVertex3d(2, prefs.windowSize[Y] - 20 * height + 9, 0.5);
    glVertex3d(prefs.windowSize[X] - 200, prefs.windowSize[Y] - 20 * height + 9, 0.5);

    glEnd();

    // credits
    /*
    glColor3fv(COLORS[ABOUT_COLOR]);
    height++;
    length = glutBitmapLength(FONT, (const unsigned char*)"Credits");
    drawString((prefs.windowSize[X] - 200 - length) / 2, prefs.windowSize[Y] - 20 * height++, "Credits");
    height++;
    
    const int CREDITS = 1;
    const char* lines[CREDITS * 2] = {
	"Programmer", "Yair Yarom"//,
	//"Sound FX", "TBD",
	//"Music", "TBD",
	//"Cross Platform integration", "TBD"
    };
    int minWidth = glutBitmapWidth(FONT, '-');
    
    char line[120];
    for (int i = 0; i < CREDITS; i++) {
	strcpy(line, lines[2*i]);
	length = glutBitmapLength(FONT, (const unsigned char*)line);
	drawString((prefs.windowSize[X] - 200 - minWidth) / 2 - 7 - length, prefs.windowSize[Y] - 20 * height, line);
	drawChar((prefs.windowSize[X] - 200 - minWidth) / 2, prefs.windowSize[Y] - 20 * height, '-');
	strcpy(line, lines[2*i + 1]);
	drawString((prefs.windowSize[X] - 200 + minWidth) / 2 + 7, prefs.windowSize[Y] - 20 * height++, line);
    }

    height += 9;
    drawString(30, prefs.windowSize[Y] - 20 * height++,"(Due to recent cutbacks, there were no more credits to give)");
    */
    
    glColor3fv(COLORS[ABOUT_COLOR]);
    height++;
    length = glutBitmapLength(FONT, (const unsigned char*)"Empty space");
    drawString((prefs.windowSize[X] - 200 - length) / 2, prefs.windowSize[Y] - 20 * height++, "Empty space");    
    length = glutBitmapLength(FONT, (const unsigned char*)"End of empty space");
    drawString((prefs.windowSize[X] - 200 - length) / 2, 50, "End of empty space");
        
    // line
    glBegin(GL_LINES);

    glColor3fv(COLORS[LIGHT_GRAY]);
    glVertex3d(1, 31, 0.5);
    glVertex3d(prefs.windowSize[X] - 200, 31, 0.5);

    glColor3fv(COLORS[GRAY]);
    glVertex3d(0, 30, 0.5);
    glVertex3d(prefs.windowSize[X] - 200, 30, 0.5);

    glColor3fv(COLORS[DARK_GRAY]);
    glVertex3d(2, 29, 0.5);
    glVertex3d(prefs.windowSize[X] - 200, 29, 0.5);

    glEnd();

    glColor3fv(COLORS[ABOUT_COLOR]);
    length = glutBitmapLength(FONT, (const unsigned char*)"Programmed by Yair Yarom <irush@users.sourceforge.net>");
    drawString((prefs.windowSize[X] - 200 - length) / 2, 10, "Programmed by Yair Yarom <irush@users.sourceforge.net>");
}

static inline void settingMenuColor(settings_t setting) {
    int color;
    if (setting == currentSetting)
	color = MENU_SELECT_COLOR;
    else
	color = MENU_COLOR;
    glColor3fv(COLORS[color]);
}

static inline void drawCenteredText(const char* text, int height) {
    int length = glutBitmapLength(FONT, (const unsigned char*)text);
    drawString((prefs.windowSize[X] - 200 - length) / 2, prefs.windowSize[Y] - height * 20, text);
}

static inline void drawCenteredPair(const char* key, const char* value, int height) {
    int length = glutBitmapLength(FONT, (const unsigned char*)key);
    drawString((prefs.windowSize[X] - 200) / 2 - length - 7, prefs.windowSize[Y] - height * 20, key);
    drawString((prefs.windowSize[X] - 200) / 2 + 7, prefs.windowSize[Y] - height * 20, value);
}

static inline void drawSettings() {
    int height = 2;
    char str[100];

    glColor3fv(COLORS[MENU_HEADER_COLOR]);
    int length = glutBitmapLength(FONT, (const unsigned char*)"Settings");
    drawString((prefs.windowSize[X] - 200 - length) / 2, prefs.windowSize[Y] - 20 * height, "Settings");
    height += 2;
    
    settingMenuColor(s_fullScreen);
    if (nResolutions > 0) {
	drawCenteredPair("FullScreen:", (prefs.fullscreen ? "On" : "Off"), height++);

	settingMenuColor(s_resolution);
	snprintf(str, 100, "%dx%d", prefs.fullScreenResolution[X], prefs.fullScreenResolution[Y]);
	drawCenteredPair("Resolution:", str, height++);
    } else {
	drawCenteredPair("FullScreen:", "N/A", height++);
	settingMenuColor(s_resolution);
	drawCenteredPair("Resolution:", "N/A", height++);
    }
    height++;

    settingMenuColor(s_walls);
    drawCenteredPair("Show walls:", (prefs.showWalls ? "On" : "Off"), height++);
    settingMenuColor(s_speed);

    /*
    int i = -1;
    int step = prefs.stepInterval;
    while (step) {
	step >>= 1;
	i++;
    }
    */
    
    drawCenteredPair("Starting speed:", SPEEDS_NAMES[prefs.stepInterval], height++);
    settingMenuColor(s_grid1);
    drawCenteredPair("Snake view grid level:", GRID_NAMES[prefs.gridLevel1], height++);
    settingMenuColor(s_grid3);
    drawCenteredPair("World view grid level:", GRID_NAMES[prefs.gridLevel3], height++);
    settingMenuColor(s_background);
    drawCenteredPair("Background color:", COLORS_NAMES[prefs.backgroundColor], height++);
    height += 2;

    settingMenuColor(s_resetBoard);
    drawCenteredPair("Reset", "Board rotation", height++);
    settingMenuColor(s_resetKeys);
    drawCenteredPair("Reset", "keys", height++);
    settingMenuColor(s_resetSettings);
    drawCenteredPair("Reset", "All settings", height++);
}

static inline void snakesMenuColor(int snake, snakes_t snakePrefs) {
    int color;
    if (snake == currentSnake && snakePrefs == currentSnakePrefs) {
	if (inputMode())
	    color = INPUT_COLOR;
	else
	    color = MENU_SELECT_COLOR;
    } else if (snakePrefs == st_name)
	color = MENU_HEADER_COLOR;
    else
	color = MENU_COLOR;
    glColor3fv(COLORS[color]);
}

static inline void drawSnakes() {
    int height = 2;
    char str[255];
    
    glColor3fv(COLORS[MENU_HEADER_COLOR]);
    int length = glutBitmapLength(FONT, (const unsigned char*)"Snakes Configuration");
    drawString((prefs.windowSize[X] - 200 - length) / 2, prefs.windowSize[Y] - 20 * height, "Snake Configuration");
    height += 2;

    for (int i = 0; i < MAX_SNAKES; i++) {
	//sprintf(str, "Snake %i", i + 1);
	//glColor3fv(COLORS[MENU_HEADER_COLOR]);
	if (i == currentSnake && inputMode())
	    getInput(str);
	else 
	    snprintf(str, 255, names[i]);
	snakesMenuColor(i, st_name);
	drawString(20, prefs.windowSize[Y] - 20 * height++, str);

	snakesMenuColor(i, st_ai);
	drawString(30, prefs.windowSize[Y] - 20 * height, "AI:");
	drawString(100, prefs.windowSize[Y] - 20 * height++, AI_NAMES[prefs.ai[i]]);

	if (prefs.ai[i] != ai_off) {
	    if (prefs.ai[i] == ai_remote) {
		if (remoteSnakes[i] != ai_remote) {
		    drawString(155, prefs.windowSize[Y] - 20 * (height - 1), " - ");
		    drawString(180, prefs.windowSize[Y] - 20 * (height - 1), AI_NAMES[remoteSnakes[i]]);
		} else {
		    drawString(155, prefs.windowSize[Y] - 20 * (height - 1), " - waiting connection...");
		}
	    }
	    //if (prefs.ai[i] != ai_human || (prefs.ai[i] == ai_remote && (remoteSnakes[i] != ai_human && remoteSnakes[i] != ai_remote))) {
	    if (prefs.ai[i] == ai_basic || (prefs.ai[i] == ai_remote && remoteSnakes[i] == ai_basic)) {
		snakesMenuColor(i, st_aiTarget);
		drawString(30, prefs.windowSize[Y] - 20 * height, "Target:");
		drawString(100, prefs.windowSize[Y] - 20 * height++, AIT_NAMES[prefs.aiTarget[i]]);

		if (prefs.aiTarget[i] == ait_head || prefs.aiTarget[i] == ait_tail) {
		    snakesMenuColor(i, st_aitSnake);
		    drawString(30, prefs.windowSize[Y] - 20 * height, "Target:");
		    //sprintf(str, "snake %i", prefs.aitSnake[i] + 1);
		    drawString(100, prefs.windowSize[Y] - 20 * height++, names[prefs.aitSnake[i]]);
		}
	    }

	    if (prefs.ai[i] != ai_remote || (remoteSnakes[i] != ai_remote && remoteSnakes[i] != ai_off)) {
		snakesMenuColor(i, st_color);
		drawString(30, prefs.windowSize[Y] - 20 * height, "Color: ");
		if (prefs.snakeColor[i] == prefs.backgroundColor)
		    glColor3fv(COLORS[WHITE]);
		else
		    glColor3fv(COLORS[prefs.snakeColor[i]]);
		drawString(100, prefs.windowSize[Y] - 20 * height++, COLORS_NAMES[prefs.snakeColor[i]]);
	    }
	}
	
	height += 2;
    }

}

static inline void networkMenuColor(int menu) {
    int color;
    if (menu == currentNetMenu) {
	if (inputMode())
	    color = INPUT_COLOR;
	else
	    color = MENU_SELECT_COLOR;
    } else
	color = MENU_COLOR;
    glColor3fv(COLORS[color]);
}

static inline void drawNetwork() {
    int height = 2;
    char str[255];
    
    glColor3fv(COLORS[MENU_HEADER_COLOR]);
    drawCenteredText("Network Game Configuration", height++);
    
    switch (insideMenu ? currentNetSelect : networkState) {
      case net_none:
	  height += 2;
	  networkMenuColor(0);
	  drawCenteredText("Start Server", height++);
	  networkMenuColor(1);
	  drawCenteredText("Start Client", height++);
	  break;

      case net_server:
	  glColor3fv(COLORS[MENU_HEADER_COLOR]);
	  drawCenteredText("Server configuration", height++);
	  if (networkState == net_server)
	      drawCenteredText("***** Server is running *****", height);
	  height += 2;

	  myIP.port = prefs.serverPort;
	  
	  glColor3fv(COLORS[LIGHT_GRAY]);
	  drawCenteredPair("Name", myIP.name, height++);
	  host2str(myIP, str);
	  drawCenteredPair("IP", str, height++);
	  
	  if (networkState != net_server)
	      networkMenuColor(0);
	  if (inputMode())
	      getInput(str);
	  else
	      port2str(myIP, str);
	  drawCenteredPair("Port", str, height++);

	  if (networkState != net_server)
	      networkMenuColor(1);
	  if (prefs.fineTune)
	      drawCenteredText("Auto fine-tune speed enabled", height);
	  else
	      drawCenteredText("Auto fine-tune speed disabled", height);
	  
	  height += 2;
	  networkMenuColor(2);
	  if (networkState == net_server) {
	      drawCenteredText("Stop Server", height++);
	      height++;
	      glColor3fv(COLORS[LIGHT_GRAY]);
	      drawCenteredText("Go to the Snakes menu to let others join", height);
	  } else
	      drawCenteredText("Start Server", height++);	      
	  break;
	  
      case net_client:
	  glColor3fv(COLORS[MENU_HEADER_COLOR]);
	  drawCenteredText("Client configuration", height++);
	  if (networkState == net_client)
	      drawCenteredText("***** Connected to Server *****", height);
	  height += 2;

	  if (currentNetMenu == 0 && inputMode())
	      getInput(str);
	  else
	      strncpy(str, prefs.clientConnection.name, 255);

	  glColor3fv(COLORS[LIGHT_GRAY]);
	  if (networkState != net_client)
	      networkMenuColor(0);
	  drawCenteredPair("Server Name", str, height++);

	  if (currentNetMenu == 1 && inputMode())
	      getInput(str);
	  else
	      host2str(prefs.clientConnection, str);

	  if (networkState != net_client)
	      networkMenuColor(1);
	  drawCenteredPair("Server IP", str, height++);
	  
	  if (currentNetMenu == 2 && inputMode())
	      getInput(str);
	  else
	      port2str(prefs.clientConnection, str);
	  if (networkState != net_client)
	      networkMenuColor(2);
	  drawCenteredPair("Server Port", str, height);
	  height += 2;

	  networkMenuColor(3);
	  if (networkState == net_client)
	      drawCenteredText("Disconnect from server", height++);
	  else
	      drawCenteredText("Connect to server", height++);
	  break;
    }
}

static inline int drawInstructionsNavigateMenu(int height, int indent) {
    int length = indent;
    glColor3fv(COLORS[INSTRUCTIONS_COLOR]);
    length += drawString(length, height, "Use ");
    glColor3fv(COLORS[HIGHLIGHT_COLOR]);
    length += drawString(length, height, "Up");
    glColor3fv(COLORS[INSTRUCTIONS_COLOR]);
    length += drawString(length, height, " & ");
    glColor3fv(COLORS[HIGHLIGHT_COLOR]);
    length += drawString(length, height, "Down");
    glColor3fv(COLORS[INSTRUCTIONS_COLOR]);
    length += drawString(length, height, " keys ");
    
    height -= 20;
    drawString(indent, height, "to navigate the menu");
    return height - 20;
}

static inline int drawInstructionsEscape(int height, int indent) {
    int length = indent;
    length += drawString(length, height, "Press ");
    colorIt(HIGHLIGHT_COLOR);
    length += drawString(length, height, "escape");
    colorIt(INSTRUCTIONS_COLOR);
    length += drawString(length, height, " to exit");
    return height - 20;
}

static inline int drawInstructionsUseXYKeys(int height, int indent, char* x, char* y) {
    int length = indent;
    length += drawString(length, height, "Use ");
    colorIt(HIGHLIGHT_COLOR);
    length += drawString(length, height, x);
    colorIt(INSTRUCTIONS_COLOR);
    length += drawString(length, height, " & ");
    colorIt(HIGHLIGHT_COLOR);
    length += drawString(length, height, y);
    colorIt(INSTRUCTIONS_COLOR);
    drawString(length, height, " keys");
    return height -= 20;
}

static inline void drawInstructions(int height) {
    glColor3fv(COLORS[INSTRUCTIONS_COLOR]);
    static const int INDENT = 8;
    height = prefs.windowSize[Y] - 20 * height;
    int length = INDENT;
    char str[21];
    switch (currentMenu) {
      case m_start:
	  length += drawString(INDENT, height, "Press ");
	  glColor3fv(COLORS[HIGHLIGHT_COLOR]);
	  length += drawString(length, height, "enter");
	  glColor3fv(COLORS[INSTRUCTIONS_COLOR]);
	  length += drawString(length, height, " to start");

	  height -= 40;
	  height = drawInstructionsNavigateMenu(height, INDENT);
	  break;
	  
      case m_keys:
	  if (insideMenu) {
	      if (insideKey) {
		  drawString(INDENT, height, "Press the key you");
		  height -= 20;
		  drawString(INDENT, height, "want to use for:");
		  height -= 20;
		  strncpy(str, KEY_NAME[currentKey], sizeof(str));
		  drawString(INDENT, height, str);
	      } else {
		  length += drawString(length, height, "Use the ");
		  colorIt(HIGHLIGHT_COLOR);
		  length += drawString(length, height, "arrow");
		  colorIt(INSTRUCTIONS_COLOR);
		  drawString(length, height, " keys");
		  
		  height -= 20;
		  drawString(INDENT, height, "to navigate");

		  height -= 40;
		  length = INDENT;
		  length += drawString(length, height, "Press ");
		  colorIt(HIGHLIGHT_COLOR);
		  length += drawString(length, height, "enter");
		  colorIt(INSTRUCTIONS_COLOR);
		  drawString(length, height, " to select");

		  height -= 20;
		  drawInstructionsEscape(height, INDENT);
	      }
	  }

	  else {
	      length += drawString(length, height, "Press ");
	      glColor3fv(COLORS[HIGHLIGHT_COLOR]);
	      length += drawString(length, height, "enter");
	      glColor3fv(COLORS[INSTRUCTIONS_COLOR]);
	      length += drawString(length, height," to change");
	      
	      height -= 20;
	      drawString(INDENT, height, "key configurations");
	      
	      height -= 40;
	      drawInstructionsNavigateMenu(height, INDENT);
	  }
	  
	  height -= 60;
	  drawString(INDENT, height, "S- Shift");
	  height -= 20;
	  drawString(INDENT, height, "C- Control");
	  height -= 20;
#if (defined __APPLE__ && defined __MACH__)
	  drawString(INDENT, height, "M- Command");
#else
	  drawString(INDENT, height, "M- Meta");
#endif
	  height -= 20;
	  drawString(INDENT, height, "A- Alt");
	  break;

      case m_worlds:
	  if (insideMenu) {
	      height = drawInstructionsUseXYKeys(height, INDENT, "Left", "Right");

	      length = INDENT;
	      drawString(length, height, "to switch board type");
	      
	      height -= 40;
	      height = drawInstructionsUseXYKeys(height, INDENT, "Up", "Down");
	      drawString(INDENT, height, "to switch boards size");
	      
	      height -= 40;
	      drawInstructionsEscape(height, INDENT);
	  }
	  
	  else {
	      length += drawString(length, height, "Press ");
	      colorIt(HIGHLIGHT_COLOR);
	      length += drawString(length, height, "enter");
	      colorIt(INSTRUCTIONS_COLOR);
	      length += drawString(length, height, " to change");
	      
	      height -= 20;
	      length = INDENT;
	      drawString(length, height, "board type/size");

	      height -= 40;
	      drawInstructionsNavigateMenu(height, INDENT);
	  }
	  break;

      case m_snakes:
	  if (insideMenu) {
	      switch (currentSnakePrefs) {
		case st_ai:
		    height = drawInstructionsUseXYKeys(height, INDENT, "Left", "Right");
		    drawString(INDENT, height, "change the snake's AI");
		    break;
		    
		case st_aiTarget:
		    height = drawInstructionsUseXYKeys(height, INDENT, "Left", "Right");
		    drawString(INDENT, height, "select the snake's");
		    height -= 20;
		    drawString(INDENT, height, "target");
		    break;
		    
		case st_aitSnake:
		    height = drawInstructionsUseXYKeys(height, INDENT, "Left", "Right");
		    drawString(INDENT, height, "select the snake's");
		    height -= 20;
		    drawString(INDENT, height, "target snake");
		    break;
		    
		case st_color:
		    height = drawInstructionsUseXYKeys(height, INDENT, "Left", "Right");
		    drawString(INDENT, height, "change the snake's");
		    height -= 20;
		    drawString(INDENT, height, "color");
		    break;

		case st_name:
		    length += drawString(INDENT, height, "Press ");
		    glColor3fv(COLORS[HIGHLIGHT_COLOR]);
		    length += drawString(length, height, "enter");
		    glColor3fv(COLORS[INSTRUCTIONS_COLOR]);
		    length += drawString(length, height, " to change");
		    height -= 20;
		    drawString(INDENT, height, "your snake's name");
		    break;
	      }

	      height -= 40;
	      length = INDENT;
	      if (inputMode()) {
		  length += drawString(INDENT, height, "Press ");
		  glColor3fv(COLORS[HIGHLIGHT_COLOR]);
		  length += drawString(length, height, "Up");
		  glColor3fv(COLORS[INSTRUCTIONS_COLOR]);
		  length += drawString(length, height, ", ");
		  glColor3fv(COLORS[HIGHLIGHT_COLOR]);
		  length += drawString(length, height, "Down");
		  glColor3fv(COLORS[INSTRUCTIONS_COLOR]);
		  length += drawString(length, height, " or");
		  height -= 20;
		  length = INDENT;
		  glColor3fv(COLORS[HIGHLIGHT_COLOR]);
		  length += drawString(length, height, "escape");
		  glColor3fv(COLORS[INSTRUCTIONS_COLOR]);
		  length += drawString(length, height, " to cancel");
		  height -= 20;
		  drawString(INDENT, height, "name change");
	      } else {
		  height = drawInstructionsUseXYKeys(height, INDENT, "Up", "Down");
		  drawString(INDENT, height, "to navigate between");
		  height -= 20;
		  drawString(INDENT, height, "configurations");
		  height -= 40;
		  drawInstructionsEscape(height, INDENT);
	      }
	  }

	  else {
	      length += drawString(length, height, "Press ");
	      colorIt(HIGHLIGHT_COLOR);
	      length += drawString(length, height, "enter");
	      colorIt(INSTRUCTIONS_COLOR);
	      length += drawString(length, height, " to change");

	      height -= 20;
	      drawString(INDENT, height, "snakes configuration");
	      
	      height -= 40;
	      drawInstructionsNavigateMenu(height, INDENT);
	  }
	  break;
	  
      case m_settings:
	  if (insideMenu) {
	      switch (currentSetting) {
		case s_resetBoard:
		case s_resetKeys:
		case s_resetSettings:
		    drawColoredString(INDENT, height, "Press $Henter$I to reset");
		    break;
		    
		default:
		    height -= 20;
		    drawColoredString(INDENT, height, "Use the $HLeft$I, $HRight$I");
		    height -= 20;
		    drawColoredString(INDENT, height, "or $Henter$I keys to");
		    height -= 20;
		    drawString(INDENT, height, "change setting");
	      }
	      
	      height -= 40;
	      height = drawInstructionsNavigateMenu(height, INDENT) - 20;
	      drawInstructionsEscape(height, INDENT);
	  }

	  else {
	      drawColoredString(INDENT, height, "Press $Henter$I to change");
	      height -= 20;
	      drawString(INDENT, height, "settings");
	      height -= 40;
	      drawInstructionsNavigateMenu(height, INDENT);
	  }

	  break;

      case m_network:
	  if (insideMenu) {
	      switch (currentNetSelect) {
		case net_none:
		    if (currentNetMenu == 0) {
			drawColoredString(INDENT, height, "Press $Henter$I to");
			drawString(INDENT, height -= 20, "configure server");
		    } else {
			drawColoredString(INDENT, height, "Press $Henter$I to");
			drawString(INDENT, height -= 20, "configure client");
		    }
		    height = drawInstructionsNavigateMenu(height -= 40, INDENT);
		    break;
			
		case net_server:
		    switch (currentNetMenu) {
		      case 0:
			  if (inputMode()) {
			      drawColoredString(INDENT, height, "Press $Henter$I to change");
			      drawString(INDENT, height -= 20, "listening port number");
			      drawColoredString(INDENT, height -= 40, "Press $HUp$I, $HDown$I or");
			      drawColoredString(INDENT, height -= 20, "$Hescape$I to cancel");
			      drawColoredString(INDENT, height -= 20, "port change");
			      height -= 20;
			  } else {
			      drawColoredString(INDENT, height, "Press $Henter$I to");
			      drawString(INDENT, height -= 20, "configure listening");
			      drawString(INDENT, height -= 20, "port number");
			      height = drawInstructionsNavigateMenu(height -= 40, INDENT);
			  }
			  break;
			  
		      case 1:
			  drawColoredString(INDENT, height, "Press $Henter$I to");
			  if (prefs.fineTune)
			      drawString(INDENT, height -= 20, "disable speed fine");
			  else
			      drawString(INDENT, height -= 20, "enable speed tuning");
			  drawColoredString(INDENT, height -= 20, "over network");
			  height = drawInstructionsNavigateMenu(height -= 40, INDENT);
			  break;
			  
		      case 2:
			  if (networkState == net_server) {
			      drawColoredString(INDENT, height, "Press $Henter$I to close");
			      drawString(INDENT, height -= 20, "server connections");
			      height -= 20;
			  } else {
			      drawColoredString(INDENT, height, "Press $Henter$I to start");
			      drawString(INDENT, height -= 20, "server");  
			      height = drawInstructionsNavigateMenu(height -= 40, INDENT);
			  }
			  break;
		    }
		    break;

		case net_client:
		    switch (currentNetMenu) {
		      case 0:
			  if (inputMode()) {
			      drawColoredString(INDENT, height, "Press $Henter$I to change");
			      drawString(INDENT, height -= 20, "the server");
			      drawColoredString(INDENT, height -= 40, "Press $HUp$I, $HDown$I or");
			      drawColoredString(INDENT, height -= 20, "$Hescape$I to cancel");
			      drawColoredString(INDENT, height -= 20, "server change");
			      height -= 20;
			  } else {
			      drawColoredString(INDENT, height, "Press $Henter$I to change");
			      drawString(INDENT, height -= 20, "the name of the");
			      drawString(INDENT, height -= 20, "server");
			      height = drawInstructionsNavigateMenu(height -= 40, INDENT);
			  }
			  break;

		      case 1:
			  if (inputMode()) {
			      drawColoredString(INDENT, height, "Press $Henter$I to change");
			      drawString(INDENT, height -= 20, "the server");
			      drawColoredString(INDENT, height -= 40, "Press $HUp$I, $HDown$I or");
			      drawColoredString(INDENT, height -= 20, "$Hescape$I to cancel");
			      drawColoredString(INDENT, height -= 20, "server change");
			      height -= 20;
			  } else {
			      drawColoredString(INDENT, height, "Press $Henter$I to change");
			      drawString(INDENT, height -= 20, "the address of the");
			      drawString(INDENT, height -= 20, "server");
			      height = drawInstructionsNavigateMenu(height -= 40, INDENT);
			  }
			  break;

		      case 2:
			  if (inputMode()) {
			      drawColoredString(INDENT, height, "Press $Henter$I to change");
			      drawString(INDENT, height -= 20, "port number");
			      drawColoredString(INDENT, height -= 40, "Press $HUp$I, $HDown$I or");
			      drawColoredString(INDENT, height -= 20, "$Hescape$I to cancel");
			      drawColoredString(INDENT, height -= 20, "port change");
			      height -= 20;
			  } else {
			      drawColoredString(INDENT, height, "Press $Henter$I to change");
			      drawString(INDENT, height -= 20, "port number");
			      height = drawInstructionsNavigateMenu(height -= 40, INDENT);
			  }
			  break;

		      case 3:
			  if (networkState == net_client) {
			      drawColoredString(INDENT, height, "Press $Henter$I to close");
			      drawString(INDENT, height -= 20, "connection");
			      height -= 20;
			  } else {
			      drawColoredString(INDENT, height, "Press $Henter$I to");
			      drawString(INDENT, height -= 20, "connect to server");  
			      height = drawInstructionsNavigateMenu(height -= 40, INDENT);
			  }
			  break;
			  
		    }
		    break;
	      }
	      drawInstructionsEscape(height -= 20, INDENT);
	  }
	  
	  else {
	      drawColoredString(INDENT, height, "Press $Henter$I to setup");
	      drawString(INDENT, height -= 20, "a network game");
	      drawInstructionsNavigateMenu(height -= 40, INDENT);
	  }
	  
	  break;

	  /*
      case m_about:
	  drawString(INDENT, height, "Read the about");
	  height++;
	  drawString(INDENT, height, "Use Up, Down keys");
	  drawString(INDENT, height, "to navigate menu");
	  break;
	  */
	  
      case m_quit:
	  drawColoredString(INDENT, height, "Press $Henter$I to quit");
	  drawInstructionsNavigateMenu(height -= 40, INDENT);
	  break;
    }
}

static inline void display() {
    char str[120];
    
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    glLightfv(GL_LIGHT0, GL_POSITION, LIGHT_POSITION);

    glViewport(200, 0, prefs.windowSize[X] - 200, prefs.windowSize[Y]);

    // draw according to menu selection
    int length;
    switch (currentMenu) {
      case m_worlds:
	  glMatrixMode(GL_PROJECTION);
	  glLoadMatrixf(screenProjectionMatrix);
	  glMatrixMode(GL_MODELVIEW);
	  glLoadIdentity();
	  glDisable(GL_LIGHTING);
	  glColor3fv(COLORS[WHITE]);

	  //strcpy(name, "<-- ");
	  //strncpy(name + 4, mainBoard->getName(), 110);
	  //strcpy(name + strlen(name), " -->");
	  strncpy(str, mainBoard->getName(), 110);

	  glMatrixMode(GL_PROJECTION);
	  glLoadMatrixf(viewProjectionMatrix);
	  length = glutBitmapLength(FONT, (const unsigned char*)str);
	  drawString((prefs.windowSize[X] - 200 - length) / 2, prefs.windowSize[Y] - 20, str);
	  
	  //length = glutBitmapLength(FONT, (const unsigned char*)mainBoard->getName());
	  //drawString((windowSize[X] - 200 + length) / 2, 15, mainBoard->getName());

	  glEnable(GL_LIGHTING);
	  
      case m_start:
	  glMatrixMode(GL_PROJECTION);
	  glLoadMatrixf(thirdPersonProjectionMatrix);
	  glMatrixMode(GL_MODELVIEW) ;
	  glLoadIdentity();
	  drawboard(false);
	  break;

      case m_snakes:
	  glMatrixMode(GL_PROJECTION);
	  glLoadMatrixf(viewProjectionMatrix);
	  drawSnakes();
	  break;
	  
      case m_keys:
	  glMatrixMode(GL_PROJECTION);
	  glLoadMatrixf(viewProjectionMatrix);
	  drawKeys();
	  break;

      case m_settings:
	  glMatrixMode(GL_PROJECTION);
	  glLoadMatrixf(viewProjectionMatrix);
	  drawSettings();
	  break;

      case m_network:
	  glMatrixMode(GL_PROJECTION);
	  glLoadMatrixf(viewProjectionMatrix);
	  drawNetwork();
	  break;
	  
	  //case m_about:
      case m_quit:
	  glMatrixMode(GL_PROJECTION);
	  glLoadMatrixf(viewProjectionMatrix);
	  glMatrixMode(GL_MODELVIEW);
	  glLoadIdentity();
	  drawAbout();
	  break;
    }

    if (networkState != net_none) {
	glDisable(GL_LIGHTING);
	glMatrixMode(GL_PROJECTION);
	glLoadMatrixf(viewProjectionMatrix);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glColor3fv(COLORS[WHITE]);
	if (networkState == net_client)
	    snprintf(str, 120, "Connected to server: %s", prefs.clientConnection.name);
	else {
	    int available = 0;
	    int connected = 0;
	    int ready = 0;
	    for (int i = 0; i < MAX_SNAKES; i++) {
		if (prefs.ai[i] == ai_remote) available++;
		if (remoteSnakes[i] != ai_remote) connected++;
		if (recvTicks[i]) ready++;
	    }
	    snprintf(str, 120, "Running server: %i/%i clients connected (%i ready)", connected, available, ready);
	}
	length = glutBitmapLength(FONT, (const unsigned char*)str);
	drawString((prefs.windowSize[X] - 200 - length) / 2, 15, str);
	glEnable(GL_LIGHTING);
    }
    
    // draw screen
    glMatrixMode(GL_PROJECTION);
    glLoadMatrixf(screenProjectionMatrix);
    glViewport(0, 0, prefs.windowSize[X], prefs.windowSize[Y]);
    glMatrixMode(GL_MODELVIEW) ;
    glLoadIdentity();
    drawScreen(false);
    drawMenu();

    int lineY = MENU_SIZE * 20 + 120;
    glBegin(GL_LINES);
    glColor3fv(COLORS[LIGHT_GRAY]);
    glVertex3d(0, prefs.windowSize[Y] - lineY + 1, 0.5);
    glVertex3d(200, prefs.windowSize[Y] - lineY + 1, 0.5);

    glColor3fv(COLORS[GRAY]);
    glVertex3d(0, prefs.windowSize[Y] - lineY, 0.5);
    glVertex3d(200, prefs.windowSize[Y] - lineY, 0.5);

    glColor3fv(COLORS[DARK_GRAY]);
    glVertex2d(0, prefs.windowSize[Y] - lineY - 1);
    glVertex2d(200, prefs.windowSize[Y] - lineY - 1);
    glEnd();


    drawInstructions(lineY / 20 + 2);

    //glutSwapBuffers();
    SDL_GL_SwapBuffers();
    redisplay = false;
}

static inline void keysKeyboard(SDL_keysym key) {
    if (insideKey) {
	prefs.SDLkeys[currentKey] = key;
	insideKey = false;
	
    } else switch (key.sym) {
      case SDLK_RETURN: insideKey = true; break;
      case SDLK_ESCAPE: deactivateMenu(); break;
      case SDLK_LEFT:  currentKey = KEY_MENUS_MOVEMENT[currentKey][left]; break;
      case SDLK_DOWN:  currentKey = KEY_MENUS_MOVEMENT[currentKey][down]; break;
      case SDLK_RIGHT: currentKey = KEY_MENUS_MOVEMENT[currentKey][right];break;
      case SDLK_UP:    currentKey = KEY_MENUS_MOVEMENT[currentKey][up];   break;
      default:;
    }
}

static inline void startKeyboard(SDL_keysym key) {
    int i = 0;
    for (i = 0; i < 4; i++) {
	if (keyPressed(key, k_boardRotate[i])) {
	    state->boardRotation++;
	    state->boardRotating[i] = true;
	    break;
	}
    }

    if (i == 4) {
	if (keyPressed(key, k_boardYawLeft)) {
	    state->boardRotation++;
	    state->boardRotating[forward] = true;
	} else if (keyPressed(key, k_boardYawRight)) {
	    state->boardRotation++;
	    state->boardRotating[backward] = true;
	} else if (keyPressed(key, k_zoomIn))
	    state->boardZoom[0] = true;
	else if (keyPressed(key, k_zoomOut))
	    state->boardZoom[1] = true;
	else if (keyPressed(key, k_resetBoard))
	    defaultRotation(true);
	else if (keyPressed(key, k_fullScreen))
	    switchFullScreen();
	else if (keyPressed(key, k_showWalls))
	    prefs.showWalls ^= true;
	else if (keyPressed(key, k_backgroundColor)) {
	    prefs.backgroundColor++;
	    prefs.backgroundColor %= NCOLORS;
	    glClearColor(COLORS[prefs.backgroundColor][RED], COLORS[prefs.backgroundColor][GREEN], COLORS[prefs.backgroundColor][BLUE], 0.0);
	} else if (keyPressed(key, k_grid3))
	    prefs.gridLevel3 = (prefs.gridLevel3 == 3 ? 0 : prefs.gridLevel3 + 1);
	else if (keyPressed(key, k_speedUp)) {
	    speedUp();
	} else if (keyPressed(key, k_speedDown)) {
	    speedDown();
	}
    }
}

static inline void settingsKeyboard(SDL_keysym key) {
    int diff;
    switch (key.sym) {
      case SDLK_UP:
	  currentSetting = SETTINGS_MENU_MOVEMENT_UP[currentSetting];
	  break;
	  
      case SDLK_DOWN:
	  currentSetting = SETTINGS_MENU_MOVEMENT_DOWN[currentSetting];
	  break;

      case SDLK_LEFT:
      case SDLK_RIGHT:
	  
	  diff = (key.sym == SDLK_RIGHT) * 2 - 1;
	  switch (currentSetting) {
	    case s_fullScreen:
		switchFullScreen();
		break;
		
	    case s_resolution:
		if (nResolutions == 0)
		    break;
		prefs.resolution += nResolutions + diff;
		prefs.resolution %= nResolutions;
		prefs.fullScreenResolution[X] = resolutions[prefs.resolution][X];
		prefs.fullScreenResolution[Y] = resolutions[prefs.resolution][Y];
		/*
		if (!checkGameResolution()) {
		    prefs.resolution += NRESOLUTIONS - diff;
		    prefs.resolution %= NRESOLUTIONS;
		    checkGameResolution();
		    } else */
		if (prefs.fullscreen) {
		    //enterFullScreen();
		    //menuCallbacks(false);
		    prefs.windowSize[X] = prefs.fullScreenResolution[X];
		    prefs.windowSize[Y] = prefs.fullScreenResolution[Y];
		    reshape();
		}
		break;
				
	    case s_grid1:
		prefs.gridLevel1 += diff + 4;
		prefs.gridLevel1 %= 4;
		break;
		
	    case s_grid3:
		prefs.gridLevel3 += diff + 4;
		prefs.gridLevel3 %= 4;
		break;
		
	    case s_walls:
		prefs.showWalls ^= true;
		break;
		
	    case s_background:
		prefs.backgroundColor += NCOLORS + diff;
		prefs.backgroundColor %= NCOLORS;
		glClearColor(COLORS[prefs.backgroundColor][RED], COLORS[prefs.backgroundColor][GREEN], COLORS[prefs.backgroundColor][BLUE], 0.0);
		break;
		
	    case s_speed:
		if (key.sym == SDLK_LEFT) {
		    //if (prefs.stepInterval < (0x1 << (NSPEEDS - 1))) {
		    //prefs.stepInterval <<= 1;
		    //}
		    speedDown();
		} else  {//if (prefs.stepInterval > 0x1) {
		    //prefs.stepInterval >>= 1;
		    speedUp();
		}
		break;

	    case s_resetKeys:
	    case s_resetSettings: 
	    case s_resetBoard:
		break;
	  }
	  break;
	  
      case SDLK_RETURN:
	  switch (currentSetting) {
	    case s_resetBoard:
		defaultRotation(true);
		break;
		
	    case s_resetKeys:
		defaultKeys();
		break;
		
	    case s_resetSettings:
		defaultPreferences(false);
		reshape();
		/*
		if (glutGameModeGet(GLUT_GAME_MODE_ACTIVE) != prefs.fullscreen) {
		    prefs.fullscreen ^= true;
		    switchFullScreen();
		    }*/
		glClearColor(COLORS[prefs.backgroundColor][RED], COLORS[prefs.backgroundColor][GREEN], COLORS[prefs.backgroundColor][BLUE], 0.0);
		if (networkState != net_client)
		    rebuildBoard = true;
		break;

	    default:
		key.sym = SDLK_RIGHT;
		settingsKeyboard(key);
		break;
	  }
	  break;

      case SDLK_ESCAPE:
	  deactivateMenu();
	  break;
	  
      default:;
    }
}

static inline void snakesKeyboard(SDL_keysym key) {
    int diff;
    ai_t currentAI;
    switch (key.sym) {
      case SDLK_ESCAPE:
	  if (inputMode()) {
	      endUserInput();
	  } else
	      deactivateMenu();
	  return;
	  
      case SDLK_DOWN:
	  endUserInput();
	  if (networkState == net_client) {
	      if (currentSnakePrefs == st_color)
		  currentSnakePrefs = st_name;
	      else
		  currentSnakePrefs = st_color;
	      return;
	  }

      skdown:
	  currentSnakePrefs += 1;
	  if (currentSnakePrefs > st_color) {
	      currentSnake += 1;
	      currentSnake %= MAX_SNAKES;
	      currentSnakePrefs = 0;
	  }
	  currentAI = prefs.ai[currentSnake];
	  switch (currentSnakePrefs) {
	    case st_name: if (currentAI != ai_human) goto skdown; break;
	    case st_ai: if (currentAI == ai_human) goto skdown; break;
	    case st_aiTarget: if (currentAI != ai_basic) goto skdown; break;
	    case st_aitSnake:
		if (currentAI != ai_basic || (prefs.aiTarget[currentSnake] != ait_head && prefs.aiTarget[currentSnake] != ait_tail)) goto skdown;
		break;
	    case st_color: if (currentAI == ai_remote || currentAI == ai_off) goto skdown; break;
	  }
	  break;
	  
      case SDLK_UP:
	  endUserInput();
	  if (networkState == net_client) {
	      if (currentSnakePrefs == st_color)
		  currentSnakePrefs = st_name;
	      else
		  currentSnakePrefs = st_color;
	      return;
	  }

      skup:
	  currentSnakePrefs -= 1;
	  if (currentSnakePrefs == -1) {
	      currentSnake += MAX_SNAKES - 1;
	      currentSnake %= MAX_SNAKES;
	      currentSnakePrefs = st_color;
	  }
	  currentAI = prefs.ai[currentSnake];
	  switch (currentSnakePrefs) {
	    case st_name: if (currentAI != ai_human) goto skup; break;
	    case st_ai: if (currentAI == ai_human) goto skup; break;
	    case st_aiTarget: if (currentAI != ai_basic) goto skup; break;
	    case st_aitSnake:
		if (currentAI != ai_basic || (prefs.aiTarget[currentSnake] != ait_head && prefs.aiTarget[currentSnake] != ait_tail)) goto skup;
		break;
	    case st_color: if (currentAI == ai_remote || currentAI == ai_off) goto skup; break;
	  }
	  break;
	      
      case SDLK_RETURN:
	  if (currentSnakePrefs == st_name) {
	      if (inputMode()) {
		  char str[20];
		  endUserInput(str, 20);
		  if (strlen(str) > 0) {
		      strncpy(prefs.name, str, 20);
		      strncpy(names[currentSnake], prefs.name, 20);
		      if (networkState != net_none)
			  sendSnakeConfig(currentSnake);
		  }
	      } else
		  beginUserInput(letters);
	      break;
	  }
	  key.sym = SDLK_RIGHT;
	  
      case SDLK_LEFT:
      case SDLK_RIGHT:
	  diff = (key.sym == SDLK_RIGHT) * 2 - 1;
          int newTarget;
	  switch (currentSnakePrefs) {
	    case st_ai:
		if (prefs.ai[currentSnake] != ai_human &&
		    (networkState == net_server || prefs.ai[currentSnake] != ai_remote)) { /*disable human1*/
		    int tempNAI = NAI + (networkState == net_server);
		    int newAI = prefs.ai[currentSnake];
		    newAI += tempNAI + diff;
		    newAI %= tempNAI;
		    if (newAI == ai_human) {
			newAI += tempNAI + diff;
			newAI %= tempNAI;
		    }
		    if (prefs.ai[currentSnake] == ai_remote) {
			remoteSnakes[currentSnake] = ai_remote;
			recvTicks[currentSnake] = false;
		    }
		    prefs.ai[currentSnake] = (ai_t)newAI;
		} /*disable human1*/
		break;
		
	    case st_aiTarget:
		newTarget = prefs.aiTarget[currentSnake];
		newTarget += NAI_TARGET + diff;
		newTarget %= NAI_TARGET;
		prefs.aiTarget[currentSnake] = (aiTarget_t)newTarget;
		break;
		
	    case st_aitSnake:
		prefs.aitSnake[currentSnake] += MAX_SNAKES + diff;
		prefs.aitSnake[currentSnake] %= MAX_SNAKES;
		break;

	    case st_color:
		prefs.snakeColor[currentSnake] += NCOLORS + diff;
		prefs.snakeColor[currentSnake] %= NCOLORS;
		if (prefs.snakeColor[currentSnake] == RED) {
		    // snakesKeyboard(key); don't send packet twice
		    prefs.snakeColor[currentSnake] += NCOLORS + diff;
		    prefs.snakeColor[currentSnake] %= NCOLORS;
		}
		break;
	  }
	  if (networkState != net_none)
	      sendSnakeConfig(currentSnake);

	  
      default:
	  if (inputMode()) 
	      inputKey(key);
    }

}

static inline void worldsKeyboard(SDL_keysym key) {
    switch (key.sym) {
      case SDLK_UP:
	  prefs.currentBoard++;
	  prefs.currentBoard %= nBoards[prefs.currentBoardType];
	  rebuildBoard = true;
	  break;

      case SDLK_DOWN:
	  prefs.currentBoard += nBoards[prefs.currentBoardType] - 1;
	  prefs.currentBoard %= nBoards[prefs.currentBoardType];
	  rebuildBoard = true;
	  break;
	  
      case SDLK_RIGHT:
	  prefs.currentBoardType++;
	  prefs.currentBoardType %= nBoardsTypes;
	  //if (prefs.currentBoard >= nBoards[prefs.currentBoardType])
	  //prefs.currentBoard = nBoards[prefs.currentBoardType] - 1;
	  prefs.currentBoard = boardsDefaultSize[prefs.currentBoardType];
	  rebuildBoard = true;
	  break;
	  
      case SDLK_LEFT:
	  prefs.currentBoardType += nBoardsTypes - 1;
	  prefs.currentBoardType %= nBoardsTypes;
	  prefs.currentBoard = boardsDefaultSize[prefs.currentBoardType];
	  rebuildBoard = true;
	  break;
	  
      case SDLK_ESCAPE:
	  deactivateMenu();
	  break;
	  
      default:;
    }
}

static inline void networkKeyboard(SDL_keysym key) {
    int diff;
    //char str[100];
    
    switch (key.sym) {
      case SDLK_ESCAPE:
	  if (inputMode())
	      endUserInput();
	  else {
	      if (currentNetSelect == networkState)
		  deactivateMenu();
	      else {
		  if (currentNetSelect == net_server)
		      currentNetMenu = 0;
		  else
		      currentNetMenu = 1;
		  currentNetSelect = net_none;
	      }
	  }
	  return;
	  
      case SDLK_DOWN:
      case SDLK_UP:
	  endUserInput();
	  if (networkState == net_none) {
	      diff = (key.sym == SDLK_DOWN) * 2 - 1;
	      currentNetMenu += diff + N_NET_MENUS[currentNetSelect];
	      currentNetMenu %= N_NET_MENUS[currentNetSelect];
	  }
	  break;
	  
      case SDLK_RETURN:
	  switch(currentNetSelect) {
	    case net_none:
		initNetwork();
		if (currentNetMenu == 0) {
		    currentNetSelect = net_server;
		    currentNetMenu = 2;
		} else {
		    currentNetSelect = net_client;
		    currentNetMenu = 0;
		}
		break;
		
	    case net_server:
		if (currentNetMenu == 0) {
		    if (inputMode()) {
			Uint32 temp;
			endUserInput(&temp);
			if (temp < 0xffff && temp > 0x0)
			    prefs.serverPort = temp;
		    } else
			beginUserInput(digits);
		} else if (currentNetMenu == 1) {
		    prefs.fineTune ^= true;
		    sendToggleFineTune(prefs.fineTune);
		} else if (currentNetMenu == 2) {
		    if (networkState == net_server) {
			breakConnection();
		    } else {
			startServer();
			/*
			  check for net errors before applying this
			  if (networkState == net_server) {
			  deactivateMenu();
			  currentMenu = m_snakes;
			  remoteSnakes[1] = ai_remote;
			  prefs.ai[1] = ai_remote;
			  activateMenu();
			  till this is fixed, just get out of menu
			  }
			*/
		    }
		}
		break;
		
	    case net_client:
		switch (currentNetMenu) {
		  case 0:
		      if (inputMode()) {
			  ip_t ip;
			  ip.host = 0;
			  endUserInput(ip.name);
			  fixIP(&ip);
			  if (ip.host != 0) {
			      prefs.clientConnection.host = ip.host;
			      strcpy(prefs.clientConnection.name, ip.name);
			  }
		      } else
			  beginUserInput(letters);
		      break;

		  case 1:
		      if (inputMode()) {
			  ip_t ip;
			  ip.host = 0;
			  endUserInput(ip.name);
			  fixIP(&ip);
			  if (ip.host != 0) {
			      ip.name[0] = 0;
			      fixIP(&ip);
			      prefs.clientConnection.host = ip.host;
			      strcpy(prefs.clientConnection.name, ip.name);
			  }
		      } else
			  beginUserInput(dotDigits);
		      break;

		  case 2:
		      if (inputMode()) {
			  Uint32 temp;
			  endUserInput(&temp);
			  if (temp < 0xffff && temp > 0x0)
			      prefs.clientConnection.port = temp;
		      } else
			  beginUserInput(digits);
		      break;

		  case 3:
		      if (networkState == net_client) {
			  breakConnection();
		      } else
			  startClient();
		      break;

		}
		break;
		
	  }
	  
      default:
	  if (inputMode())
	      inputKey(key);
    }
}

static inline void keyboard(SDL_keysym key) {
    
    // inside menu
    if (insideMenu) {
	switch (currentMenu) {
	  case m_keys: keysKeyboard(key); break;
	  case m_worlds: worldsKeyboard(key); break;
	      //case m_about: break;
	  case m_quit: break;
	  case m_snakes: snakesKeyboard(key); break;
	  case m_settings: settingsKeyboard(key); break;
	  case m_network: networkKeyboard(key); break;
	  case m_start:
	  default:
	      if (key.sym == SDLK_ESCAPE)
		  deactivateMenu();
	}
    }
    
    // outside menu
    else {
	
	// enter menu
	if (key.sym == SDLK_RETURN)
	    activateMenu();

	// quit
	else if (keyPressed(key, k_quit))
	    quit(0);

	// navigate menu
	else if (key.sym == SDLK_UP || key.sym == SDLK_DOWN) {
	    static float rotationMatrix[16];
	    static float fov;
	    static int grid;
	    static bool walls;
	    if (currentMenu == m_worlds) {
		glMatrixMode(GL_MODELVIEW);
		glLoadMatrixf(rotationMatrix);
		glGetFloatv(GL_MODELVIEW_MATRIX, prefs.boardRotationMatrix);
		prefs.fieldOfView = fov;
		prefs.gridLevel3 = grid;
		prefs.showWalls = walls;
		//reshape(prefs.windowSize[X], prefs.windowSize[Y]);
		loadPerspectives();
		savePreferences();
	    } else if (currentMenu == m_start)
		emptyState();


	    switch (key.sym) {
	      case SDLK_DOWN:
		  currentMenu++;
		  currentMenu %= MENU_SIZE;
		  break;
	      case SDLK_UP:
		  currentMenu--;
		  currentMenu += MENU_SIZE;
		  currentMenu %= MENU_SIZE;
		  break;
	      default:;
	    }
	    
	    if (currentMenu == m_worlds) {
		memcpy(rotationMatrix, prefs.boardRotationMatrix, sizeof(rotationMatrix));
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		glRotatef(20, 1.0,   0,   0); 
		glGetFloatv(GL_MODELVIEW_MATRIX, prefs.boardRotationMatrix);
		fov = prefs.fieldOfView;
		grid = prefs.gridLevel3;
		walls = prefs.showWalls;
		prefs.gridLevel3 = 1;
		prefs.showWalls = true;
		prefs.fieldOfView = INIT_FIELD_OF_VIEW;
		//reshape(prefs.windowSize[X], prefs.windowSize[Y]);
		loadPerspectives();
	    }
	}
	
	// for unactivatalbe menus
	if (currentMenu == m_start)
	    startKeyboard(key);
    }
    
    redisplay = true;
}

static inline void keyboardUp(SDL_keysym key) {
    if (currentMenu == m_start) {
	for (int i = 0; i < 4; i++) {
	    if (keyPressed(key, k_boardRotate[i])) {
		state->boardRotation--;
		state->boardRotating[i] = false;
		break;
	    }
	}
	if (keyPressed(key, k_boardYawLeft)) {
	    state->boardRotation--;
	    state->boardRotating[forward] = false;
	} else if (keyPressed(key, k_boardYawRight)) {
	    state->boardRotation--;
	    state->boardRotating[backward] = false;
	} else if (keyPressed(key, k_zoomIn))
	    state->boardZoom[0] = false;
	else if (keyPressed(key, k_zoomOut))
	    state->boardZoom[1] = false;
    }
}

//static inline Uint32 timer(Uint32 interval, void* value) {
static inline void tick(int times) {
    if (p_mode != pm_menu)// || value != gameModeValue)
	return;

    globalTick(times);

    if (currentMenu == m_worlds) {
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
 
	glRotatef(-BOARD_ROTATION_ANGLE / 2,   0, 1.0,   0);
	glMultMatrixf(prefs.boardRotationMatrix);
	glGetFloatv(GL_MODELVIEW_MATRIX, prefs.boardRotationMatrix);
    }
    
    //glutPostRedisplay();
    redisplay = true;
    //glutTimerFunc(BASE_INTERVAL, timer, value);   // uint millis int value
    return;
}

void menuLoop() {
    //if (init) {
    
    timerLong = 0;
    stepPercent = 0;
    insideMenu = false;
    currentMenu = m_start;
    //p_mode = pm_menu;
    paused = false;

    if (networkState != net_client)
	rebuildBoard = true;
    if (networkState != net_none)
	sendStop();
    
    emptyState();

    SDL_Event event;
    redisplay = true;
    //bool addTimer = true;
    /*gameModeValue++;*/
    SDL_AddTimer(BASE_INTERVAL, timer, (void*)pm_menu);
    while (p_mode == pm_menu) {
	if (rebuildBoard)
	    rebuildBoardFunc();
	
	if (redisplay) display();

	/*
	if (addTimer) {
	    SDL_AddTimer(BASE_INTERVAL, timer, (void*)gameModeValue);
	    addTimer = false;
	}
	*/
	
	SDL_WaitEvent(&event);
	
	switch (event.type) {
	    
	  case SDL_VIDEORESIZE:
	      prefs.windowSize[X] = event.resize.w;
	      prefs.windowSize[Y] = event.resize.h;
	      reshape();
	      break;
	      
	  case SDL_KEYDOWN:
	      //printf("key down: %s\n", SDL_GetKeyName(event.key.keysym.sym));
	      if (updateKeyPressed(&event.key.keysym))
		  keyboard(event.key.keysym);
	      break;
	      
	  case SDL_KEYUP:
	      updateKeyPressed(&event.key.keysym);
	      keyboardUp(event.key.keysym);
	      break;

	  case SDL_USEREVENT:
	      if (event.user.code == pm_menu) {
		  int times = 1;
		  SDL_Event e;
		  while (SDL_PeepEvents(&e, 1, SDL_PEEKEVENT, SDL_EVENTMASK(SDL_USEREVENT))) {
		      if (e.user.code == pm_menu) {
			  times++;
			  SDL_PeepEvents(&e, 1, SDL_GETEVENT, SDL_EVENTMASK(SDL_USEREVENT));
		      } else
			  break;
		  }
		  //printf("%i tickes\n", times);
		  tick(times);
		  //addTimer = true;
	      }
	      break;
	      
	  case SDL_QUIT:
	      quit(0);
	      break;
	}

	if (networkError) {
	    breakConnection();
	}
    }
}

