/*  
 *  snake3d, a snake game in a 3d world.
 *  Copyright (C) 2005-2006 Yair Yarom
 *  
 *  This program is free software; you can redistribute it and/or modify it under
 *  the terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or (at your option) any later
 *  version.
 *  
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 *  PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 *  Place, Suite 330, Boston, MA 02111-1307 USA
 *  
 *  http://sourceforge.net/projects/worms3d
 *  
 */


#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#if (defined __APPLE__ && defined __MACH__)
#include <SDL_net/SDL_net.h>
#else
#include <SDL_net.h>
#endif

#include "network.hpp"
#include "functions.hpp"
#include "preferences.hpp"
#include "menu.hpp"
#include "server.hpp"
#include "client.hpp"
#include "packets.hpp"

netState_t networkState = net_none;
ip_t myIP;

SDL_Thread* netThread = NULL;
bool networkError = false;
ai_t remoteSnakes[MAX_SNAKES] = {ai_remote, ai_remote, ai_remote, ai_remote};

//SDL_mutex* stepLock = NULL;
SDL_sem* stepLock = NULL;

bool reseting;
bool sentTick;
bool recvTick;
bool recvTicks[MAX_SNAKES];
bool recvReset[MAX_SNAKES];

int clientFood[3];

static bool networkInited = false;

void getMyIP(ip_t* ip) {    
    IPaddress sdlip;
    sdlip.host = ip->host;
    sdlip.port = ip->port;
    char localhostname[256];
    if((gethostname(localhostname, 256)>=0)) {
	if(SDLNet_ResolveHost(&sdlip,localhostname,0)==-1) {
	    fprintf(stderr, "Could not resolve host \"%s\"\n%s\n", localhostname, SDLNet_GetError());
	    quit(1);
	}
    }
    ip->host = sdlip.host;
    ip->port = sdlip.port;
    fixIP(ip);
}

void destNetwork() {
    if (!networkInited)
	return;
    
    //SDL_DestroyMutex(netLock);
    SDL_DestroySemaphore(netLock);
    //SDL_DestroyMutex(stepLock);
    SDL_DestroySemaphore(stepLock);
    breakConnection();
    SDLNet_Quit();
}

void initNetwork() {
    if (networkInited)
	return;
    
    if(SDLNet_Init() == -1) {
	fprintf(stderr, "SDLNet_Init: %s\n", SDLNet_GetError());
	quit(1);
    }
    //if ((stepLock = SDL_CreateMutex()) == NULL) {
    if ((stepLock = SDL_CreateSemaphore(1)) == NULL) {
	fprintf(stderr, "Couldn't create semaphore\n");
	quit(1);
    }
    //if ((netLock = SDL_CreateMutex()) == NULL) {
    if ((netLock = SDL_CreateSemaphore(1)) == NULL) {
	//fprintf(stderr, "Couldn't create mutex\n");
        fprintf(stderr, "Couldn't create semaphore\n");
	quit(1);
    }
    atexit(&destNetwork);
    getMyIP(&myIP);
    
    netThread = NULL;
    networkInited = true;
}

void fixIP(ip_t* ip) {
    IPaddress sdlip;
    if (ip->host != 0) {
	sdlip.host = ip->host;
	const char* temp = SDLNet_ResolveIP(&sdlip);
	if (temp != NULL)
	    strncpy(ip->name, temp, 255);
    } else if (ip->name[0] != 0) {
	if (SDLNet_ResolveHost(&sdlip, ip->name, ip->port) == 0)
	    ip->host = sdlip.host;
    }
}

void host2str(ip_t ip, char* str) {
    Uint32 host = SDLNet_Read32(&ip.host);
    sprintf(str, "%d.%d.%d.%d",
	    host >> 24,
	    (host >> 16) & 0xff,
	    (host >> 8) & 0xff,
	    host & 0xff);
}

void port2str(ip_t ip, char* str) {
    Uint16 port = SDLNet_Read16(&ip.port);
    sprintf(str, "%d", port);
}

void sendSnakeConfig(int snake) {
    switch (networkState) {
      case net_none: break;
      case net_server: serverSendSnakeConfig(snake); break;
      case net_client: clientSendSnakeConfig(snake); break;
    }
}

void sendBoardConfig() {
    if (networkState == net_server)
	serverSendBoardConfig();
}

void sendStart() {
    if (networkState == net_server)
	serverSendStart();
    else
	clientSendStart();
}

void sendStop() {
    if (networkState == net_server)
	serverSendStop();
    else
	clientSendStop();
}

void sendTick() {
    sentTick = true;
    if (networkState == net_server)
	serverSendTick();
    else
	clientSendTick();
}

void sendChangeDir(int snake, direction dir) {
    if (networkState == net_server)
	serverSendChangeDir(snake, dir);
    else
	clientSendChangeDir(snake, dir);
}

void sendInc(int snake) {
    if (networkState == net_server)
	serverSendInc(snake);
}

void sendFood() {
    if (networkState == net_server)
	serverSendFood();
}

void sendSpeed() {
    if (networkState == net_server)
	serverSendSpeed();
}

void sendReset() {
    if (networkState == net_server)
	serverSendReset();
}

void sendFineTuneSpeed(int snake, unsigned char amount) {
    if (networkState == net_server)
	serverSendFineTune(snake, amount);
}

void sendPause(bool pause) {
    switch (networkState) {
      case net_none: return; break;
      case net_client: clientSendPause(pause); break;
      case net_server: serverSendPause(pause); break;
    }
}

void sendToggleFineTune(bool fineTune) {
    if (networkState == net_server)
	serverSendToggleFineTune(fineTune);
}

void breakConnection() {
    DEBUG_PRINT("breaking connection\n");
    if (networkState == net_none)
	return;
    networkError = true;
    SDL_WaitThread(netThread, NULL);
    netThread = NULL;
    networkState = net_none;
    networkError = false;

    for (int i = 0; i < MAX_SNAKES; i++) {
	if (prefs.ai[i] == ai_remote || prefs.ai[i] == ai_human) {
	    //prefs.ai[i] = ai_off;
	    if (remoteSnakes[i] != ai_human && remoteSnakes[i] != ai_remote)
		prefs.ai[i] = remoteSnakes[i];
	    else
		prefs.ai[i] = ai_off;
	}
	remoteSnakes[i] = ai_remote;
    }
    prefs.ai[0] = ai_human;
    resetNames();
    
    switch (p_mode) {
      case pm_menu: rebuildBoard = true; break;
      case pm_game: p_mode = pm_menu; break;
      case pm_quit:;
    }
}

void startServer() {
    DEBUG_PRINT("starting server\n");
    if (networkState != net_none)
	return;
    //if (SPEEDS[prefs.stepInterval] < 2)
    //speedDown();
    initNetwork();
    networkState = net_server;
    netThread = SDL_CreateThread(&server ,NULL);
}

void startClient() {
    if (networkState != net_none)
	return;
    //if (SPEEDS[prefs.stepInterval] < 2)
    //speedDown();
    initNetwork();
    if (prefs.clientConnection.host == 0)
	return;
    networkState = net_client;
    netThread = SDL_CreateThread(&client, NULL);
}

