/*  
 *  snake3d, a snake game in a 3d world.
 *  Copyright (C) 2005-2006 Yair Yarom
 *  
 *  This program is free software; you can redistribute it and/or modify it under
 *  the terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or (at your option) any later
 *  version.
 *  
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 *  PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 *  Place, Suite 330, Boston, MA 02111-1307 USA
 *  
 *  http://sourceforge.net/projects/worms3d
 *  
 */

#ifndef NETWORK_HPP
#define NETWORK_HPP

#include "includes.hpp"
#include "consts.hpp"

typedef enum {
    net_none,
    net_server,
    net_client
} netState_t;

typedef struct {
    char name[255];
    Uint32 host;            /* 32-bit IPv4 host address */
    Uint16 port;            /* 16-bit protocol port */
} ip_t;

extern netState_t networkState;
extern ip_t myIP;
extern bool networkError;
extern ai_t remoteSnakes[MAX_SNAKES]; /* remote snake ai, ai_remote - not connected */

//extern SDL_mutex* stepLock;
extern SDL_sem* stepLock;

extern int clientFood[3];

extern bool reseting;
extern bool sentTick;
extern bool recvTick;
extern bool recvTicks[MAX_SNAKES];
extern bool recvReset[MAX_SNAKES];

void initNetwork();
void fixIP(ip_t* ip);
void host2str(ip_t ip, char* str);
void port2str(ip_t ip, char* str);

void sendSnakeConfig(int snake);
void sendBoardConfig();
void sendStart();
void sendStop();
void sendTick();
void sendChangeDir(int snake, direction dir);
void sendInc(int snake);
void sendFood();
void sendSpeed();
void sendReset();
void sendFineTuneSpeed(int snake, unsigned char amount);
void sendPause(bool pause);
void sendToggleFineTune(bool fineTune);
		  
void startServer();
void startClient();
void breakConnection();

inline void lockStep(const char* where) {    
    //DEBUG_PRINT2("(step lock: %s)\n", where);
    SDL_SemWait(stepLock);
};

inline void unlockStep(const char* where) {
    //DEBUG_PRINT2("(step unlock: %s)\n", where);
    SDL_SemPost(stepLock);
};

#endif
