/*  
 *  snake3d, a snake game in a 3d world.
 *  Copyright (C) 2005-2006 Yair Yarom
 *  
 *  This program is free software; you can redistribute it and/or modify it under
 *  the terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or (at your option) any later
 *  version.
 *  
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 *  PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 *  Place, Suite 330, Boston, MA 02111-1307 USA
 *  
 *  http://sourceforge.net/projects/worms3d
 *  
 */

#include <string.h>

#include "packets.hpp"
#include "preferences.hpp"
#include "global.hpp"
#include "board.hpp"

//SDL_mutex* netLock;
SDL_sem* netLock;

inline bool sendData(TCPsocket socket, unsigned char* data, int length) {
    SDL_SemWait(netLock);
    bool result = (SDLNet_TCP_Send(socket, data, length) == length);
    SDL_SemPost(netLock);
    return result;
}

inline bool recvData(TCPsocket socket, unsigned char* data, int length) {
    SDL_SemWait(netLock);
    bool result = (SDLNet_TCP_Recv(socket, data, length) == length);
    SDL_SemPost(netLock);
    return result;
}

bool sendPacket_Assign(TCPsocket socket, unsigned char snake) {
    unsigned char data[12];
    data[0] = ASSIGN_PACKET;
    data[1] = snake;
    strncpy((char*)data + 2, VERSION, 10);
    return sendData(socket, data, 12);
}

bool recvPacket_Assign(TCPsocket socket, unsigned char* snake) {
    unsigned char data[11];
    *snake = 0;
    if (!recvData(socket, data, 11))
	return false;
    *snake = 1;
    if (strcmp((char*)(data + 1),  VERSION) != 0)
	return false;
    *snake = data[0];
    return true;
}

bool sendPacket_SnakeConfig(TCPsocket socket, unsigned char snake) {
    unsigned char data[27];
    data[0] = SNAKE_CONFIG_PACKET;
    data[1] = snake;
    data[2] = prefs.snakeColor[snake];
    data[3] = prefs.ai[snake];
    data[4] = prefs.aiTarget[snake];
    data[5] = prefs.aitSnake[snake];
    data[6] = remoteSnakes[snake];
    strncpy((char*)data + 7, names[snake], 20);
    return sendData(socket, data, 27);
}

bool recvPacket_SnakeConfig(TCPsocket socket, unsigned char* snake) {
    unsigned char data[26];
    if (!recvData(socket, data, 26))
	//if (SDLNet_TCP_Recv(socket, data, 6) < 6)
	return false;
    *snake = data[0];
    prefs.snakeColor[*snake] = data[1];
    prefs.aiTarget[*snake] = (aiTarget_t)data[3];
    prefs.aitSnake[*snake] = data[4];
    if (networkState == net_server) {
	remoteSnakes[*snake] = (ai_t)data[2];
	prefs.ai[*snake] = (ai_t)data[5];
    } else {
	if (data[2] == ai_remote)
	    remoteSnakes[*snake] = (ai_t)data[5];
	else
	    remoteSnakes[*snake] = (ai_t)data[2];
	prefs.ai[*snake] = ai_remote; //(ai_t)data[2];
    }
    strncpy(names[*snake], (char*)data + 6, 20);
    return true;
}

bool sendPacket_BoardConfig(TCPsocket socket) {
    int packetSize = 6;
    unsigned char data[packetSize];
    data[0] = BOARD_CONFIG_PACKET;
    data[1] = prefs.currentBoardType;
    data[2] = prefs.currentBoard;
    const int* food = mainBoard->getFood();
    data[3] = food[X];
    data[4] = food[Y];
    data[5] = food[Z];
    /*
    if (SDLNet_TCP_Send(socket, data, packetSize) < packetSize)
	return false;
    return true;
    */
    return sendData(socket, data, packetSize);
}

bool recvPacket_BoardConfig(TCPsocket socket, int* food) {
    int packetSize = 5;
    unsigned char data[packetSize];
    //if (SDLNet_TCP_Recv(socket, data, packetSize) < packetSize)
    if (!recvData(socket, data, packetSize))
	return false;
    prefs.currentBoardType = data[0];
    prefs.currentBoard = data[1];
    food[X] = data[2];
    food[Y] = data[3];
    food[Z] = data[4];
    return true;
}

bool sendPacket_Start(TCPsocket socket) {
    unsigned char data = START_PACKET;
    /*
    if (SDLNet_TCP_Send(socket, &data, 1) < 1)
	return false;
    return true;
    */
    return sendData(socket, &data, 1);
}

bool sendPacket_Stop(TCPsocket socket) {
    unsigned char data = STOP_PACKET;
    /*
    if (SDLNet_TCP_Send(socket, &data, 1) < 1)
	return false;
    return true;
    */
    return sendData(socket, &data, 1);
}

bool sendPacket_Tick(TCPsocket socket) {
    unsigned char data = TICK_PACKET;
    /*
    if (SDLNet_TCP_Send(socket, &data, 1) < 1)
	return false;
    return true;
    */
    return sendData(socket, &data, 1);
}

bool sendPacket_ChangeDir(TCPsocket socket, unsigned char snake, direction dir) {
    int packetSize = 3;
    unsigned char data[packetSize];
    data[0] = CHANGE_DIR_PACKET;
    data[1] = snake;
    data[2] = dir;
    /*
    if (SDLNet_TCP_Send(socket, data, packetSize) < packetSize)
	return false;
    return true;
    */
    return sendData(socket, data, packetSize);
}

bool recvPacket_ChangeDir(TCPsocket socket, unsigned char* snake, direction* dir) {
    int packetSize = 2;
    unsigned char data[packetSize];
    //if (SDLNet_TCP_Recv(socket, data, packetSize) < packetSize)
    if (!recvData(socket, data, packetSize))
	return false;
    *snake = data[0];
    *dir = (direction)data[1];
    return true;
}

bool sendPacket_Inc(TCPsocket socket, unsigned char snake) {
    int packetSize = 2;
    unsigned char data[packetSize];
    data[0] = INC_PACKET;
    data[1] = snake;
    /*
    if (SDLNet_TCP_Send(socket, data, packetSize) < packetSize)
	return false;
    return true;
    */
    return sendData(socket, data, packetSize);
}

bool recvPacket_Inc(TCPsocket socket, unsigned char* snake) {
    return recvData(socket, snake, 1);
    /*
    if (SDLNet_TCP_Recv(socket, snake, 1) < 1)
	return false;
    return true;
    */
}

bool sendPacket_Food(TCPsocket socket) {
    int packetSize = 4;
    unsigned char data[packetSize];
    const int* food = mainBoard->getFood();
    data[0] = FOOD_PACKET;
    data[1] = food[X];
    data[2] = food[Y];
    data[3] = food[Z];
    /*
    if (SDLNet_TCP_Send(socket, data, packetSize) < packetSize)
	return false;
    return true;
    */
    return sendData(socket, data, packetSize);
}

bool recvPacket_Food(TCPsocket socket, int* food) {
    int packetSize = 3;
    unsigned char data[packetSize];
    //if (SDLNet_TCP_Recv(socket, data, packetSize) < packetSize)
    if (!recvData(socket, data, packetSize))
	return false;
    food[X] = data[0];
    food[Y] = data[1];
    food[Z] = data[2];
    return true;
}

bool sendPacket_Speed(TCPsocket socket) {
    int packetSize = 2;
    unsigned char data[packetSize];
    data[0] = SPEED_PACKET;
    data[1] = prefs.stepInterval;
    return sendData(socket, data, packetSize);
}

bool recvPacket_Speed(TCPsocket socket, unsigned char* speed) {
    return recvData(socket, speed, 1);
}

bool sendPacket_Reset(TCPsocket socket) {
    unsigned char data = RESET_PACKET;
    return sendData(socket, &data, 1);
}

bool sendPacket_FineTune(TCPsocket socket, unsigned char amount) {
    int packetSize = 2;
    unsigned char data[packetSize];
    data[0] = FINE_TUNE_PACKET;
    data[1] = amount;
    return sendData(socket, data, packetSize);
}

bool recvPacket_FineTune(TCPsocket socket, unsigned char* amount) {
    return recvData(socket, amount, 1);
}

bool sendPacket_Pause(TCPsocket socket, unsigned char pause) {
    int packetSize = 2;
    unsigned char data[packetSize];
    data[0] = PAUSE_PACKET;
    data[1] = pause;
    return sendData(socket, data, packetSize);
}

bool recvPacket_Pause(TCPsocket socket, unsigned char* pause) {
    return recvData(socket, pause, 1);
}

bool sendPacket_ToggleFineTune(TCPsocket socket, unsigned char fineTune) {
    int packetSize = 2;
    unsigned char data[packetSize];
    data[0] = TOGGLE_FINE_TUNE_PACKET;
    data[1] = fineTune;
    return sendData(socket, data, packetSize);    
}

bool recvPacket_ToggleFineTune(TCPsocket socket, unsigned char* fineTune) {
    return recvData(socket, fineTune, 1);
}

