/*  
 *  snake3d, a snake game in a 3d world.
 *  Copyright (C) 2005-2006 Yair Yarom
 *  
 *  This program is free software; you can redistribute it and/or modify it under
 *  the terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or (at your option) any later
 *  version.
 *  
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 *  PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 *  Place, Suite 330, Boston, MA 02111-1307 USA
 *  
 *  http://sourceforge.net/projects/worms3d
 *  
 */

#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <arpa/inet.h>

#include "preferences.hpp"
#include "global.hpp"
#include "functions.hpp"

char PREFS_FILE[255] = {0};

void defaultRotation(bool callReshape){
    static const float IDENT[16] = {
	1.0, 0.0, 0.0, 0.0,
	0.0, 1.0, 0.0, 0.0,
	0.0, 0.0, 1.0, 0.0,
	0.0, 0.0, 0.0, 1.0
    };
    memcpy(prefs.boardRotationMatrix, IDENT, sizeof(prefs.boardRotationMatrix));
    prefs.fieldOfView = INIT_FIELD_OF_VIEW;
    //if (callReshape) reshape(prefs.windowSize[X], prefs.windowSize[Y]);
    if (callReshape) loadPerspectives();
    emptyState();
}

void defaultKeys() {
    for (int i = 0; i < NKEYS; i++) {
	prefs.SDLkeys[i] = DEFAULT_KEYS_SDL[i];
    }
}

void defaultPreferences(bool reshape) {
    defaultRotation(reshape);
    defaultKeys();
    
    prefs.gridLevel1 = 3;
    prefs.gridLevel3 = 1;
    prefs.showWalls = true;
    prefs.fullscreen = false;
    prefs.backgroundColor = BROWN;
    prefs.stepInterval = DEFAULT_STEP_INTERVAL;
    prefs.windowSize[X] = 800;
    prefs.windowSize[Y] = 600;
    prefs.realWindowSize[X] = 800;
    prefs.realWindowSize[Y] = 600;
    prefs.currentBoard = DEFAULT_BOARD;
    prefs.currentBoardType = DEFAULT_BOARD_TYPE;
    prefs.serverPort = htons(DEFAULT_PORT);
    //getMyIP(&prefs.clientConnection);
    strcpy(prefs.clientConnection.name, myIP.name);
    prefs.clientConnection.host = myIP.host;
    //prefs.clientConnection.host = 0;
    prefs.clientConnection.port = htons(DEFAULT_PORT);
    if (nResolutions > 0) {
	prefs.resolution = defaultResolution;
	prefs.fullScreenResolution[X] = resolutions[defaultResolution][X];
	prefs.fullScreenResolution[Y] = resolutions[defaultResolution][Y];
    }
    for (int i = 0; i < MAX_SNAKES; i++) {
	prefs.ai[i] = ai_basic;
	prefs.aiTarget[i] = ait_food;
	prefs.aitSnake[i] = 0;
    }
    prefs.ai[0] = ai_human;
    prefs.snakeColor[0] = GREEN;
    prefs.snakeColor[1] = BLUE;
    prefs.snakeColor[2] = MAGENTA;
    prefs.snakeColor[3] = WHITE;
    prefs.fineTune = false;
    prefs.showIndicator = true;
    if (getenv("USER")) {
	strncpy(prefs.name, getenv("USER"), 20);
    } else {
	strcpy(prefs.name, "me");
    }
    resetNames();
};

void getPreferences(bool reshape) {
    if (!loadPreferences(reshape))
	defaultPreferences(reshape);
};

void buildPREFS_FILE() {
    //char* file = stpcpy(PREFS_FILE, getenv("HOME"));
    if (getenv("HOME")) {
	strncpy(PREFS_FILE, getenv("HOME"), 244);
	char* file = (char*)memchr(PREFS_FILE, 0x0, 244);
	strcpy(file, "/.snake3d");
    } 
}

bool loadPreferences(bool callReshape) {
    if (!*PREFS_FILE) buildPREFS_FILE();
    if (!*PREFS_FILE) {
	//fprintf(stderr, "can't load preferences file, don't have a HOME environment, getting defaults");
	return false;
    }
    FILE*  file = fopen(PREFS_FILE, "r");
    if (!file) {
	return false;
    }
    char str[80];
    if (!fscanf(file, "%s\n", str)) {
	fclose(file);
	return false;
    }
    if (strcmp(str, VERSION) != 0) {
	fclose(file);
	return false;
    }
    snprintf(str, 80, "%%%luc", (long unsigned)sizeof(prefs));
    fscanf(file, str, &prefs);
    fclose(file);

    //if (callReshape) reshape(prefs.windowSize[X], prefs.windowSize[Y]);
    if (callReshape) loadPerspectives();
    emptyState();

    for (int i = 0; i < MAX_SNAKES; i++) {
	if (prefs.ai[i] == ai_remote || prefs.ai[i] == ai_human)
	    prefs.ai[i] = ai_basic;
    }
    prefs.ai[0] = ai_human;
    return true;
};

bool savePreferences() {
    if (!*PREFS_FILE) buildPREFS_FILE();
    if (!*PREFS_FILE) {
	//fprintf(stderr, "can't save preferences file, don't have a HOME environment, getting defaults");
	return false;	
    }
    int file = open(PREFS_FILE, O_WRONLY | O_CREAT, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
    if (file < 0)
	return false;
    write(file, VERSION, strlen(VERSION));
    write(file, "\n", 1);
    write(file, &prefs, sizeof(prefs));
    close(file);
    return true;
};

