/*  
 *  snake3d, a snake game in a 3d world.
 *  Copyright (C) 2005-2006 Yair Yarom
 *  
 *  This program is free software; you can redistribute it and/or modify it under
 *  the terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or (at your option) any later
 *  version.
 *  
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 *  PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 *  Place, Suite 330, Boston, MA 02111-1307 USA
 *  
 *  http://sourceforge.net/projects/worms3d
 *  
 */

#include "includes.hpp"
#include "rooms2.hpp"

Rooms2::Rooms2(int size) {
    _roomSize = size + (size % 2);
    _corSize = _roomSize / 2;
    _corSize = _corSize + (_corSize % 2);

    _maxSize = _roomSize * 2 + _corSize;
    _startingLocations = new int*[MAX_SNAKES];
    _startingDirections = new direction[MAX_SNAKES];
    _startingUps = new direction[MAX_SNAKES];
    for (int i = 0; i < MAX_SNAKES; i++)
	_startingLocations[i] = new int[4];

    int roomStart = (_corSize + _roomSize) / 2;
    int roomEnd = roomStart + _roomSize;

    _startingLocations[0][X] = 0;
    _startingLocations[0][Y] = roomStart;
    _startingLocations[0][Z] = roomStart;
    _startingDirections[0] = right;
    _startingUps[0] = up;
    
    _startingLocations[1][X] = (int)_maxSize - 1;
    _startingLocations[1][Y] = roomEnd - 1;
    _startingLocations[1][Z] = roomStart;
    _startingDirections[1] = left;
    _startingUps[1] = down;

    _startingLocations[2][X] = 0;
    _startingLocations[2][Y] = roomEnd - 1;
    _startingLocations[2][Z] = roomEnd - 1;
    _startingDirections[2] = right;
    _startingUps[2] = down;

    _startingLocations[3][X] = (int)_maxSize - 1;
    _startingLocations[3][Y] = roomStart;
    _startingLocations[3][Z] = roomEnd - 1;
    _startingDirections[3] = left;
    _startingUps[3] = up;

}

occupancy_t Rooms2::checkLocation(int* location, Snake* snake) {
    int roomStart = (_corSize + _roomSize) / 2;
    int roomEnd = roomStart + _roomSize;
    int roomStart2 = _corSize + _roomSize;
    
    if (location[X] < 0         || location[X] >= _maxSize || 
	location[Y] < roomStart || location[Y] >= roomEnd  ||
	location[Z] < roomStart || location[Z] >= roomEnd  ||
	(location[X] >= _roomSize && location[X] < roomStart2 &&
	 !(location[Y] < _maxSize / 2 + 1 && location[Y] >= _maxSize / 2 - 1 &&
	  location[Z] < _maxSize / 2 + 1 && location[Z] >= _maxSize / 2 - 1)))
	return o_outside;
    return Board::checkLocation(location, snake);
}

void Rooms2::drawFrame() {
    // glColor3fv(COLORS[LIGHT_GRAY]);
    // glPushMatrix();
    // glTranslatef(_maxSize / 2, _maxSize / 2, _maxSize / 2);
    // glutWireCube(_maxSize);
    // glPopMatrix();

    glColor3fv(COLORS[WHITE]);
    glLineWidth(2);

    glPushMatrix();
    glTranslatef(_roomSize / 2, _maxSize / 2, _maxSize / 2);
    glutWireCube(_roomSize);
    glTranslatef(_corSize + _roomSize, 0, 0);
    glutWireCube(_roomSize);
    glPopMatrix();

    glPushMatrix();
    glTranslatef(_maxSize / 2, _maxSize / 2, _maxSize / 2);
    glScalef(_corSize / 2, 1, 1);
    glutWireCube(2);
    glPopMatrix();

    glLineWidth(1);
};

void Rooms2::drawGrid(int grid) {
    int roomStart = (_corSize + _roomSize) / 2;
    int roomEnd = roomStart + _roomSize;
    int roomStart2 = _corSize + _roomSize;

    glColor3fv(COLORS[LIGHT_GRAY]);
    glBegin(GL_LINES);

    switch (grid) {	  
      case 3:
	  for (int i = 1; i < _roomSize; i++) {
	      for (int j = 1; j < _roomSize; j++) {
		  // room1
		  glVertex3d(i, roomStart + j, roomStart);
		  glVertex3d(i, roomStart + j, roomEnd);
		  glVertex3d(0        , roomStart + j, roomStart + i);
		  glVertex3d(_roomSize, roomStart + j, roomStart + i);
		  glVertex3d(i, roomStart, roomStart + j);
		  glVertex3d(i, roomEnd,   roomStart + j);
		  
		  // room2
		  glVertex3d(roomStart2 + i, roomStart + j, roomStart);
		  glVertex3d(roomStart2 + i, roomStart + j, roomEnd);
		  glVertex3d(roomStart2, roomStart + j, roomStart + i);
		  glVertex3d(_maxSize,   roomStart + j, roomStart + i);
		  glVertex3d(roomStart2 + i, roomStart, roomStart + j);
		  glVertex3d(roomStart2 + i, roomEnd,   roomStart + j);
	      }
	  }
	  
	  // cor
	  for (int i = 0; i <= _corSize; i++) {
	      glVertex3d(_roomSize + i, _maxSize / 2 - 1, _maxSize / 2);
	      glVertex3d(_roomSize + i, _maxSize / 2 + 1, _maxSize / 2);
	      glVertex3d(_roomSize + i, _maxSize / 2, _maxSize / 2 - 1);
	      glVertex3d(_roomSize + i, _maxSize / 2, _maxSize / 2 + 1);
	  }

      case 2:	  	  
	  for (int i = 1; i < _roomSize; i++) {
	      // up1
	      glVertex3d(i, roomEnd, roomStart);
	      glVertex3d(i, roomEnd, roomEnd);
	      glVertex3d(0, roomEnd, roomStart + i);
	      glVertex3d(_roomSize, roomEnd, roomStart + i);
	      
	      // front1
	      glVertex3d(i, roomStart, roomEnd);
	      glVertex3d(i, roomEnd, roomEnd);
	      glVertex3d(0, roomStart + i, roomEnd);
	      glVertex3d(_roomSize, roomStart + i, roomEnd);

	      // right2
	      glVertex3d(_maxSize, roomStart, roomStart + i);
	      glVertex3d(_maxSize, roomEnd, roomStart + i);
	      glVertex3d(_maxSize, roomStart + i, roomStart);
	      glVertex3d(_maxSize, roomStart + i, roomEnd);

	      // up2
	      glVertex3d(roomStart2 + i, roomEnd, roomStart);
	      glVertex3d(roomStart2 + i, roomEnd, roomEnd);
	      glVertex3d(roomStart2, roomEnd, roomStart + i);
	      glVertex3d(_maxSize, roomEnd, roomStart + i);

	      // back2
	      glVertex3d(roomStart2 + i, roomStart, roomEnd);
	      glVertex3d(roomStart2 + i, roomEnd, roomEnd);
	      glVertex3d(roomStart2, roomStart + i, roomEnd);
	      glVertex3d(_maxSize, roomStart + i, roomEnd);
	      
	  }

	  // right1
	  for (int i = 1; i < _roomSize / 2; i++) {
	      glVertex3d(_roomSize, roomStart, roomStart + i);
	      glVertex3d(_roomSize, roomEnd, roomStart + i);
	      glVertex3d(_roomSize, roomStart + i, roomStart);
	      glVertex3d(_roomSize, roomStart + i, roomEnd);
	      glVertex3d(_roomSize, roomStart, roomEnd - i);
	      glVertex3d(_roomSize, roomEnd, roomEnd - i);
	      glVertex3d(_roomSize, roomEnd -  i, roomStart);
	      glVertex3d(_roomSize, roomEnd -  i, roomEnd);
	  }
	  glVertex3d(_roomSize, roomStart, roomStart + _roomSize / 2);
	  glVertex3d(_roomSize, roomStart + _roomSize / 2 - 1, roomStart + _roomSize / 2);
	  glVertex3d(_roomSize, roomStart + _roomSize / 2 + 1, roomStart + _roomSize / 2);
	  glVertex3d(_roomSize, roomEnd, roomStart + _roomSize / 2);
	  glVertex3d(_roomSize, roomStart + _roomSize / 2, roomStart);
	  glVertex3d(_roomSize, roomStart + _roomSize / 2, roomStart + _roomSize / 2 - 1);
	  glVertex3d(_roomSize, roomStart + _roomSize / 2, roomStart + _roomSize / 2 + 1);
	  glVertex3d(_roomSize, roomStart + _roomSize / 2, roomEnd);

	  // up3
	  glVertex3d(_roomSize,            _maxSize / 2 + 1, _maxSize / 2);
	  glVertex3d(_roomSize + _corSize, _maxSize / 2 + 1, _maxSize / 2);

	  // front3
	  glVertex3d(_roomSize,            _maxSize / 2, _maxSize / 2 + 1);
	  glVertex3d(_roomSize + _corSize, _maxSize / 2, _maxSize / 2 + 1);
	  
	  for (int i = 1; i < _corSize; i++) {
	      // up3
	      glVertex3d(_roomSize + i, _maxSize / 2 + 1, _maxSize / 2 - 1);
	      glVertex3d(_roomSize + i, _maxSize / 2 + 1, _maxSize / 2 + 1);

	      // front3
	      glVertex3d(_roomSize + i, _maxSize / 2 - 1, _maxSize / 2 + 1);
	      glVertex3d(_roomSize + i, _maxSize / 2 + 1, _maxSize / 2 + 1);
	  }

      case 1:	  
	  for (int i = 1; i < _roomSize; i++) {
	      // down1
	      glVertex3d(i, roomStart, roomStart);
	      glVertex3d(i, roomStart, roomEnd);
	      glVertex3d(0, roomStart, roomStart + i);
	      glVertex3d(_roomSize, roomStart, roomStart + i);
	      
	      // back1
	      glVertex3d(i, roomStart, roomStart);
	      glVertex3d(i, roomEnd, roomStart);
	      glVertex3d(0, roomStart + i, roomStart);
	      glVertex3d(_roomSize, roomStart + i, roomStart);

	      // left1
	      glVertex3d(0, roomStart, roomStart + i);
	      glVertex3d(0, roomEnd, roomStart + i);
	      glVertex3d(0, roomStart + i, roomStart);
	      glVertex3d(0, roomStart + i, roomEnd);

	      // down2
	      glVertex3d(roomStart2 + i, roomStart, roomStart);
	      glVertex3d(roomStart2 + i, roomStart, roomEnd);
	      glVertex3d(roomStart2, roomStart, roomStart + i);
	      glVertex3d(_maxSize, roomStart, roomStart + i);

	      // back2
	      glVertex3d(roomStart2 + i, roomStart, roomStart);
	      glVertex3d(roomStart2 + i, roomEnd, roomStart);
	      glVertex3d(roomStart2, roomStart + i, roomStart);
	      glVertex3d(_maxSize, roomStart + i, roomStart);
	      
	  }

	  // left2
	  for (int i = 1; i < _roomSize / 2; i++) {
	      glVertex3d(roomStart2, roomStart, roomStart + i);
	      glVertex3d(roomStart2, roomEnd, roomStart + i);
	      glVertex3d(roomStart2, roomStart + i, roomStart);
	      glVertex3d(roomStart2, roomStart + i, roomEnd);
	      glVertex3d(roomStart2, roomStart, roomEnd - i);
	      glVertex3d(roomStart2, roomEnd, roomEnd - i);
	      glVertex3d(roomStart2, roomEnd -  i, roomStart);
	      glVertex3d(roomStart2, roomEnd -  i, roomEnd);
	  }
	  glVertex3d(roomStart2, roomStart, roomStart + _roomSize / 2);
	  glVertex3d(roomStart2, roomStart + _roomSize / 2 - 1, roomStart + _roomSize / 2);
	  glVertex3d(roomStart2, roomStart + _roomSize / 2 + 1, roomStart + _roomSize / 2);
	  glVertex3d(roomStart2, roomEnd, roomStart + _roomSize / 2);
	  glVertex3d(roomStart2, roomStart + _roomSize / 2, roomStart);
	  glVertex3d(roomStart2, roomStart + _roomSize / 2, roomStart + _roomSize / 2 - 1);
	  glVertex3d(roomStart2, roomStart + _roomSize / 2, roomStart + _roomSize / 2 + 1);
	  glVertex3d(roomStart2, roomStart + _roomSize / 2, roomEnd);

	  // down3
	  glVertex3d(_roomSize, _maxSize / 2 - 1, _maxSize / 2);
	  glVertex3d(_roomSize + _corSize, _maxSize / 2 - 1, _maxSize / 2);

	  // back3
	  glVertex3d(_roomSize, _maxSize / 2, _maxSize / 2 - 1);
	  glVertex3d(_roomSize + _corSize, _maxSize / 2, _maxSize / 2 - 1);
	  
	  for (int i = 1; i < _corSize; i++) {
	      // down3
	      glVertex3d(_roomSize + i, _maxSize / 2 - 1, _maxSize / 2 - 1);
	      glVertex3d(_roomSize + i, _maxSize / 2 - 1, _maxSize / 2 + 1);

	      // back3
	      glVertex3d(_roomSize + i, _maxSize / 2 - 1, _maxSize / 2 - 1);
	      glVertex3d(_roomSize + i, _maxSize / 2 + 1, _maxSize / 2 - 1);
	  }

    }
    glEnd();
};

void Rooms2::drawWalls() {
    glEnable(GL_BLEND);
    glDepthMask(GL_FALSE);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE);
    glMaterialfv(GL_FRONT, GL_AMBIENT, WALL_AMBIENT);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, WALL_DIFFUSE);
    glMaterialfv(GL_FRONT, GL_SPECULAR, WALL_SPECULAR);
    glMaterialf(GL_FRONT, GL_SHININESS, WALL_SHININESS);
    glMaterialfv(GL_FRONT, GL_EMISSION, WALL_EMISSION);
    glColor4fv(WALL_DIFFUSE);
    glPushMatrix();

    int roomStart = (_corSize + _roomSize) / 2;
    int roomEnd = roomStart + _roomSize;
    int roomStart2 = _corSize + _roomSize;
    
    glBegin(GL_QUADS);
    // left
    glNormal3d(-1, 0, 0);

    // left1
    glVertex3d(0, roomStart, roomStart);
    glVertex3d(0, roomStart, roomEnd);
    glVertex3d(0, roomEnd,   roomEnd);
    glVertex3d(0, roomEnd,   roomStart);

    // left2
    glVertex3d(roomStart2, roomStart, 	     roomStart);
    glVertex3d(roomStart2, roomStart, 	     roomEnd);
    glVertex3d(roomStart2, _maxSize / 2 - 1, roomEnd);
    glVertex3d(roomStart2, _maxSize / 2 - 1, roomStart);

    glVertex3d(roomStart2, _maxSize /2 + 1, roomStart);
    glVertex3d(roomStart2, _maxSize /2 + 1, roomEnd);
    glVertex3d(roomStart2, roomEnd,         roomEnd);
    glVertex3d(roomStart2, roomEnd,         roomStart);
    
    glVertex3d(roomStart2, _maxSize / 2 - 1, roomStart);
    glVertex3d(roomStart2, _maxSize / 2 - 1, _maxSize / 2 - 1);
    glVertex3d(roomStart2, _maxSize / 2 + 1, _maxSize / 2 - 1);
    glVertex3d(roomStart2, _maxSize / 2 + 1, roomStart);
    
    glVertex3d(roomStart2, _maxSize / 2 - 1, _maxSize / 2 + 1);
    glVertex3d(roomStart2, _maxSize / 2 - 1, roomEnd);
    glVertex3d(roomStart2, _maxSize / 2 + 1, roomEnd);
    glVertex3d(roomStart2, _maxSize / 2 + 1, _maxSize / 2 + 1);

    // right
    glNormal3d(1, 0, 0);

    // right2
    glVertex3d(_maxSize, roomStart, roomStart);
    glVertex3d(_maxSize, roomStart, roomEnd);
    glVertex3d(_maxSize, roomEnd,   roomEnd);
    glVertex3d(_maxSize, roomEnd,   roomStart);

    // right1
    glVertex3d(_roomSize, roomStart,  roomStart);
    glVertex3d(_roomSize, roomStart, roomEnd);
    glVertex3d(_roomSize, _maxSize / 2 - 1, roomEnd);
    glVertex3d(_roomSize, _maxSize / 2 - 1,  roomStart);
    
    glVertex3d(_roomSize, _maxSize / 2 + 1,  roomStart);
    glVertex3d(_roomSize, _maxSize / 2 + 1, roomEnd);
    glVertex3d(_roomSize, roomEnd, roomEnd);
    glVertex3d(_roomSize, roomEnd,  roomStart);
    
    glVertex3d(_roomSize, _maxSize / 2 - 1, roomStart);
    glVertex3d(_roomSize, _maxSize / 2 - 1, _maxSize / 2 - 1);
    glVertex3d(_roomSize, _maxSize / 2 + 1, _maxSize / 2 - 1);
    glVertex3d(_roomSize, _maxSize / 2 + 1, roomStart);
    
    glVertex3d(_roomSize,  _maxSize / 2 - 1, _maxSize / 2 + 1);
    glVertex3d(_roomSize,  _maxSize / 2 - 1, roomEnd);
    glVertex3d(_roomSize, _maxSize / 2 + 1, roomEnd);
    glVertex3d(_roomSize, _maxSize / 2 + 1, _maxSize / 2 + 1);
    
    // up
    glNormal3d(0, 1, 0);
    
    // up1
    glVertex3d(0, roomEnd,  roomStart);
    glVertex3d(0, roomEnd, roomEnd);
    glVertex3d(_roomSize, roomEnd, roomEnd);
    glVertex3d(_roomSize, roomEnd,  roomStart);

    // up2
    glVertex3d(roomStart2, roomEnd,  roomStart);
    glVertex3d(roomStart2, roomEnd, roomEnd);
    glVertex3d(_maxSize, roomEnd, roomEnd);
    glVertex3d(_maxSize, roomEnd,  roomStart);

    // up3
    glVertex3d( _roomSize, _maxSize / 2 + 1,  _maxSize / 2 - 1);
    glVertex3d( _roomSize, _maxSize / 2 + 1, _maxSize / 2 + 1);
    glVertex3d(roomStart2, _maxSize / 2 + 1, _maxSize / 2 + 1);
    glVertex3d(roomStart2, _maxSize / 2 + 1,  _maxSize / 2 - 1);
    
    // down
    glNormal3d(0, -1, 0);

    // down1
    glVertex3d(0, roomStart,  roomStart);
    glVertex3d(0, roomStart, roomEnd);
    glVertex3d(_roomSize, roomStart, roomEnd);
    glVertex3d(_roomSize, roomStart,  roomStart);

    // down3
    glVertex3d( _roomSize, _maxSize / 2 - 1,  _maxSize / 2 - 1);
    glVertex3d( _roomSize, _maxSize / 2 - 1, _maxSize / 2 + 1);
    glVertex3d(roomStart2, _maxSize / 2 - 1, _maxSize / 2 + 1);
    glVertex3d(roomStart2, _maxSize / 2 - 1,  _maxSize / 2 - 1);

    // down2
    glVertex3d(roomStart2, roomStart,  roomStart);
    glVertex3d(roomStart2, roomStart, roomEnd);
    glVertex3d(_maxSize, roomStart, roomEnd);
    glVertex3d(_maxSize, roomStart,  roomStart);
    
    // forward
    glNormal3d(0, 0, -1);

    // forward1
    glVertex3d(0, roomStart, roomStart);
    glVertex3d(0, roomEnd, roomStart);
    glVertex3d(_roomSize, roomEnd, roomStart);
    glVertex3d(_roomSize, roomStart, roomStart);

    // forward3
    glVertex3d( _roomSize, _maxSize / 2 - 1, _maxSize / 2 - 1);
    glVertex3d( _roomSize, _maxSize / 2 + 1, _maxSize / 2 - 1);
    glVertex3d(roomStart2, _maxSize / 2 + 1, _maxSize / 2 - 1);
    glVertex3d(roomStart2, _maxSize / 2 - 1, _maxSize / 2 - 1);

    // forward2
    glVertex3d(roomStart2, roomStart, roomStart);
    glVertex3d(roomStart2, roomEnd, roomStart);
    glVertex3d(_maxSize, roomEnd, roomStart);
    glVertex3d(_maxSize, roomStart, roomStart);
    
    // backward
    glNormal3d(0, 0, 1);

    // backward1
    glVertex3d(0, roomStart, roomEnd);
    glVertex3d(0, roomEnd, roomEnd);
    glVertex3d(_roomSize, roomEnd, roomEnd);
    glVertex3d(_roomSize, roomStart, roomEnd);

    // backward3
    glVertex3d( _roomSize, _maxSize / 2 - 1, _maxSize / 2 + 1);
    glVertex3d( _roomSize, _maxSize / 2 + 1, _maxSize / 2 + 1);
    glVertex3d(roomStart2, _maxSize / 2 + 1, _maxSize / 2 + 1);
    glVertex3d(roomStart2, _maxSize / 2 - 1, _maxSize / 2 + 1);

    // backward2
    glVertex3d(roomStart2, roomStart, roomEnd);
    glVertex3d(roomStart2, roomEnd, roomEnd);
    glVertex3d(_maxSize, roomEnd, roomEnd);
    glVertex3d(_maxSize, roomStart, roomEnd);
    glEnd();

    glPopMatrix();
    glDepthMask(GL_TRUE);
    glDisable(GL_BLEND);
}

Board* Rooms2::clone() {
    return new Rooms2(_roomSize);
}

const char* Rooms2::getName() {
    static int tinyRoomSize   = TINY_BOARD   + (TINY_BOARD % 2);
    static int smallRoomSize  = SMALL_BOARD  + (SMALL_BOARD % 2);
    static int mediumRoomSize = MEDIUM_BOARD + (MEDIUM_BOARD % 2);
    static int largeRoomSize  = LARGE_BOARD  + (LARGE_BOARD % 2);
    static int hugeRoomSize   = HUGE_BOARD   + (HUGE_BOARD % 2);
    if      (_roomSize == tinyRoomSize)   return "2 rooms(small)";
    else if (_roomSize == smallRoomSize)  return "2 rooms(medium)";
    else if (_roomSize == mediumRoomSize) return "2 rooms(large)";
    else if (_roomSize == largeRoomSize)  return "2 rooms(huge)";
    else if (_roomSize == hugeRoomSize)   return "2 rooms(extra huge)";
    return "2 rooms(unknown)";
};
