/*  
 *  snake3d, a snake game in a 3d world.
 *  Copyright (C) 2005-2006 Yair Yarom
 *  
 *  This program is free software; you can redistribute it and/or modify it under
 *  the terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or (at your option) any later
 *  version.
 *  
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 *  PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 *  Place, Suite 330, Boston, MA 02111-1307 USA
 *  
 *  http://sourceforge.net/projects/worms3d
 *  
 */

#ifndef SNAKE_HPP
#define SNAKE_HPP

#include "types.hpp"

class Board;

class Snake {
    friend class Board;
    
private:
    static const int HAS_FOOD = 3;
    static const int ENTER = 4;
    static const int EXIT = 5;
    int** _joints; // X, Y, Z, has food, enter(old) direction, exit(new) direction
    int _length;
    int _currentLength;
    ai_t _autoPilot;
    aiTarget_t _aiTarget;
    Snake* _aitSnake;
    const direction _startDir;
    const direction _startUp;
    direction _dir;
    direction _up;
    direction _oldDir;
    direction _oldUp;
    bool _dead;
    //bool _longDead;
    int _start[3];
    int _color;
    int _id;
    
    Snake(int x, int y, int z, direction dir, direction up, int color, int id);
    void move();
    void kill();
    void drawJoint(int joint);
    void draw(bool withHead);
    void reset();
    occupancy_t collision(int* location, bool me);

public:
    void inc();
    void autoPilot(ai_t apMode, aiTarget_t aiTarget, Snake* aitSnake);
    ai_t getAI() {return _autoPilot;}
    void changeDirection(direction dir);
    void changeDirection() {changeDirection(_dir);}
    void changeSelfDirection(direction dir);
    void changeYaw(direction dir);
    int getColor();
    int getLength();
    direction getDirection() {return _dir;}
    void headLocation(int location[3]);
    bool isDead();
    void lookAt();
    ~Snake();
};

#endif
