/*  
 *  snake3d, a snake game in a 3d world.
 *  Copyright (C) 2005-2006 Yair Yarom
 *  
 *  This program is free software; you can redistribute it and/or modify it under
 *  the terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or (at your option) any later
 *  version.
 *  
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 *  PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License along with
 *  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 *  Place, Suite 330, Boston, MA 02111-1307 USA
 *  
 *  http://sourceforge.net/projects/worms3d
 *  
 */


#include <stdlib.h>
#include <sys/time.h>
#include <cmath>
#include <string.h>
#include <signal.h>

#include "global.hpp"
#include "functions.hpp"
#include "menu.hpp"
#include "game.hpp"
#include "preferences.hpp"
#include "cube.hpp"
#include "rooms2.hpp"
#include "classic2d.hpp"
#include "network.hpp"

void quitHandler(int signal) {
    //printf("signal: %i (", signal);
    //switch (signal) {
    //  case 0:printf("?)\n"); break;
    //  case 1:printf("SIGHUP)\n"); break;
    //  case 2:printf("SIGINT)\n"); break;
    //  case 3:printf("SIGQUIT)\n"); break;
    //  case 15: printf("SIGTERM)\n"); break;
    //  default:
    //	  printf("?)\n");
    //}
    //fflush(stdout);
    
    exit(signal + 10);
}

void mainLoop() {
    while (true) {
	switch (p_mode) {
	  case pm_menu: menuLoop(); break;
	  case pm_game: gameLoop(); break;
	  case pm_quit:;
	      //??
	}
    }
}

#ifdef __cplusplus
extern "C"
#endif
int main(int argc, char * argv[]) {  

    /* Initilazing glut */
    glutInit(&argc, argv);
    //glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH) ;

    /* Initializing SDL */
    
    //if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
    if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER) < 0) {
        fprintf(stderr, "Couldn't initialize SDL: %s\n", SDL_GetError());
        quit(1);
    }

    //initNetwork();
    
    atexit(SDL_Quit);
    
    SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5);
    SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 5);
    SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5);
    SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16);
    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1);
        
    //     glutInitWindowSize(50,50);
    //     glutInitWindowPosition(50,50);
    //     printf("window %i\n", glutCreateWindow("test"));
    //     emptyCallbacks();
    //     glutVisibilityFunc(ttt);
    //     glutHideWindow();

    
    // catch some signals
    struct sigaction quitAction;
    memset(&quitAction, 0, sizeof(quitAction));
    quitAction.sa_handler = &quitHandler;
    sigemptyset(&quitAction.sa_mask);
    sigaction(SIGQUIT, &quitAction, NULL);
    sigaction(SIGINT, &quitAction, NULL);
    sigaction(SIGHUP, &quitAction, NULL);
    sigaction(SIGTERM, &quitAction, NULL);
    sigaction(SIGQUIT, &quitAction, NULL);
    sigaction(SIGABRT, &quitAction, NULL);
    
    atexit(&quit);

    // init board prototypes
    nBoardsTypes = 3;
    nBoards = new int[nBoardsTypes];
    boardsDefaultSize = new int[nBoardsTypes];
    boardPrototype = new Board**[nBoardsTypes];
    
    // Cube
    nBoards[0] = 5;
    boardsDefaultSize[0] = 2;
    boardPrototype[0] = new Board*[nBoards[0]];
    boardPrototype[0][0] = new Cube(TINY_BOARD);
    boardPrototype[0][1] = new Cube(SMALL_BOARD);
    boardPrototype[0][2] = new Cube(MEDIUM_BOARD);
    boardPrototype[0][3] = new Cube(LARGE_BOARD);
    boardPrototype[0][4] = new Cube(HUGE_BOARD);
    
    // Rooms2
    nBoards[1] = 3;
    boardsDefaultSize[1] = 1;
    boardPrototype[1] = new Board*[nBoards[1]];
    boardPrototype[1][0] = new Rooms2(TINY_BOARD);
    boardPrototype[1][1] = new Rooms2(SMALL_BOARD);
    boardPrototype[1][2] = new Rooms2(MEDIUM_BOARD);

    // Classic2d
    nBoards[2] = 5;
    boardsDefaultSize[2] = 2;
    boardPrototype[2] = new Board*[nBoards[2]];
    boardPrototype[2][0] = new Classic2d(TINY_BOARD);
    boardPrototype[2][1] = new Classic2d(SMALL_BOARD);
    boardPrototype[2][2] = new Classic2d(MEDIUM_BOARD);
    boardPrototype[2][3] = new Classic2d(LARGE_BOARD);
    boardPrototype[2][4] = new Classic2d(HUGE_BOARD);
    

    
    //readDefaultKeys();
    getPreferences(false);

    resetNames();
    
    /*
    if (!checkGameResolution()) {
	prefs.resolution = DEFAULT_RESOLUTION;
	checkGameResolution();
    }
    leaveFullScreen();
    */

    
    //prefs.windowSize[X] = prefs.realWindowSize[X];
    //prefs.windowSize[Y] = prefs.realWindowSize[Y];

    /*
    // SDL_FULLSCREEN, SDL_RESIZABLE
    if( SDL_SetVideoMode(prefs.windowSize[X], prefs.windowSize[Y], video->vfmt->BitsPerPixel, SDL_OPENGL | SDL_RESIZABLE) == 0 ) {
	fprintf(stderr, 
                "Couldn't set video mode: %s\n", SDL_GetError());
        exit(1);
    }
    reshape(prefs.windowSize[X], prefs.windowSize[Y]);
    */


    /* build resolutions */
    SDL_Rect **modes;
    int i;
    modes = SDL_ListModes(NULL, SDL_OPENGL | SDL_FULLSCREEN);
    if(modes == (SDL_Rect **)0){
	nResolutions = 0;
	resolutions = NULL;
    } else if(modes == (SDL_Rect **)-1){
	nResolutions = 4;
	resolutions = new int*[nResolutions];
	for (i = 0; i < nResolutions; i++)
	    resolutions[i] = new int[2];
	resolutions[0][X] = 800 ; resolutions[0][Y] = 600;
	resolutions[1][X] = 1024; resolutions[1][Y] = 768;
	resolutions[2][X] = 1280; resolutions[2][Y] = 1024;
	resolutions[3][X] = 1600; resolutions[3][Y] = 1200;
    }
    else{
	for (nResolutions = 0; modes[nResolutions]; ++nResolutions);
	resolutions = new int*[nResolutions];
	for (i = 0; i < nResolutions; i++) {
	    resolutions[i] = new int[2];
	    resolutions[i][X] = modes[nResolutions - i - 1]->w;
	    resolutions[i][Y] = modes[nResolutions - i - 1]->h;
	    if (resolutions[i][X] == 800)
		defaultResolution = i;
	}
    }
    
    if (nResolutions == 0) {
	prefs.fullscreen = false;
    } else {
	for (i = 0; i < nResolutions; i++) {
	    if (prefs.fullScreenResolution[X] == resolutions[i][X] &&
		prefs.fullScreenResolution[Y] == resolutions[i][Y]) {
		prefs.resolution = i;
		break;
	    }
	}
	if (i == nResolutions) {
	    prefs.resolution = defaultResolution;
	    prefs.fullScreenResolution[X] = resolutions[defaultResolution][X];
	    prefs.fullScreenResolution[Y] = resolutions[defaultResolution][Y];
	}
    }

    /*
    for (int i = 0; i < nResolutions; i++) {
	printf("%dx%d\n", resolutions[i][X], resolutions[i][Y]);
    }
    */
    
    reshape();
    
    //glutInitWindowSize(prefs.windowSize[X], prefs.windowSize[Y]);
    //glutInitWindowPosition(INIT_WINDOW_POS[X], INIT_WINDOW_POS[Y]);
    //currentWindow = glutCreateWindow("Snake 3D");

//     glClearColor(COLORS[prefs.backgroundColor][RED], COLORS[prefs.backgroundColor][GREEN], COLORS[prefs.backgroundColor][BLUE], 0.0) ;
//     glEnable(GL_DEPTH_TEST);
//     lighting(true);
//     glEnable(GL_LIGHT0);
//     glEnable(GL_NORMALIZE);
//     glShadeModel(GL_SMOOTH);
//     glLightfv(GL_LIGHT0, GL_AMBIENT, LIGHT_AMBIENT);
//     glLightfv(GL_LIGHT0, GL_DIFFUSE, LIGHT_DIFFUSE);
//     glLightfv(GL_LIGHT0, GL_SPECULAR, LIGHT_SPECULAR);
//     if (cylinder) gluDeleteQuadric(cylinder);
//     cylinder = gluNewQuadric();
//     gluQuadricNormals(cylinder, GLU_SMOOTH);

    
    /*
    glutInitWindowSize(prefs.windowSize[X], prefs.windowSize[Y]);
    glutInitWindowPosition(INIT_WINDOW_POS[X], INIT_WINDOW_POS[Y]);
    currentWindow = glutCreateWindow("Snake 3D");
    glutSetKeyRepeat(GLUT_KEY_REPEAT_OFF);

    / * Initilazing GL * /
    glClearColor(COLORS[prefs.backgroundColor][RED], 
		 COLORS[prefs.backgroundColor][GREEN], 
		 COLORS[prefs.backgroundColor][BLUE], 0.0);
    //glMatrixMode(GL_MODELVIEW);
    //glLoadIdentity();
    //glGetFloatv(GL_MODELVIEW_MATRIX, prefs.boardRotationMatrix);
    glEnable(GL_DEPTH_TEST);
    lighting(true);
    glEnable(GL_LIGHT0);
    glEnable(GL_NORMALIZE);
    glShadeModel(GL_SMOOTH);
    glLightfv(GL_LIGHT0, GL_AMBIENT, LIGHT_AMBIENT);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, LIGHT_DIFFUSE);
    glLightfv(GL_LIGHT0, GL_SPECULAR, LIGHT_SPECULAR);
    */

    /* other initializations */

    /* bends data */
    for (int i = 0; i < 9; i++) {
        JOINT_BENDS[i][X] = (0.5 + JOINT_EDGES[i][X]) * cos(M_PI / 6) - 0.5;
        JOINT_BENDS[i][Y] = (0.5 + JOINT_EDGES[i][X]) * sin(M_PI / 6) - 0.5;
        JOINT_BENDS[i][Z] = JOINT_EDGES[i][Z];

        /*
	  BEND_NORMALS[i][X] = JOINT_NORMALS[i][X] * cos(M_PI / 6);
	  BEND_NORMALS[i][Y] = JOINT_NORMALS[i][Y] * sin(M_PI / 6);
	  BEND_NORMALS[i][Z] = JOINT_NORMALS[i][Z];
	*/
	BEND_NORMALS[i][X] = (0.5 + JOINT_NORMALS[i][X]) * cos(M_PI / 6) - 0.5;
	BEND_NORMALS[i][Y] = (0.5 + JOINT_NORMALS[i][X]) * sin(M_PI / 6) - 0.5;
	BEND_NORMALS[i][Z] = JOINT_NORMALS[i][Z];
    }
    
    /* randomizer */
    timeval tv;
    gettimeofday(&tv, NULL);
    srandom(tv.tv_sec + tv.tv_usec);

    /* cylinder */
    //cylinder = gluNewQuadric();
    //gluQuadricNormals(cylinder, GLU_SMOOTH);

    /* Filling callback functions */
    //gameCallbacks();
    //menuCallbacks(true);
    p_mode = pm_menu;

    /* Entering glut main loop */
    //glutMainLoop() ;

    mainLoop();
    return 0 ;
}
